'===========================================================================
' Copyright(c)Microsoft Corporation. All rights reserved.
' File: shlobj.h
'===========================================================================
$ifndef __shlobj_inc__
$define __shlobj_inc__

$ifndef __shellapi_inc__
$include "shellapi.inc"
$endif

$ifndef __ole2_inc__
$include "ole2.inc"
$endif

$ifndef __prsht_inc__
$include "prsht.inc"
$endif

$ifndef __commctrl_inc__
$include "commctrl.inc"	' for LPTBBUTTON
$endif

$ifndef __shlguid_inc__
$include "shlguid.inc"
$endif

$ifndef __shtypes_inc__
$include "shtypes.inc"
$endif

$ifndef __shobjidl_inc__
$include "shobjidl.inc"
$endif
/*===========================================================================
' Task allocator API
'	All the shell extensions MUST use the task allocator(see OLE 2.0
' programming guild for its definition)when they allocate or free
' memory objects(mostly ITEMIDLIST)that are returned across any
' shell interfaces. There are two ways to access the task allocator
' from a shell extension depending on whether or not it is linked with
' OLE32.DLL or not(purely for efficiency).
'(1)A shell extension which calls any OLE API(i.e., linked with
'	OLE32.DLL)should call OLE's task allocator(by retrieving
'	the task allocator by calling CoGetMalloc API).
'(2)A shell extension which does not call any OLE API(i.e., not linked
'	with OLE32.DLL)should call the shell task allocator API(defined
'	below), so that the shell can quickly loads it when OLE32.DLL is not
'	loaded by any application at that point.
' Notes:
'	next version of Windowso release, SHGetMalloc will be replaced by
' the following macro.
' $define SHGetMalloc(ppmem)	CoGetMalloc(MEMCTX_TASK, ppmem)
'===========================================================================*/

declare import, SHGetMalloc(pointer ppMalloc)

declare import, SHAlloc(SIZE_T cb),pointer
declare import, SHFree(pointer pv)

/*===========================================================================
' IContextMenu interface
' [OverView]
'	The shell uses the IContextMenu interface following three cases.
' case-1: The shell is loading context menu extensions.
'	When the user clicks the right mouse button on an item within the shell's
'	name space(i.g., file, directory, server, work-group, etc.), it creates
'	the default context menu for its type, then loads context menu extensions
'	that are registered for that type(and its base type)so that they can
'	add extra menu items. Those context menu extensions are registered at
'	HKCR\{ProgID}\shellex\ContextMenuHandlers.
' case-2: The shell is retrieving a context menu of sub-folders extended
'	name-space.
'	When the explorer's name space is extended by name space extensions,
'	the shell calls their IShellFolder::GetUIObjectOf to get the IContextMenu
'	objects when it creates context menus for folders under those extended
'	name spaces.
' case-3: The shell is loading non-default drag and drop handler for directories.
'	When the user performed a non-default drag and drop onto one of file
'	system folders(i.e., directories), it loads shell extensions that are
'	registered at HKCR\{ProgID}\DragDropHandlers.
' [Member functions]
' IContextMenu::QueryContextMenu
'	This member function may insert one or more menuitems to the specified
'	menu(hmenu)at the specified location(indexMenu which is never be -1).
'	The IDs of those menuitem must be the specified range(idCmdFirst and
'	idCmdLast). It returns the maximum menuitem ID offset(ushort)the
'	'code' field(low word)of the scode.
'	The uFlags specify the context. It may have one or more of following
'	flags.
'	CMF_DEFAULTONLY: This flag is passed if the user is invoking the default
'	action(typically by double-clicking, case 1 and 2 only). Context menu
'	extensions(case 1)should not add any menu items, and returns NOERROR.
'	CMF_VERBSONLY: The explorer passes this flag if it is constructing
'	a context menu for a short-cut object(case 1 and case 2 only). If this
'	flag is passed, it should not add any menu-items that is not appropriate
'	from a short-cut.
'	A good example is the "Delete" menuitem, which confuses the user
'	because it is not clear whether it deletes the link source item or the
'	link itself.
'	CMF_EXPLORER: The explorer passes this flag if it has the left-side pane
'	(case 1 and 2 only). Context menu extensions should ignore this flag.
'	High word(16-bit)are reserved for context specific communications
'	and the rest of flags(13-bit)are reserved by the system.
' IContextMenu::InvokeCommand
'	This member is called when the user has selected one of menuitems that
'	are inserted by previous QueryContextMenu member. this case, the
'	LOWORD(lpici->lpVerb)contains the menuitem ID offset(menuitem ID -
'	idCmdFirst).
'	This member function may also be called programmatically. such a case,
'	lpici->lpVerb specifies the canonical name of the command to be invoked,
'	which is typically retrieved by GetCommandString member previously.
'	Parameters lpci:
'	cbSize -- Specifies the size of this structure(sizeof(*lpci))
'	hwnd	-- Specifies the owner window for any message/dialog box.
'	fMask	-- Specifies whether or not dwHotkey/hIcon paramter is valid.
'	lpVerb -- Specifies the command to be invoked.
'	lpParameters -- Parameters(optional)
'	lpDirectory	-- Working directory(optional)
'	nShow -- Specifies the flag to be passed to ShowWindow(SW_*).
'	dwHotKey -- Hot key to be assigned to the app after invoked(optional).
'	hIcon -- Specifies the icon(optional).
'	hMonitor -- Specifies the default monitor(optional).
' IContextMenu::GetCommandString
'	This member function is called by the explorer either to get the
'	canonical(language independent)command name(uFlags == GCS_VERB)or
'	the help text((uFlags & GCS_HELPTEXT)!= 0)for the specified command.
'	The retrieved canonical string may be passed to its InvokeCommand
'	member function to invoke a command programmatically. The explorer
'	displays the help texts its status bar; therefore, the length of
'	the help text should be reasonably short(<40 characters).
'	Parameters:
'	idCmd -- Specifies menuitem ID offset(from idCmdFirst)
'	uFlags -- Either GCS_VERB or GCS_HELPTEXT
'	pwReserved -- Reserved(must pass NULL when calling, must ignore when called)
'	pszName -- Specifies the string buffer.
'	cchMax -- Specifies the size of the string buffer.
'===========================================================================*/

' QueryContextMenu uFlags
$define CMF_NORMAL	0x00000000
$define CMF_DEFAULTONLY	0x00000001
$define CMF_VERBSONLY	0x00000002
$define CMF_EXPLORE	0x00000004
$define CMF_NOVERBS	0x00000008
$define CMF_CANRENAME	0x00000010
$define CMF_NODEFAULT	0x00000020
$define CMF_INCLUDESTATIC	0x00000040
$define CMF_EXTENDEDVERBS	0x00000100	' rarely used verbs
$define CMF_RESERVED	0xffff0000	' View specific

' GetCommandString uFlags
$define GCS_VERBA	0x00000000	' canonical verb
$define GCS_HELPTEXTA	0x00000001	' help text(for status bar)
$define GCS_VALIDATEA	0x00000002	' validate command exists
$define GCS_VERBW	0x00000004	' canonical verb(unicode)
$define GCS_HELPTEXTW	0x00000005	' help text(unicode version)
$define GCS_VALIDATEW	0x00000006	' validate command exists(unicode)
$define GCS_UNICODE	0x00000004	' for bit testing - Unicode string

$define CMDSTR_NEWFOLDERA	"NewFolder"
$define CMDSTR_VIEWLISTA	"ViewList"
$define CMDSTR_VIEWDETAILSA "ViewDetails"
$define CMDSTR_NEWFOLDERW	L"NewFolder"
$define CMDSTR_VIEWLISTW	L"ViewList"
$define CMDSTR_VIEWDETAILSW L"ViewDetails"

$ifdef UNICODE
$define CMDSTR_NEWFOLDER	L"NewFolder"
$define CMDSTR_VIEWLIST	L"ViewList"
$define CMDSTR_VIEWDETAILS	L"ViewDetails"
$else
$define CMDSTR_NEWFOLDER	"NewFolder"
$define CMDSTR_VIEWLIST	"ViewList"
$define CMDSTR_VIEWDETAILS	"ViewDetails"
$endif

$define CMIC_MASK_HOTKEY	0x00000020
$define CMIC_MASK_ICON	0x00000010
$define CMIC_MASK_FLAG_NO_UI	0x00000400
$define CMIC_MASK_UNICODE	0x00004000
$define CMIC_MASK_NO_CONSOLE	0x00008000
'$define CMIC_MASK_HASLINKNAME	SEE_MASK_HASLINKNAME
'$define CMIC_MASK_FLAG_SEP_VDM	SEE_MASK_FLAG_SEPVDM
'$define CMIC_MASK_HASTITLE	SEE_MASK_HASTITLE
$define CMIC_MASK_ASYNCOK	0x00100000
$define CMIC_MASK_NOZONECHECKS	0x00800000

$define CMIC_MASK_SHIFT_DOWN	0x10000000
$define CMIC_MASK_CONTROL_DOWN	0x40000000
$define CMIC_MASK_FLAG_LOG_USAGE 0x04000000

$define CMIC_MASK_PTINVOKE	0x20000000

'$include <pshpack8.h>

'NOTE: When SEE_MASK_HMONITOR is set, hIcon is treated as hMonitor
type CMINVOKECOMMANDINFO
	DWORD cbSize	' sizeof(CMINVOKECOMMANDINFO)
	DWORD fMask	' any combination of CMIC_MASK_*
	HWND hwnd	' might be NULL(indicating no owner window)
	LPCSTR lpVerb	' either a string or MAKEINTRESOURCE(idOffset)
	LPCSTR lpParameters ' might be NULL(indicating no parameter)
	LPCSTR lpDirectory	' might be NULL(indicating no specific directory)
	int nShow	' one of SW_ values for ShowWindow()API

	DWORD dwHotKey
	HANDLE hIcon
endtype

type CMINVOKECOMMANDINFOEX
	DWORD cbSize	' must be sizeof(CMINVOKECOMMANDINFOEX)
	DWORD fMask	' any combination of CMIC_MASK_*
	HWND hwnd	' might be NULL(indicating no owner window)
	LPCSTR lpVerb	' either a string or MAKEINTRESOURCE(idOffset)
	LPCSTR lpParameters ' might be NULL(indicating no parameter)
	LPCSTR lpDirectory	' might be NULL(indicating no specific directory)
	int nShow	' one of SW_ values for ShowWindow()API

	DWORD dwHotKey

	HANDLE hIcon
	LPCSTR lpTitle	' For CreateProcess-StartupInfo.lpTitle
	LPCWSTR lpVerbW	' Unicode verb(for those who can use it)
	LPCWSTR lpParametersW	' Unicode parameters(for those who can use it)
	LPCWSTR lpDirectoryW	' Unicode directory(for those who can use it)
	LPCWSTR lpTitleW	' Unicode title(for those who can use it)
	POINT	ptInvoke	' Point where it's invoked
endtype

'$include <poppack.h>	/* Return to byte packing */

interface IContextMenu ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IContextMenu
	stdmethod QueryContextMenu(HMENU hmenu,UINT indexMenu,UINT idCmdFirst,UINT idCmdLast,UINT uFlags),HRESULT
	stdmethod InvokeCommand(pointer lpici),HRESULT
	stdmethod GetCommandString(UINT_PTR idCmd,UINT uType,pointer	pwReserved,LPSTR pszName,UINT cchMax),HRESULT
endinterface

/* IContextMenu2(IContextMenu with one new member)
' IContextMenu2::HandleMenuMsg
'	This function is called, if the client of IContextMenu is aware of
' IContextMenu2 interface and receives one of following messages while
' it is calling TrackPopupMenu(in the window proc of hwnd):
'	WM_INITPOPUP, WM_DRAWITEM and WM_MEASUREITEM
'	The callee may handle these messages to draw owner draw menuitems.

'	IContextMenu2*/

interface IContextMenu2 ' IContextMenu
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IContextMenu
	stdmethod QueryContextMenu(HMENU hmenu,UINT indexMenu,UINT idCmdFirst,UINT idCmdLast,UINT uFlags),HRESULT
	stdmethod InvokeCommand(pointer lpici),HRESULT
	stdmethod GetCommandString(UINT_PTR idCmd,UINT uType,pointer	pwReserved,LPSTR pszName,UINT cchMax),HRESULT
'IContextMenu2
	stdmethod HandleMenuMsg(UINT uMsg, WPARAM wParam, LPARAM lParam),HRESULT
endinterface

/* IContextMenu3(IContextMenu with one new member)
' IContextMenu3::HandleMenuMsg2
'	This function is called, if the client of IContextMenu is aware of
' IContextMenu3 interface and receives a menu message while
' it is calling TrackPopupMenu(in the window proc of hwnd):

'	IContextMenu3*/

interface IContextMenu3 ' IContextMenu2
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IContextMenu
	stdmethod QueryContextMenu(HMENU hmenu,UINT indexMenu,UINT idCmdFirst,UINT idCmdLast,UINT uFlags),HRESULT
	stdmethod InvokeCommand(pointer lpici),HRESULT
	stdmethod GetCommandString(UINT_PTR idCmd,UINT uType,pointer	pwReserved,LPSTR pszName,UINT cchMax),HRESULT
'IContextMenu2
	stdmethod HandleMenuMsg(UINT uMsg, WPARAM wParam, LPARAM lParam),HRESULT
'IContextMenu3
	stdmethod HandleMenuMsg2(UINT uMsg, WPARAM wParam, LPARAM lParam, pointer plResult),HRESULT
endinterface

'	IPersistFolder3

$define CSIDL_FLAG_PFTI_TRACKTARGET 0x4000

/* DESCRIPTION: PERSIST_FOLDER_TARGET_INFO
'	This stucture is used for Folder Shortcuts which allow the shell to
' have a file system folder act like another area the name space.
' One of pidlTargetFolder, szTargetParsingName, or csidl needs to
' specify the destination name space.
' pidlTargetFolder: This is a full pidl to the target folder.	Can be NULL the IPersistFolder3::InitializeEx()
'	call but not the GetFolderTargetInfo()return structure.
' szTargetParsingName: Empty string if not specified. Ortherwise, it is the parsible name
'	to the target.	This name can be parsed by IShellFolder::
'	ParsedName()from the desktop.
' szNetworkProvider: Can be an empty string.	If not empty, it specifies the type of network
'	provider that will be used when binding to the target.	This is used
'	for performance optimizations for the WNet APIs.
' dwAttributes: -1 if not known.	These are the SFGAO_ flags for IShellFolder::GetAttributesOf()
' csidl: This is -1 if it's not used.	This can be used instead of pidlTargetFolder or
'	szTargetParsingName to indicate the TargetFolder.	See the list of CSIDL_ folders
'	below.	CSIDL_FLAG_PFTI_TRACKTARGET means that the IShellFolder's target folder
'	should change if the user changes the target of the underlying CSIDL value.
'	You can also pass CSIDL_FLAG_CREATE to indicate that the target folder
'	should be created if it does not exist.	No other CSIDL_FLAG_* values are supported.

'$include <pshpack8.h>*/

type PERSIST_FOLDER_TARGET_INFO
	pointer pidlTargetFolder	' pidl for the folder we want to intiailize
	iwstring	szTargetParsingName[MAX_PATH]	' optional parsing name for the target
	iwstring	szNetworkProvider[MAX_PATH]	' optional network provider
	DWORD	dwAttributes	' optional FILE_ATTRIBUTES_ flags(-1 if not used)
	int	csidl	' optional folder index(SHGetFolderPath())-1 if not used
endtype

/*$include <poppack.h>	 Return to byte packing

' DESCRIPTION: IPersistFolder3
'	This interface is implemented by an IShellFolder object that wants non-default
' handling of Folder Shortcuts.	general, shell name space extensions should use
' pidlRoot(the alias pidl)as their location the name space and pass it to public
' APIs, such as ShellExecute().	The one exception is that pidlTarget should be used
' when sending ChangeNotifies or registering to listen for change notifies
'(see SFVM_GETNOTIFY).
' InitializeEx: This method initializes an IShellFolder and specifies where
'	it is rooted the name space.
'	pbc: May be NULL.
'	pidlRoot: This is the same parameter as IPersistFolder::Initialize(). Caller allocates
'	and frees this parameter.
'	ppfti: May be NULL, which case this is the same as a call to IPersistFolder::Initialize().
'	Otherwise this is a Folder Shortcut and this structure specifies the target
'	folder and it's attributes.
' GetFolderTargetInfo: This is used by the caller to find information about
'	the folder shortcut.	This structure may not be initialized by the caller,
'	so the callee needs to initialize every member.	The callee allocates
'	pidlTargetFolder and the caller will free it.	Filling pidlTargetFolder is
'	ALWAYS required.*/
interface IPersistFolder3 ' IPersistFolder2
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
	stdmethod GetClassID(pointer pClassID),HRESULT
'IPersistFolder
	stdmethod Initialize(pointer pidl),HRESULT
'IPersistFolder2
	stdmethod GetCurFolder(/*out*/pointer ppidl),HRESULT
'IPersistFolder3
	stdmethod InitializeEx(comref pbc,pointer pidlRoot, pointer ppfti),HRESULT
	stdmethod GetFolderTargetInfo(pointer ppfti),HRESULT
endinterface

/*===========================================================================
' IExtractIcon interface
'	This interface is used two different places the shell.
' Case-1: Icons of sub-folders for the scope-pane of the explorer.
'	It is used by the explorer to get the "icon location" of
' sub-folders from each shell folders. When the user expands a folder
' the scope pane of the explorer, the explorer does following:
'	(1)binds to the folder(gets IShellFolder),
'	(2)enumerates its sub-folders by calling its EnumObjects member,
'	(3)calls its GetUIObjectOf member to get IExtractIcon interface
'	for each sub-folders.
'	this case, the explorer uses only IExtractIcon::GetIconLocation
' member to get the location of the appropriate icon. An icon location
' always consists of a file name(typically DLL or EXE)and either an icon
' resource or an icon index.
' Case-2: Extracting an icon image from a file
'	It is used by the shell when it extracts an icon image
' from a file. When the shell is extracting an icon from a file,
' it does following:
'	(1)creates the icon extraction handler object(by getting its CLSID
'	under the {ProgID}\shell\ExtractIconHanler key and calling
'	CoCreateInstance requesting for IExtractIcon interface).
'	(2)Calls IExtractIcon::GetIconLocation.
'	(3)Then, calls IExtractIcon::ExtractIcon with the location/index pair.
'	(4)If(3)returns NOERROR, it uses the returned icon.
'	(5)Otherwise, it recursively calls this logic with new location
'	assuming that the location string contains a fully qualified path name.
'	From extension programmer's point of view, there are only two cases
' where they provide implementations of IExtractIcon:
'	Case-1)providing explorer extensions(i.e., IShellFolder).
'	Case-2)providing per-instance icons for some types of files.
' Because Case-1 is described above, we'll explain only Case-2 here.
' When the shell is about display an icon for a file, it does following:
'	(1)Finds its ProgID and ClassID.
'	(2)If the file has a ClassID, it gets the icon location string from the
'	"DefaultIcon" key under it. The string indicates either per-class
'	icon(e.g., "FOOBAR.DLL,2")or per-instance icon(e.g., "%1,1").
'	(3)If a per-instance icon is specified, the shell creates an icon
'	extraction handler object for it, and extracts the icon from it
'	(which is described above).
'	It is important to note that the shell calls IExtractIcon::GetIconLocation
' first, then calls IExtractIcon::Extract. Most application programs
' that support per-instance icons will probably store an icon location
'(DLL/EXE name and index/id)rather than an icon image each file.
' those cases, a programmer needs to implement only the GetIconLocation
' member and it Extract member simply returns S_FALSE. They need to
' implement Extract member only if they decided to store the icon images
' within files themselved or some other database(which is very rare).
' [Member functions]
' IExtractIcon::GetIconLocation
'	This function returns an icon location.
'	Parameters:
'	uFlags	[in]	-- Specifies if it is opened or not(GIL_OPENICON or 0)
'	szIconFile [out] -- Specifies the string buffer buffer for a location name.
'	cchMax	[in]	-- Specifies the size of szIconFile(almost always MAX_PATH)
'	piIndex	[out] -- Sepcifies the address of UINT for the index.
'	pwFlags	[out] -- Returns GIL_* flags
'	Returns:
'	NOERROR, if it returns a valid location; S_FALSE, if the shell use a
'	default icon.
'	Notes: The location may or may not be a path to a file. The caller can
'	not assume anything unless the subsequent Extract member call returns
'	S_FALSE.
'	if the returned location is not a path to a file, GIL_NOTFILENAME should
'	be set the returned flags.
' IExtractIcon::Extract
'	This function extracts an icon image from a specified file.
'	Parameters:
'	pszFile [in] -- Specifies the icon location(typically a path to a file).
'	nIconIndex [in] -- Specifies the icon index.
'	phiconLarge [out] -- Specifies the HICON variable for large icon.
'	phiconSmall [out] -- Specifies the HICON variable for small icon.
'	nIconSize [in] -- Specifies the size icon required(size of large icon)
'	LOWORD is the requested large icon size
'	HIWORD is the requested small icon size
'	Returns:
'	NOERROR, if it extracted the from the file.
'	S_FALSE, if the caller should extract from the file specified the
'	location.
'===========================================================================

' GetIconLocation()input flags*/

$define GIL_OPENICON	0x0001	' allows containers to specify an "open" look
$define GIL_FORSHELL	0x0002	' icon is to be displayed a ShellFolder
$define GIL_ASYNC	0x0020	' this is an async extract, return E_PENDING
$define GIL_DEFAULTICON	0x0040	' get the default icon location if the final one takes too long to get
$define GIL_FORSHORTCUT	0x0080	' the icon is for a shortcut to the object

' GetIconLocation()return flags

$define GIL_SIMULATEDOC	0x0001	' simulate this document icon for this
$define GIL_PERINSTANCE	0x0002	' icons from this class are per instance(each file has its own)
$define GIL_PERCLASS	0x0004	' icons from this class per class(shared for all files of this type)
$define GIL_NOTFILENAME	0x0008	' location is not a filename, must call ::ExtractIcon
$define GIL_DONTCACHE	0x0010	' this icon should not be cached

interface IExtractIconA ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IExtractIconA
	stdmethod GetIconLocation(UINT uFlags,LPSTR szIconFile,UINT cchMax,pointer piIndex,pointer pwFlags),HRESULT
	stdmethod Extract(LPCSTR pszFile,UINT nIconIndex,pointer phiconLarge,pointer phiconSmall,UINT nIconSize),HRESULT
endinterface

interface IExtractIconW : ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IExtractIconW
	stdmethod GetIconLocation(UINT uFlags,LPWSTR szIconFile,UINT cchMax,pointer piIndex,pointer pwFlags),HRESULT
	stdmethod Extract(LPCWSTR pszFile,UINT nIconIndex,pointer phiconLarge,pointer phiconSmall,UINT nIconSize),HRESULT
endinterface

$ifdef UNICODE
interface IExtractIcon ' IExtractIconW
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IExtractIconW
	stdmethod GetIconLocation(UINT uFlags,LPWSTR szIconFile,UINT cchMax,pointer piIndex,pointer pwFlags),HRESULT
	stdmethod Extract(LPCWSTR pszFile,UINT nIconIndex,pointer phiconLarge,pointer phiconSmall,UINT nIconSize),HRESULT
endinterface
$else
interface IExtractIcon ' IExtractIconA
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IExtractIconA
	stdmethod GetIconLocation(UINT uFlags,LPSTR szIconFile,UINT cchMax,pointer piIndex,pointer pwFlags),HRESULT
	stdmethod Extract(LPCSTR pszFile,UINT nIconIndex,pointer phiconLarge,pointer phiconSmall,UINT nIconSize),HRESULT
endinterface
$endif

/*===========================================================================
' IShellIcon Interface
' used to get a icon index for a IShellFolder object.
' this interface can be implemented by a IShellFolder, as a quick way to
' return the icon for a object the folder.
' a instance of this interface is only created once for the folder, unlike
' IExtractIcon witch is created once for each object.
' if a ShellFolder does not implement this interface, the standard
' GetUIObject(....IExtractIcon)method will be used to get a icon
' for all objects.
' the following standard imagelist indexs can be returned:
'	0	document(blank page)(not associated)
'	1	document(with stuff on the page)
'	2	application(exe, com, bat)
'	3	folder(plain)
'	4	folder(open)
' IShellIcon:GetIconOf(pidl, flags, lpIconIndex)
'	pidl	object to get icon for.
'	flags	GIL_* input flags(GIL_OPEN, ...)
'	lpIconIndex	place to return icon index.
'	returns:
'	NOERROR, if lpIconIndex contains the correct system imagelist index.
'	S_FALSE, if unable to get icon for this object, go through
'	GetUIObject, IExtractIcon, methods.
'===========================================================================*/

interface IShellIcon ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellIcon
	stdmethod GetIconOf(pointer pidl, UINT flags,pointer lpIconIndex),HRESULT
endinterface

/*===========================================================================
' IShellIconOverlayIdentifier
' Used to identify a file as a member of the group of files that have this specific
' icon overlay
' Users can create new IconOverlayIdentifiers and place them the following registry
' location together with the Icon overlay image and their priority.
' HKEY_LOCAL_MACHINE "Software\\Microsoft\\Windows\\CurrentVersion\\ShellIconOverlayIdentifiers"
' The shell will enumerate through all IconOverlayIdentifiers at start, and prioritize
' them according to internal rules, case the internal rules don't apply, we use their
' input priority
' IShellIconOverlayIdentifier:IsMemberOf(pointer pwszPath, DWORD dwAttrib)
'	pwszPath	full path of the file
'	dwAttrib	attribute of this file
'	returns:
'	S_OK,	if the file is a member
'	S_FALSE, if the file is not a member
'	E_FAIL,	if the operation failed due to bad WIN32_FIND_DATA
' IShellIconOverlayIdentifier::GetOverlayInfo(LPWSTR pwszIconFile,pointer pIndex,pointer dwFlags),HRESULT
'	pszIconFile	the path of the icon file
'	pIndex	Depend on the flags, this could contain the IconIndex
'	dwFlags	defined below
' IShellIconOverlayIdentifier::GetPriority(pointer pIPriority),HRESULT
'	pIPriority	the priority of this Overlay Identifier
'===========================================================================*/

interface IShellIconOverlayIdentifier ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellIconOverlayIdentifier
	stdmethod IsMemberOf(pointer pwszPath, DWORD dwAttrib),HRESULT
	stdmethod GetOverlayInfo(pointer pwszIconFile, int cchMax,pointer pIndex,pointer pdwFlags),HRESULT
	stdmethod GetPriority(pointer pIPriority),HRESULT
endinterface

$define ISIOI_ICONFILE	0x00000001	' path is returned through pwszIconFile
$define ISIOI_ICONINDEX	0x00000002	' icon index pwszIconFile is returned through pIndex

/*===========================================================================
' IShellIconOverlayManager
' Used to return the icon overlay information including OverlayIndex, Image Index or Priority for an IShellFolder object.
' IShellIconOverlayManager:GetFileOverlayInfo(pointer pwszPath, DWORD dwAttrib,pointer pIndex, DWORD dwflags)
'	pwszPath	full path of the file
'	dwAttrib	attribute of this file
'	pIndex	pointer to the Icon Index the system image list
'	pOverlayIndex	pointer to the OverlayIndex the system image list
'	pPriority	pointer to the Priority of this overlay
' IShellIconOverlayManager:GetReservedOverlayInfo(pointer pwszPath, DWORD dwAttrib,pointer pIndex, DWORD dwflags, int iReservedID)
'	iReservedID	reserved icon overlay id
'	returns:
'	S_OK,	if the index of an Overlay is found
'	S_FALSE, if no Overlay exists for this file
'	E_FAIL, if lpfd is bad
' IShellIconOverlayManager:RefreshOverlayImages(DWORD dwFlags)
'	This will refresh the overlay cache, depends on the dwFlags passed in
'	It will reload the icons into the imagelist, when passed SIOM_ICONINDEX
' IShellIconOverlayManager::LoadNonloadedOverlayIdentifiers()
'	This method loads any registered overlay identifiers(handlers)that
'	are not currently loaded.
' IShellIconOverlayManager::OverlayIndexFromImageIndex(int iImage,pointer piIndex, BOOL fAdd)
'	iImage	existing shell image list index to look for
'	piIndex	returned overlay index
'	fAdd	Add image if not already present?
'===========================================================================*/

interface IShellIconOverlayManager ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellIconOverlayManager
	stdmethod GetFileOverlayInfo(pointer pwszPath, DWORD dwAttrib,pointer pIndex, DWORD dwflags),HRESULT
	stdmethod GetReservedOverlayInfo(pointer pwszPath, DWORD dwAttrib,pointer pIndex, DWORD dwflags, int iReservedID),HRESULT
	stdmethod RefreshOverlayImages(DWORD dwFlags),HRESULT
	stdmethod LoadNonloadedOverlayIdentifiers(),HRESULT
	stdmethod OverlayIndexFromImageIndex(int iImage,pointer piIndex, BOOL fAdd),HRESULT
endinterface
$define SIOM_OVERLAYINDEX	1
$define SIOM_ICONINDEX	2
' $define SIOM_PRIORITY	3
$define SIOM_RESERVED_SHARED	0
$define SIOM_RESERVED_LINK	1
$define SIOM_RESERVED_SLOWFILE	2

/*===========================================================================
' IShellIconOverlay
' Used to return the icon overlay index or its icon index for an IShellFolder object,
' this is always implemented with IShellFolder
' IShellIconOverlay:GetOverlayIndex(pointer pidl,pointer pdwIndex)
'	pidl	object to identify icon overlay for.
'	pdwIndex	the Overlay Index the system image list
' IShellIconOverlay:GetOverlayIconIndex(pointer pidl,pointer pdwIndex)
'	pdwIconIndex	the Overlay Icon index the system image list
' This method is only used for those who are interested seeing the real bits
' of the Overlay Icon
'	returns:
'	S_OK,	if the index of an Overlay is found
'	S_FALSE, if no Overlay exists for this file
'	E_FAIL, if pidl is bad
'===========================================================================*/

'	IShellIconOverlay

interface IShellIconOverlay ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellIconOverlay
	stdmethod GetOverlayIndex(pointer pidl,pointer pIndex),HRESULT
	stdmethod GetOverlayIconIndex(pointer pidl,pointer pIconIndex),HRESULT
endinterface

$define OI_ASYNC 0xFFFFEEEE

/*-------------------------------------------------------------------------
' SHGetIconOverlayIndex
' This function takes the path and icon/res id to the icon and convert it into
' an overlay index the system image list.
' Note: there are totally only 15 slots for system image overlays, some of which
' was reserved by the system, or taken by the overlayidentifiers, so it's possible
' that this function would fail and return -1
' To get the default overlays the system, such as the share hand, link shortcut
' and slow files, pass NULL as the file name, then the IDO_SHGIOI_* flags as the icon index
'-------------------------------------------------------------------------*/

$define IDO_SHGIOI_SHARE	0x0FFFFFFF
$define IDO_SHGIOI_LINK	0x0FFFFFFE
$define IDO_SHGIOI_SLOWFILE 0x0FFFFFFFD
declare import, SHGetIconOverlayIndexA(pointer pszIconPath, int iIconIndex),int
declare import, SHGetIconOverlayIndexW(pointer pszIconPath, int iIconIndex),int

$ifdef UNICODE
declare import, SHGetIconOverlayIndex alias SHGetIconOverlayIndexW(pointer pszIconPath, int iIconIndex),int
$else
declare import, SHGetIconOverlayIndex alias SHGetIconOverlayIndexA(pointer pszIconPath, int iIconIndex),int
$endif

' IShellLinkDataList::GetFlags()/SetFlags()
typedef SHELL_LINK_DATA_FLAGS int
$define SLDF_HAS_ID_LIST	0x00000001	' Shell link saved with ID list
$define SLDF_HAS_LINK_INFO	0x00000002	' Shell link saved with LinkInfo
$define SLDF_HAS_NAME	0x00000004
$define SLDF_HAS_RELPATH	0x00000008
$define SLDF_HAS_WORKINGDIR	0x00000010
$define SLDF_HAS_ARGS	0x00000020
$define SLDF_HAS_ICONLOCATION	0x00000040
$define SLDF_UNICODE	0x00000080	' the strings are unicode
$define SLDF_FORCE_NO_LINKINFO	0x00000100	' don't create a LINKINFO(make a dumb link)
$define SLDF_HAS_EXP_SZ	0x00000200	' the link contains expandable env strings
$define SLDF_RUN_IN_SEPARATE	0x00000400	' Run the 16-bit target exe a separate VDM/WOW
$define SLDF_HAS_LOGO3ID	0x00000800	' this link is a special Logo3/MSICD link
$define SLDF_HAS_DARWINID	0x00001000	' this link is a special Darwin link
$define SLDF_RUNAS_USER	0x00002000	' Run this link as a different user
$define SLDF_HAS_EXP_ICON_SZ	0x00004000	' contains expandable env string for icon path
$define SLDF_NO_PIDL_ALIAS	0x00008000	' don't ever resolve to a logical location
$define SLDF_FORCE_UNCNAME	0x00010000	' make GetPath()prefer the UNC name to the local name
$define SLDF_RUN_WITH_SHIMLAYER	0x00020000	' Launch the target of this link w/ shim layer active
$define SLDF_RESERVED	0x80000000	' Reserved-- so we can use the low word as an index value the future

type DATABLOCK_HEADER
	DWORD	cbSize	' Size of this extra data block
	DWORD	dwSignature	' signature of this extra data block
endtype
type DBLIST
	DWORD	cbSize	' Size of this extra data block
	DWORD	dwSignature	' signature of this extra data block
endtype

type NT_CONSOLE_PROPS
	DATABLOCK_HEADER dbh
	USHORT wFillAttribute	' fill attribute for console
	USHORT wPopupFillAttribute	' fill attribute for console popups
	COORD	dwScreenBufferSize	' screen buffer size for console
	COORD	dwWindowSize	' window size for console
	COORD	dwWindowOrigin	' window origin for console
	DWORD	nFont
	DWORD	nInputBufferSize
	COORD	dwFontSize
	UINT	uFontFamily
	UINT	uFontWeight
	iwstring FaceName[LF_FACESIZE]
	UINT	uCursorSize
	BOOL	bFullScreen
	BOOL	bQuickEdit
	BOOL	bInsertMode
	BOOL	bAutoPosition
	UINT	uHistoryBufferSize
	UINT	uNumberOfHistoryBuffers
	BOOL	bHistoryNoDup
	COLORREF ColorTable[ 16 ]
endtype
$define NT_CONSOLE_PROPS_SIG 0xA0000002

' This is a FE Console property
type NT_FE_CONSOLE_PROPS
	DATABLOCK_HEADER dbh
	UINT	uCodePage
endtype
$define NT_FE_CONSOLE_PROPS_SIG 0xA0000004

type EXP_DARWIN_LINK
	DATABLOCK_HEADER dbh
	istring	szDarwinID[MAX_PATH]	' ANSI darwin ID associated with link
	iwstring	szwDarwinID[MAX_PATH] ' UNICODE darwin ID associated with link
endtype
$define EXP_DARWIN_ID_SIG	0xA0000006
$define EXP_LOGO3_ID_SIG	0xA0000007


$define EXP_SPECIAL_FOLDER_SIG	0xA0000005	' LPEXP_SPECIAL_FOLDER

type EXP_SPECIAL_FOLDER
	DWORD	cbSize	' Size of this extra data block
	DWORD	dwSignature	' signature of this extra data block
	DWORD	idSpecialFolder	' special folder id this link points into
	DWORD	cbOffset	' ofset into pidl from SLDF_HAS_ID_LIST for child
endtype

type EXP_SZ_LINK
	DWORD	cbSize	' Size of this extra data block
	DWORD	dwSignature	' signature of this extra data block
	istring	szTarget[ MAX_PATH ]	' ANSI target name w/EXP_SZ it
	iwstring	swzTarget[ MAX_PATH ]	' UNICODE target name w/EXP_SZ it
endtype
$define EXP_SZ_LINK_SIG	0xA0000001	' LPEXP_SZ_LINK(target)
$define EXP_SZ_ICON_SIG	0xA0000007	' LPEXP_SZ_LINK(icon)

interface IShellLinkDataList ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellLinkDataList
	stdmethod AddDataBlock(pointer pDataBlock),HRESULT
	stdmethod CopyDataBlock(DWORD dwSig,pointer ppDataBlock),HRESULT
	stdmethod RemoveDataBlock(DWORD dwSig),HRESULT
	stdmethod GetFlags(pointer pdwFlags),HRESULT
	stdmethod SetFlags(DWORD dwFlags),HRESULT
endinterface

interface IResolveShellLink ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IResolveShellLink
	stdmethod ResolveShellLink(comref punk, HWND hwnd, DWORD fFlags),HRESULT
endinterface

$ifdef __shellapi_inc__
/* for LPSHELLEXECUTEINFO */

interface IShellExecuteHookA ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
	stdmethod Execute(pointer pei),HRESULT
endinterface
interface IShellExecuteHookW ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
	stdmethod Execute(pointer pei),HRESULT
endinterface

$ifdef UNICODE
interface IShellExecuteHook ' IShellExecuteHookW
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
	stdmethod Execute(pointer pei),HRESULT
endinterface
$else
interface IShellExecuteHook ' IShellExecuteHookA
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
	stdmethod Execute(pointer pei),HRESULT
endinterface
$endif

'===========================================================================
' IURLSearchHook Interface
'===========================================================================

'	IURLSearchHook

interface IURLSearchHook ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IURLSearchHook
	stdmethod Translate(pointer lpwszSearchURL, DWORD cchBufferSize),HRESULT
endinterface

interface ISearchContext ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'ISearchContext
	stdmethod GetSearchUrl(pointer pbstrSearchUrl),HRESULT
	stdmethod GetSearchText(pointer pbstrSearchText),HRESULT
	stdmethod GetSearchStyle(pointer pdwSearchStyle),HRESULT
endinterface

interface IURLSearchHook2 ' IURLSearchHook
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IURLSearchHook
	stdmethod Translate(pointer lpwszSearchURL, DWORD cchBufferSize),HRESULT
	stdmethod TranslateWithSearchContext(pointer lpwszSearchURL, DWORD cchBufferSize,comref pSearchContext),HRESULT
endinterface

'===========================================================================
' INewShortcutHook Interface
'===========================================================================

'	INewShortcutHookA

interface INewShortcutHookA ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'INewShortcutHookA
	stdmethod SetReferent(LPCSTR pcszReferent, HWND hwnd),HRESULT
	stdmethod GetReferent(LPSTR pszReferent, int cchReferent),HRESULT
	stdmethod SetFolder(LPCSTR pcszFolder),HRESULT
	stdmethod GetFolder(LPSTR pszFolder, int cchFolder),HRESULT
	stdmethod GetName(LPSTR pszName, int cchName),HRESULT
	stdmethod GetExtension(LPSTR pszExtension, int cchExtension),HRESULT
endinterface

interface INewShortcutHookW ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'INewShortcutHookW
	stdmethod SetReferent(LPCWSTR pcszReferent, HWND hwnd),HRESULT
	stdmethod GetReferent(LPWSTR pszReferent, int cchReferent),HRESULT
	stdmethod SetFolder(LPCWSTR pcszFolder),HRESULT
	stdmethod GetFolder(LPWSTR pszFolder, int cchFolder),HRESULT
	stdmethod GetName(LPWSTR pszName, int cchName),HRESULT
	stdmethod GetExtension(LPWSTR pszExtension, int cchExtension),HRESULT
endinterface

$ifdef UNICODE
interface INewShortcutHook ' INewShortcutHookW
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'INewShortcutHook
	stdmethod SetReferent(LPCWSTR pcszReferent, HWND hwnd),HRESULT
	stdmethod GetReferent(LPWSTR pszReferent, int cchReferent),HRESULT
	stdmethod SetFolder(LPCWSTR pcszFolder),HRESULT
	stdmethod GetFolder(LPWSTR pszFolder, int cchFolder),HRESULT
	stdmethod GetName(LPWSTR pszName, int cchName),HRESULT
	stdmethod GetExtension(LPWSTR pszExtension, int cchExtension),HRESULT
endinterface
$else
interface INewShortcutHook ' INewShortcutHookA
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'INewShortcutHook
	stdmethod SetReferent(LPCSTR pcszReferent, HWND hwnd),HRESULT
	stdmethod GetReferent(LPSTR pszReferent, int cchReferent),HRESULT
	stdmethod SetFolder(LPCSTR pcszFolder),HRESULT
	stdmethod GetFolder(LPSTR pszFolder, int cchFolder),HRESULT
	stdmethod GetName(LPSTR pszName, int cchName),HRESULT
	stdmethod GetExtension(LPSTR pszExtension, int cchExtension),HRESULT
endinterface
$endif

'===========================================================================
' ICopyHook Interface
'	The copy hook is called whenever file system directories are
'	copy/moved/deleted/renamed via the shell.	It is also called by the shell
'	on changes of status of printers.
'	Clients register their id under STRREG_SHEX_COPYHOOK for file system hooks
'	and STRREG_SHEx_PRNCOPYHOOK for printer hooks.
'	the CopyCallback is called prior to the action, so the hook has the chance
'	to allow, deny or cancel the operation by returning the falues:
'	IDYES	-	means allow the operation
'	IDNO	-	means disallow the operation on this file, but continue with
'	any other operations(eg. batch copy)
'	IDCANCEL - means disallow the current operation and cancel any pending
'	operations
'	arguments to the CopyCallback
'	hwnd - window to use for any UI
'	wFunc - what operation is being done
'	wFlags - and flags(FOF_*)set the initial call to the file operation
'	pszSrcFile - name of the source file
'	dwSrcAttribs - file attributes of the source file
'	pszDestFile - name of the destiation file(for move and renames)
'	dwDestAttribs - file attributes of the destination file
'===========================================================================

$ifndef FO_MOVE 'these need to be kept sync with the ones shellapi.inc

' file operations

$define FO_MOVE	0x0001
$define FO_COPY	0x0002
$define FO_DELETE	0x0003
$define FO_RENAME	0x0004

$define FOF_MULTIDESTFILES	0x0001
$define FOF_CONFIRMMOUSE	0x0002
$define FOF_SILENT	0x0004	' don't create progress/report
$define FOF_RENAMEONCOLLISION	0x0008
$define FOF_NOCONFIRMATION	0x0010	' Don't prompt the user.
$define FOF_WANTMAPPINGHANDLE	0x0020	' Fill SHFILEOPSTRUCT.hNameMappings
		' Must be freed using SHFreeNameMappings
$define FOF_ALLOWUNDO	0x0040
$define FOF_FILESONLY	0x0080	' on* .*, do only files
$define FOF_SIMPLEPROGRESS	0x0100	' means don't show names of files
$define FOF_NOCONFIRMMKDIR	0x0200	' don't confirm making any needed dirs
$define FOF_NOERRORUI	0x0400	' don't put up error UI
$define FOF_NOCOPYSECURITYATTRIBS	0x0800	' dont copy NT file Security Attributes
$define FOF_NORECURSION	0x1000	' don't recurse into directories.
$define FOF_NO_CONNECTED_ELEMENTS	0x2000	' don't operate on connected file elements.
$define FOF_WANTNUKEWARNING	0x4000	' during delete operation, warn if nuking instead of recycling(partially overrides FOF_NOCONFIRMATION)
$define FOF_NORECURSEREPARSE	0x8000	' treat reparse points as objects, not containers

typedef FILEOP_FLAGS USHORT

' printer operations

$define PO_DELETE	0x0013	' printer is being deleted
$define PO_RENAME	0x0014	' printer is being renamed
$define PO_PORTCHANGE	0x0020	' port this printer connected to is being changed
		' if this id is set, the strings received by
		' the copyhook are a doubly-null terminated
		' list of strings.	The first is the printer
		' name and the second is the printer port.
$define PO_REN_PORT	0x0034	' PO_RENAME and PO_PORTCHANGE at same time.

' no POF_ flags currently defined

$endif
' FO_MOVE

interface ICopyHookA ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'ICopyHookA
	stdmethod CopyCallback(HWND hwnd, UINT wFunc, UINT wFlags,pointer pszSrcFile, DWORD dwSrcAttribs,LPCSTR pszDestFile, DWORD dwDestAttribs),UINT
endinterface
interface ICopyHookW ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'ICopyHookW
	stdmethod CopyCallback(HWND hwnd, UINT wFunc, UINT wFlags,pointer pszSrcFile, DWORD dwSrcAttribs,LPCWSTR pszDestFile, DWORD dwDestAttribs),UINT
endinterface

$ifdef UNICODE
interface ICopyHook ' ICopyHookW
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'ICopyHook
	stdmethod CopyCallback(HWND hwnd, UINT wFunc, UINT wFlags,pointer pszSrcFile, DWORD dwSrcAttribs,LPCWSTR pszDestFile, DWORD dwDestAttribs),UINT
endinterface
$else
interface ICopyHook ' ICopyHookA
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'ICopyHookA
	stdmethod CopyCallback(HWND hwnd, UINT wFunc, UINT wFlags,pointer pszSrcFile, DWORD dwSrcAttribs,LPCSTR pszDestFile, DWORD dwDestAttribs),UINT
endinterface
$endif

'===========================================================================
' IFileViewerSite Interface
'===========================================================================

'	IFileViewerSite

interface IFileViewerSite ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IFileViewerSite
	stdmethod SetPinnedWindow(HWND hwnd),HRESULT
	stdmethod GetPinnedWindow(pointer phwnd),HRESULT
endinterface

'===========================================================================
' IFileViewer Interface
' Implemented a FileViewer component object.	Used to tell a
' FileViewer to PrintTo or to view, the latter happening though
' ShowInitialize and Show.	The filename is always given to the
' viewer through IPersistFile.
'===========================================================================

'$include <pshpack8.h>

type FVSHOWINFO
	' Stuff passed into viewer(in)
	DWORD cbSize	' Size of structure for future expansion...
	HWND hwndOwner	' who is the owner window.
	int iShow	' The show command

	' Passed and updated	(in/Out)
	DWORD dwFlags	' flags
	WINRECT _rect	' Where to create the window may have defaults
	IUnknown punkRel	' Relese this interface when window is visible

	' Stuff that might be returned from viewer(out)
	OLECHAR strNewFile[MAX_PATH]	' New File to view.

endtype

'$include <poppack.h>	/* Return to byte packing */

	' Define File View Show Info Flags.
$define FVSIF_RECT	0x00000001	' The rect variable has valid data.
$define FVSIF_PINNED	0x00000002	' We should Initialize pinned

$define FVSIF_NEWFAILED 0x08000000	' The new file passed back failed
		' to be viewed.

$define FVSIF_NEWFILE	0x80000000	' A new file to view has been returned
$define FVSIF_CANVIEWIT 0x40000000	' The viewer can view it.

'	IFileViewerA

interface IFileViewerA ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IFileViewerA
	stdmethod ShowInitialize(pointer lpfsi),HRESULT
	stdmethod Show(pointer pvsi),HRESULT
	stdmethod PrintTo(pointer pszDriver, BOOL fSuppressUI),HRESULT
endinterface
interface IFileViewerW ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IFileViewerW
	stdmethod ShowInitialize(pointer lpfsi),HRESULT
	stdmethod Show(pointer pvsi),HRESULT
	stdmethod PrintTo(pointer pszDriver, BOOL fSuppressUI),HRESULT
endinterface

$ifdef UNICODE
interface IFileViewer ' IFileViewerW
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IFileViewerW
	stdmethod ShowInitialize(pointer lpfsi),HRESULT
	stdmethod Show(pointer pvsi),HRESULT
	stdmethod PrintTo(pointer pszDriver, BOOL fSuppressUI),HRESULT
endinterface
$else
interface IFileViewer ' IFileViewerA
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IFileViewerA
	stdmethod ShowInitialize(pointer lpfsi),HRESULT
	stdmethod Show(pointer pvsi),HRESULT
	stdmethod PrintTo(pointer pszDriver, BOOL fSuppressUI),HRESULT
endinterface
$endif

'==========================================================================
' IShellBrowser/IShellView/IShellFolder interface
'	These three interfaces are used when the shell communicates with
' name space extensions. The shell(explorer)provides IShellBrowser
' interface, and extensions implements IShellFolder and IShellView
' interfaces.
'==========================================================================

'--------------------------------------------------------------------------
' Command/menuitem IDs
'	The explorer dispatches WM_COMMAND messages based on the range of
' command/menuitem IDs. All the IDs of menuitems that the view(right
' pane)inserts must be FCIDM_SHVIEWFIRST/LAST(otherwise, the explorer
' won't dispatch them). The view should not deal with any menuitems
' FCIDM_BROWSERFIRST/LAST(otherwise, it won't work with the future
' version of the shell).
'	FCIDM_SHVIEWFIRST/LAST	for the right pane(IShellView)
'	FCIDM_BROWSERFIRST/LAST	for the explorer frame(IShellBrowser)
'	FCIDM_GLOBAL/LAST	for the explorer's submenu IDs
'--------------------------------------------------------------------------

$define FCIDM_SHVIEWFIRST	0x0000
$define FCIDM_SHVIEWLAST	0x7fff
$define FCIDM_BROWSERFIRST	0xa000
$define FCIDM_BROWSERLAST	0xbf00
$define FCIDM_GLOBALFIRST	0x8000
$define FCIDM_GLOBALLAST	0x9fff

' Global submenu IDs and separator IDs
$define FCIDM_MENU_FILE	(FCIDM_GLOBALFIRST+0x0000)
$define FCIDM_MENU_EDIT	(FCIDM_GLOBALFIRST+0x0040)
$define FCIDM_MENU_VIEW	(FCIDM_GLOBALFIRST+0x0080)
$define FCIDM_MENU_VIEW_SEP_OPTIONS(FCIDM_GLOBALFIRST+0x0081)
$define FCIDM_MENU_TOOLS	(FCIDM_GLOBALFIRST+0x00c0)' for Win9x compat
$define FCIDM_MENU_TOOLS_SEP_GOTO	(FCIDM_GLOBALFIRST+0x00c1)' for Win9x compat
$define FCIDM_MENU_HELP	(FCIDM_GLOBALFIRST+0x0100)
$define FCIDM_MENU_FIND	(FCIDM_GLOBALFIRST+0x0140)
$define FCIDM_MENU_EXPLORE	(FCIDM_GLOBALFIRST+0x0150)
$define FCIDM_MENU_FAVORITES	(FCIDM_GLOBALFIRST+0x0170)

'--------------------------------------------------------------------------
' control IDs known to the view
'--------------------------------------------------------------------------

$define FCIDM_TOOLBAR	(FCIDM_BROWSERFIRST + 0)
$define FCIDM_STATUS	(FCIDM_BROWSERFIRST + 1)

'--------------------------------------------------------------------------
' The resource id of the offline cursor
' This cursor is avaialble shdocvw.dll
$define IDC_OFFLINE_HAND	103
'--------------------------------------------------------------------------

' SBCMDID_GETPANE - not necessarily order
$define PANE_NONE	-1
$define PANE_ZONE	1
$define PANE_OFFLINE	2
$define PANE_PRINTER	3
$define PANE_SSL	4
$define PANE_NAVIGATION	5
$define PANE_PROGRESS	6
$define PANE_PRIVACY	7

'-------------------------------------------------------------------------
' ICommDlgBrowser interface
'	ICommDlgBrowser interface is the interface that is provided by the new
' common dialog window to hook and modify the behavior of IShellView.	When
' a default view is created, it queries its parent IShellBrowser for the
' ICommDlgBrowser interface.	If supported, it calls to that interface
' several cases that need to behave differently a dialog.
' Member functions:
'	ICommDlgBrowser::OnDefaultCommand()
'	Called when the user double-clicks the view or presses Enter.	The
'	browser should return S_OK if it processed the action itself, S_FALSE
'	to let the view perform the default action.
'	ICommDlgBrowser::OnStateChange(ULONG uChange)
'	Called when some states the view change.	'uChange' is one of the
'	CDBOSC_* values.	This call is made after the state(selection, focus,
'	etc)has changed.	There is no return value.
'	ICommDlgBrowser::IncludeObject(pointer pidl)
'	Called when the view is enumerating objects.	'pidl' is a relative
'	IDLIST.	The browser should return S_OK to include the object the
'	view, S_FALSE to hide it
'-------------------------------------------------------------------------

$define CDBOSC_SETFOCUS	0x00000000
$define CDBOSC_KILLFOCUS	0x00000001
$define CDBOSC_SELCHANGE	0x00000002
$define CDBOSC_RENAME	0x00000003
$define CDBOSC_STATECHANGE	0x00000004

'	ICommDlgBrowser

interface ICommDlgBrowser ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'ICommDlgBrowser
	stdmethod OnDefaultCommand(IShellView ppshv),HRESULT
	stdmethod OnStateChange(IShellView ppshv,ULONG uChange),HRESULT
	stdmethod IncludeObject(IShellView ppshv,pointer pidl),HRESULT
endinterface

'-------------------------------------------------------------------------
' ICommDlgBrowser2 interface
' Member functions:
'	ICommDlgBrowser2::Notify(pointer pshv, DWORD dwNotfyType)
'	Called when the view is wants to notify common dialog when an event
'	occurrs.
'	CDB2N_CONTEXTMENU_START indicates the context menu has started.
'	CDB2N_CONTEXTMENU_DONE	indicates the context menu has completed.
'	ICommDlgBrowser2::GetDefaultMenuText(pointer pshv,
'	pointer pszText, INT cchMax)
'	Called when the view wants to get the default context menu text.
'	pszText points to buffer and cchMax specifies the size of the
'	buffer characters.	The browser on return has filled the buffer
'	with the default context menu text.	The Shell will call this method
'	with at least a buffer size of MAX_PATH.	The browser should return
'	S_OK if it returned a new default menu text, S_FALSE to let the view
'	to use the normal default menu text.
'	ICommDlgBrowser2::GetViewFlags(pointer pdwFlags)
'	Called when the view wants to determine	if special customization needs to
'	be done for the common dialog browser. For example View calls this function to
'	determin if all files(hidden and system)needs to be shown. If the GetViewFlags returns a DWORD with
'	CDB2GVF_SHOWALLFILES	flag set then it will show all the files.
'-------------------------------------------------------------------------

$define CDB2N_CONTEXTMENU_DONE	0x00000001
$define CDB2N_CONTEXTMENU_START 0x00000002

'GetViewFlags
$define CDB2GVF_SHOWALLFILES	0x00000001

'	ICommDlgBrowser2

interface ICommDlgBrowser2 ' ICommDlgBrowser
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'ICommDlgBrowser
	stdmethod OnDefaultCommand(pointer ppshv),HRESULT
	stdmethod OnStateChange(pointer ppshv,ULONG uChange),HRESULT
	stdmethod IncludeObject(pointer ppshv,pointer pidl),HRESULT
'ICommDlgBrowser2
	stdmethod Notify(IShellView ppshv,DWORD dwNotifyType),HRESULT
	stdmethod GetDefaultMenuText(IShellView ppshv,pointer pszText, INT cchMax),HRESULT
	stdmethod GetViewFlags(pointer pdwFlags),HRESULT
endinterface

' function assumes the size of the buffer(MAX_PATH). The pidl
' should point to a file system object.

declare import, SHGetPathFromIDListA(pointer pidl,LPSTR pszPath),BOOL
declare import, SHGetPathFromIDListW(pointer pidl,LPWSTR pszPath),BOOL
$ifdef UNICODE
declare import, SHGetPathFromIDList alias SHGetPathFromIDListW(pointer pidl,LPWSTR pszPath),BOOL
$else
declare import, SHGetPathFromIDList alias SHGetPathFromIDListA(pointer pidl,LPSTR pszPath),BOOL
$endif

declare import, SHCreateDirectory(HWND hwnd,LPCWSTR pszPath),int
declare import, SHCreateDirectoryExA(HWND hwnd,LPCSTR pszPath,pointer psa),int
declare import, SHCreateDirectoryExW(HWND hwnd,LPCWSTR pszPath,pointer psa),int
$ifdef UNICODE
declare import, SHCreateDirectoryEx alias SHCreateDirectoryExW(HWND hwnd,LPCWSTR pszPath,pointer psa),int
$else
declare import, SHCreateDirectoryEx alias SHCreateDirectoryExA(HWND hwnd,LPCSTR pszPath,pointer psa),int
$endif

declare import, SHOpenFolderAndSelectItems(pointer pidlFolder, UINT cidl,pointer apidl, DWORD dwFlags),HRESULT

declare import, SHCreateShellItem(pointer pidlParent,IShellFolder psfParent,pointer pidl,pointer ppsi),HRESULT

'-------------------------------------------------------------------------
' SHGetSpecialFolderLocation
'	Caller should use SHGetMalloc to obtain an allocator that can free the pidl
'-------------------------------------------------------------------------
' registry entries for special paths are kept :
$define REGSTR_PATH_SPECIAL_FOLDERS "\\Shell Folders"

$define CSIDL_DESKTOP	0x0000	' <desktop>
$define CSIDL_INTERNET	0x0001	' Internet Explorer(icon on desktop)
$define CSIDL_PROGRAMS	0x0002	' Start Menu\Programs
$define CSIDL_CONTROLS	0x0003	' My Computer\Control Panel
$define CSIDL_PRINTERS	0x0004	' My Computer\Printers
$define CSIDL_PERSONAL	0x0005	' My Documents
$define CSIDL_FAVORITES	0x0006	' <user name>\Favorites
$define CSIDL_STARTUP	0x0007	' Start Menu\Programs\Startup
$define CSIDL_RECENT	0x0008	' <user name>\Recent
$define CSIDL_SENDTO	0x0009	' <user name>\SendTo
$define CSIDL_BITBUCKET	0x000a	' <desktop>\Recycle Bin
$define CSIDL_STARTMENU	0x000b	' <user name>\Start Menu
$define CSIDL_MYDOCUMENTS	0x000c	' logical "My Documents" desktop icon
$define CSIDL_MYMUSIC	0x000d	' "My Music" folder
$define CSIDL_MYVIDEO	0x000e	' "My Videos" folder
$define CSIDL_DESKTOPDIRECTORY	0x0010	' <user name>\Desktop
$define CSIDL_DRIVES	0x0011	' My Computer
$define CSIDL_NETWORK	0x0012	' Network Neighborhood(My Network Places)
$define CSIDL_NETHOOD	0x0013	' <user name>\nethood
$define CSIDL_FONTS	0x0014	' windows\fonts
$define CSIDL_TEMPLATES	0x0015
$define CSIDL_COMMON_STARTMENU	0x0016	' All Users\Start Menu
$define CSIDL_COMMON_PROGRAMS	0x0017	' All Users\Start Menu\Programs
$define CSIDL_COMMON_STARTUP	0x0018	' All Users\Startup
$define CSIDL_COMMON_DESKTOPDIRECTORY	0x0019	' All Users\Desktop
$define CSIDL_APPDATA	0x001a	' <user name>\Application Data
$define CSIDL_PRINTHOOD	0x001b	' <user name>\PrintHood

$ifndef CSIDL_LOCAL_APPDATA
$define CSIDL_LOCAL_APPDATA	0x001c	' <user name>\Local Settings\Applicaiton Data(non roaming)
$endif

$define CSIDL_ALTSTARTUP	0x001d	' non localized startup
$define CSIDL_COMMON_ALTSTARTUP	0x001e	' non localized common startup
$define CSIDL_COMMON_FAVORITES	0x001f

$ifndef __shfolder_inc__
$define CSIDL_INTERNET_CACHE	0x0020
$define CSIDL_COOKIES	0x0021
$define CSIDL_HISTORY	0x0022
$define CSIDL_COMMON_APPDATA	0x0023	' All Users\Application Data
$define CSIDL_WINDOWS	0x0024	' GetWindowsDirectory()
$define CSIDL_SYSTEM	0x0025	' GetSystemDirectory()
$define CSIDL_PROGRAM_FILES	0x0026	' C:\Program Files
$define CSIDL_MYPICTURES	0x0027	' C:\Program Files\My Pictures
$endif

$define CSIDL_PROFILE	0x0028	' USERPROFILE
$define CSIDL_SYSTEMX86	0x0029	' x86 system directory on RISC
$define CSIDL_PROGRAM_FILESX86	0x002a	' x86 C:\Program Files on RISC

$ifndef __shfolder_inc__
$define CSIDL_PROGRAM_FILES_COMMON	0x002b	' C:\Program Files\Common
$endif

$define CSIDL_PROGRAM_FILES_COMMONX86	0x002c	' x86 Program Files\Common on RISC
$define CSIDL_COMMON_TEMPLATES	0x002d	' All Users\Templates

$ifndef __shfolder_inc__
$define CSIDL_COMMON_DOCUMENTS	0x002e	' All Users\Documents
$define CSIDL_COMMON_ADMINTOOLS	0x002f	' All Users\Start Menu\Programs\Administrative Tools
$define CSIDL_ADMINTOOLS	0x0030	' <user name>\Start Menu\Programs\Administrative Tools
$endif

$define CSIDL_CONNECTIONS	0x0031	' Network and Dial-up Connections
$define CSIDL_COMMON_MUSIC	0x0035	' All Users\My Music
$define CSIDL_COMMON_PICTURES	0x0036	' All Users\My Pictures
$define CSIDL_COMMON_VIDEO	0x0037	' All Users\My Video
$define CSIDL_RESOURCES	0x0038	' Resource Direcotry

$ifndef __shfolder_inc__
$define CSIDL_RESOURCES_LOCALIZED	0x0039	' Localized Resource Direcotry
$endif

$define CSIDL_COMMON_OEM_LINKS	0x003a	' Links to All Users OEM specific apps
$define CSIDL_CDBURN_AREA	0x003b	' USERPROFILE\Local Settings\Application Data\Microsoft\CD Burning
' unused	0x003c
$define CSIDL_COMPUTERSNEARME	0x003d	' Computers Near Me(computered from Workgroup membership)

$ifndef __shfolder_inc__
$define CSIDL_FLAG_CREATE	0x8000	' combine with CSIDL_ value to force folder creation SHGetFolderPath()
$endif

$define CSIDL_FLAG_DONT_VERIFY	0x4000	' combine with CSIDL_ value to return an unverified folder path
$define CSIDL_FLAG_NO_ALIAS	0x1000	' combine with CSIDL_ value to insure non-alias versions of the pidl
$define CSIDL_FLAG_PER_USER_INIT	0x0800	' combine with CSIDL_ value to indicate per-user init(eg. upgrade)
$define CSIDL_FLAG_MASK	0xFF00	' mask for all possible flag values

declare import, SHGetSpecialFolderLocation(HWND hwnd, int csidl,pointer ppidl),HRESULT

declare import, SHFlushSFCache()
declare import, SHCloneSpecialIDList(HWND hwnd, int csidl, BOOL fCreate),pointer

declare import, SHGetSpecialFolderPathA(HWND hwnd,LPSTR pszPath, int csidl, BOOL fCreate),BOOL
declare import, SHGetSpecialFolderPathW(HWND hwnd,LPWSTR pszPath, int csidl, BOOL fCreate),BOOL
$ifdef UNICODE
declare import, SHGetSpecialFolderPath alias SHGetSpecialFolderPathW(HWND hwnd,LPWSTR pszPath, int csidl, BOOL fCreate),BOOL
$else
declare import, SHGetSpecialFolderPath alias SHGetSpecialFolderPathA(HWND hwnd,LPSTR pszPath, int csidl, BOOL fCreate),BOOL
$endif

typedef SHGFP_TYPE int
$define SHGFP_TYPE_CURRENT	0	' current value for user, verify it exists
$define SHGFP_TYPE_DEFAULT	1	' default value, may not exist

declare import, SHGetFolderPathA(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,LPSTR pszPath),HRESULT
declare import, SHGetFolderPathW(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,LPWSTR pszPath),HRESULT
declare import, SHGetFolderLocation(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,pointer ppidl),HRESULT
declare import, SHGetFolderPathAndSubDirA(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,LPCSTR pszSubDir, LPSTR pszPath),HRESULT
declare import, SHGetFolderPathAndSubDirW(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,LPCWSTR pszSubDir,LPWSTR pszPath),HRESULT
$ifdef UNICODE
declare import, SHGetFolderPath alias SHGetFolderPathW(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,LPWSTR pszPath),HRESULT
declare import, SHGetFolderPathAndSubDir alias SHGetFolderPathAndSubDirW(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,LPCWSTR pszSubDir,LPWSTR pszPath),HRESULT
$else
declare import, SHGetFolderPath alias SHGetFolderPathA(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,LPSTR pszPath),HRESULT
declare import, SHGetFolderPathAndSubDir alias SHGetFolderPathAndSubDirA(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,LPCSTR pszSubDir, LPSTR pszPath),HRESULT
$endif

$define FCS_READ	0x00000001
$define FCS_FORCEWRITE	0x00000002
$define FCS_WRITE	(FCS_READ | FCS_FORCEWRITE)

$define FCS_FLAG_DRAGDROP	2

' Mask which values have been retreived or being set.
$define FCSM_VIEWID	0x00000001
$define FCSM_WEBVIEWTEMPLATE	0x00000002
$define FCSM_INFOTIP	0x00000004
$define FCSM_CLSID	0x00000008
$define FCSM_ICONFILE	0x00000010
$define FCSM_LOGO	0x00000020
$define FCSM_FLAGS	0x00000040

'$include <pshpack8.h>

' Used by SHGetSetFolderCustomSettingsA
type SHFOLDERCUSTOMSETTINGSA
	DWORD	dwSize
	DWORD	dwMask	' IN/OUT	Which Attributes to Get/Set
	pointer	pvid	' - if dwReadWrite is FCS_READ, - otherwise
	' The folder's WebView template path
	LPSTR	pszWebViewTemplate	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchWebViewTemplate	' - Specifies the size of the buffer pointed to by pszWebViewTemplate
		' Ignored if dwReadWrite is FCS_READ
	LPSTR	pszWebViewTemplateVersion	' currently only
	' Infotip for the folder
	LPSTR	pszInfoTip	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchInfoTip	' - Specifies the size of the buffer pointed to by pszInfoTip
		' Ignored if dwReadWrite is FCS_READ
	' CLSID that points to more info the registry
	pointer	pclsid	' - if dwReadWrite is FCS_READ, - otherwise
	' Other flags for the folder. Takes FCS_FLAG_* values
	DWORD	dwFlags	' - if dwReadWrite is FCS_READ, - otherwise

	LPSTR	pszIconFile	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchIconFile	' - Specifies the size of the buffer pointed to by pszIconFile
		' Ignored if dwReadWrite is FCS_READ

	int	iIconIndex	' - if dwReadWrite is FCS_READ, - otherwise

	LPSTR	pszLogo	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchLogo	' - Specifies the size of the buffer pointed to by pszIconFile
		' Ignored if dwReadWrite is FCS_READ
endtype

' Used by SHGetSetFolderCustomSettingsW
type SHFOLDERCUSTOMSETTINGSW
	DWORD	dwSize
	DWORD	dwMask	' IN/OUT	Which Attributes to Get/Set
	pointer	pvid	' - if dwReadWrite is FCS_READ, - otherwise
	' The folder's WebView template path
	LPWSTR	pszWebViewTemplate	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchWebViewTemplate	' - Specifies the size of the buffer pointed to by pszWebViewTemplate
		' Ignored if dwReadWrite is FCS_READ
	LPWSTR	pszWebViewTemplateVersion	' currently only
	' Infotip for the folder
	LPWSTR	pszInfoTip	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchInfoTip	' - Specifies the size of the buffer pointed to by pszInfoTip
		' Ignored if dwReadWrite is FCS_READ
	' CLSID that points to more info the registry
	pointer	pclsid	' - if dwReadWrite is FCS_READ, - otherwise
	' Other flags for the folder. Takes FCS_FLAG_* values
	DWORD	dwFlags	' - if dwReadWrite is FCS_READ, - otherwise

	LPWSTR	pszIconFile	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchIconFile	' - Specifies the size of the buffer pointed to by pszIconFile
		' Ignored if dwReadWrite is FCS_READ

	int	iIconIndex	' - if dwReadWrite is FCS_READ, - otherwise

	LPWSTR	pszLogo	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchLogo	' - Specifies the size of the buffer pointed to by pszIconFile
		' Ignored if dwReadWrite is FCS_READ
endtype

$ifdef UNICODE
type SHFOLDERCUSTOMSETTINGS
	DWORD	dwSize
	DWORD	dwMask	' IN/OUT	Which Attributes to Get/Set
	pointer	pvid	' - if dwReadWrite is FCS_READ, - otherwise
	' The folder's WebView template path
	LPWSTR	pszWebViewTemplate	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchWebViewTemplate	' - Specifies the size of the buffer pointed to by pszWebViewTemplate
		' Ignored if dwReadWrite is FCS_READ
	LPWSTR	pszWebViewTemplateVersion	' currently only
	' Infotip for the folder
	LPWSTR	pszInfoTip	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchInfoTip	' - Specifies the size of the buffer pointed to by pszInfoTip
		' Ignored if dwReadWrite is FCS_READ
	' CLSID that points to more info the registry
	pointer	pclsid	' - if dwReadWrite is FCS_READ, - otherwise
	' Other flags for the folder. Takes FCS_FLAG_* values
	DWORD	dwFlags	' - if dwReadWrite is FCS_READ, - otherwise

	LPWSTR	pszIconFile	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchIconFile	' - Specifies the size of the buffer pointed to by pszIconFile
		' Ignored if dwReadWrite is FCS_READ

	int	iIconIndex	' - if dwReadWrite is FCS_READ, - otherwise

	LPWSTR	pszLogo	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchLogo	' - Specifies the size of the buffer pointed to by pszIconFile
		' Ignored if dwReadWrite is FCS_READ
endtype
$else
type SHFOLDERCUSTOMSETTINGS
	DWORD	dwSize
	DWORD	dwMask	' IN/OUT	Which Attributes to Get/Set
	pointer	pvid	' - if dwReadWrite is FCS_READ, - otherwise
	' The folder's WebView template path
	LPSTR	pszWebViewTemplate	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchWebViewTemplate	' - Specifies the size of the buffer pointed to by pszWebViewTemplate
		' Ignored if dwReadWrite is FCS_READ
	LPSTR	pszWebViewTemplateVersion	' currently only
	' Infotip for the folder
	LPSTR	pszInfoTip	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchInfoTip	' - Specifies the size of the buffer pointed to by pszInfoTip
		' Ignored if dwReadWrite is FCS_READ
	' CLSID that points to more info the registry
	pointer	pclsid	' - if dwReadWrite is FCS_READ, - otherwise
	' Other flags for the folder. Takes FCS_FLAG_* values
	DWORD	dwFlags	' - if dwReadWrite is FCS_READ, - otherwise

	LPSTR	pszIconFile	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchIconFile	' - Specifies the size of the buffer pointed to by pszIconFile
		' Ignored if dwReadWrite is FCS_READ

	int	iIconIndex	' - if dwReadWrite is FCS_READ, - otherwise

	LPSTR	pszLogo	' - if dwReadWrite is FCS_READ, - otherwise
	DWORD	cchLogo	' - Specifies the size of the buffer pointed to by pszIconFile
		' Ignored if dwReadWrite is FCS_READ
endtype
$endif

'$include <poppack.h>	/* Return to byte packing */

' Gets/Sets the Folder Custom Settings for pszPath based on dwReadWrite. dwReadWrite can be FCS_READ/FCS_WRITE/FCS_FORCEWRITE
declare import, SHGetSetFolderCustomSettingsA(pointer pfcs,LPCSTR pszPath, DWORD dwReadWrite),HRESULT
declare import, SHGetSetFolderCustomSettingsW(pointer pfcs,LPCWSTR pszPath, DWORD dwReadWrite),HRESULT
$ifdef UNICODE
declare import, SHGetSetFolderCustomSettings alias SHGetSetFolderCustomSettingsW(pointer pfcs,LPCWSTR pszPath, DWORD dwReadWrite),HRESULT
$else
declare import, SHGetSetFolderCustomSettings alias SHGetSetFolderCustomSettingsA(pointer pfcs,LPCSTR pszPath, DWORD dwReadWrite),HRESULT
$endif

'-------------------------------------------------------------------------
' SHBrowseForFolder API
'-------------------------------------------------------------------------

'typedef int(CALLBACK* BFFCALLBACK)(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
typedef BFFCALLBACK int
'$include <pshpack8.h>

type BROWSEINFOA
	HWND	hwndOwner
	pointer pidlRoot
	LPSTR	pszDisplayName	' Return display name of item selected.
	LPCSTR	lpszTitle	' text to go the banner over the tree.
	UINT	ulFlags	' Flags that control the return stuff
	pointer lpfn
	LPARAM	lParam	' extra info that's passed back callbacks
	int	iImage	' output var: where to return the Image index.
endtype

type BROWSEINFOW
	HWND	hwndOwner
	pointer pidlRoot
	LPWSTR	pszDisplayName	' Return display name of item selected.
	LPCWSTR	lpszTitle	' text to go the banner over the tree.
	UINT	ulFlags	' Flags that control the return stuff
	pointer lpfn
	LPARAM	lParam	' extra info that's passed back callbacks
	int	iImage	' output var: where to return the Image index.
endtype
$ifdef UNICODE
type BROWSEINFO
	HWND	hwndOwner
	pointer pidlRoot
	LPWSTR	pszDisplayName	' Return display name of item selected.
	LPCWSTR	lpszTitle	' text to go the banner over the tree.
	UINT	ulFlags	' Flags that control the return stuff
	pointer lpfn
	LPARAM	lParam	' extra info that's passed back callbacks
	int	iImage	' output var: where to return the Image index.
endtype
$else
type BROWSEINFO
	HWND	hwndOwner
	pointer pidlRoot
	LPSTR	pszDisplayName	' Return display name of item selected.
	LPCSTR	lpszTitle	' text to go the banner over the tree.
	UINT	ulFlags	' Flags that control the return stuff
	pointer lpfn
	LPARAM	lParam	' extra info that's passed back callbacks
	int	iImage	' output var: where to return the Image index.
endtype
$endif
'$include <poppack.h>	/* Return to byte packing */

' Browsing for directory.
$define BIF_RETURNONLYFSDIRS	0x0001	' For finding a folder to start document searching
$define BIF_DONTGOBELOWDOMAIN	0x0002	' For starting the Find Computer
$define BIF_STATUSTEXT	0x0004	' Top of the dialog has 2 lines of text for BROWSEINFO.lpszTitle and one line if
		' this flag is set.	Passing the message BFFM_SETSTATUSTEXTA to the hwnd can set the
		' rest of the text.	This is not used with BIF_USENEWUI and BROWSEINFO.lpszTitle gets
		' all three lines of text.
$define BIF_RETURNFSANCESTORS	0x0008
$define BIF_EDITBOX	0x0010	' Add an editbox to the dialog
$define BIF_VALIDATE	0x0020	' insist on valid result(or CANCEL)

$define BIF_NEWDIALOGSTYLE	0x0040	' Use the new dialog layout with the ability to resize
		' Caller needs to call OleInitialize()before using this API

$define BIF_USENEWUI	(BIF_NEWDIALOGSTYLE | BIF_EDITBOX)

$define BIF_BROWSEINCLUDEURLS	0x0080	' Allow URLs to be displayed or entered.(Requires BIF_USENEWUI)
$define BIF_UAHINT	0x0100	' Add a UA hint to the dialog, place of the edit box. May not be combined with BIF_EDITBOX
$define BIF_NONEWFOLDERBUTTON	0x0200	' Do not add the "New Folder" button to the dialog.	Only applicable with BIF_NEWDIALOGSTYLE.
$define BIF_NOTRANSLATETARGETS 0x0400	' don't traverse target as shortcut

$define BIF_BROWSEFORCOMPUTER	0x1000	' Browsing for Computers.
$define BIF_BROWSEFORPRINTER	0x2000	' Browsing for Printers
$define BIF_BROWSEINCLUDEFILES 0x4000	' Browsing for Everything
$define BIF_SHAREABLE	0x8000	' sharable resources displayed(remote shares, requires BIF_USENEWUI)

' message from browser
$define BFFM_INITIALIZED	1
$define BFFM_SELCHANGED	2
$define BFFM_VALIDATEFAILEDA	3	' lParam:szPath ret:1(cont),0(EndDialog)
$define BFFM_VALIDATEFAILEDW	4	' lParam:wzPath ret:1(cont),0(EndDialog)
$define BFFM_IUNKNOWN	5	' provides IUnknown to client. lParam: pointer

$ifdef UNICODE
$define BFFM_VALIDATEFAILED BFFM_VALIDATEFAILEDW
$else
$define BFFM_VALIDATEFAILED BFFM_VALIDATEFAILEDA
$endif

' messages to browser
$define BFFM_SETSTATUSTEXTA	(WM_USER + 100)
$define BFFM_ENABLEOK	(WM_USER + 101)
$define BFFM_SETSELECTIONA	(WM_USER + 102)
$define BFFM_SETSELECTIONW	(WM_USER + 103)
$define BFFM_SETSTATUSTEXTW	(WM_USER + 104)
$define BFFM_SETOKTEXT	(WM_USER + 105)' Unicode only
$define BFFM_SETEXPANDED	(WM_USER + 106)' Unicode only

$ifdef UNICODE
$define BFFM_SETSTATUSTEXT BFFM_SETSTATUSTEXTW
$define BFFM_SETSELECTION BFFM_SETSELECTIONW
$else
$define BFFM_SETSTATUSTEXT BFFM_SETSTATUSTEXTA
$define BFFM_SETSELECTION BFFM_SETSELECTIONA
$endif
declare import, SHBrowseForFolderA(pointer lpbi),pointer
declare import, SHBrowseForFolderW(pointer lpbi),pointer
$ifdef UNICODE
declare import, SHBrowseForFolder alias SHBrowseForFolderW(pointer lpbi),pointer
$else
declare import, SHBrowseForFolder alias SHBrowseForFolderA(pointer lpbi),pointer
$endif

'-------------------------------------------------------------------------
' SHLoadInProc
'	When this function is called, the shell calls CoCreateInstance
'	(or equivalent)with CLSCTX_INPROC_SERVER and the specified CLSID
'	from within the shell's process and release it immediately.
'-------------------------------------------------------------------------

declare import, SHLoadInProc(pointer rclsid),HRESULT

'-------------------------------------------------------------------------
' SHEnableServiceObject
'	Like SHLoadInProc, but gives control over the object's lifetime
'	via fEnable parameter.	TRUE tells the shell to create the object
'	and hold onto it, FALSE tells the shell to look for the previously
'	created instance of the object and release it.
'-------------------------------------------------------------------------

declare import, SHEnableServiceObject(pointer rclsid, BOOL fEnable),HRESULT

'-------------------------------------------------------------------------
' Internet Shortcut Object
'-------------------------------------------------------------------------
' Cmds for CGID_ShortCut

$define ISHCUTCMDID_DOWNLOADICON	0
$define ISHCUTCMDID_INTSHORTCUTCREATE	1

$define CMDID_INTSHORTCUTCREATE 1

'	Helper function which returns a IShellFolder interface to the desktop
' folder. This is equivalent to call CoCreateInstance with CLSID_ShellDesktop.
'	CoCreateInstance(CLSID_Desktop, NULL,
'	CLSCTX_INPROC, IID_IShellFolder, &pshf)
declare import, SHGetDesktopFolder(pointer ppshf),HRESULT

' IShellFolder pointer parameters. the IUnknown for these are
' accessed through IBindCtx::RegisterObjectParam/GetObjectParam
' use this to provide the data needed create IDLists through
' IShellFolder::ParseDisplayName(). this data applies to the last element
' of the name that is parsed(c:\foo\bar.txt, data applies to bar.txt)
' this makes creating these IDLists much faster that suppling the name only

$define STR_FILE_SYS_BIND_DATA L"File System Bind Data"

'	IFileSystemBindData

interface IFileSystemBindData ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IFileSystemBindData
	stdmethod SetFindData(pointer pfd),HRESULT
	stdmethod GetFindData(pointer pfd),HRESULT
endinterface

'	IShellDetails

interface IShellDetails ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellDetails
	stdmethod GetDetailsOf(pointer pidl, UINT iColumn,pointer pDetails),HRESULT
	stdmethod ColumnClick(UINT iColumn),HRESULT
endinterface

'-------------------------------------------------------------------------
' IObjMgr interface
' [Member functions]
' IObjMgr::Append(punk)
'	This function adds an object to the end of a list of objects.
' IObjMgr::Remove(punk)
'	This function removes an object from a list of objects.
' This is implemented by CLSID_ACLMulti so each AutoComplete List
'(CLSID_ACLHistory, CLSID_ACListISF, CLSID_ACLMRU)can be added.
' CLSID_ACLMulti's IEnumString will then be the union of the results
' from the COM Objects added.
'-------------------------------------------------------------------------

interface IObjMgr ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IObjMgr
	stdmethod Append(IUnknown punk),HRESULT
	stdmethod Remove(IUnknown punk),HRESULT
endinterface

'-------------------------------------------------------------------------
' ICurrentWorkingDirectory interface
' [Member functions]
' ICurrentWorkingDirectory::GetDirectory(LPWSTR pwzPath, DWORD cchSize)
'	This function gets the Current Working Directory from a COM object that
'	stores such state.
' ICurrentWorkingDirectory::SetDirectory(pointer pwzPath)
'	This function sets the Current Working Directory of a COM object that
'	stores such state.
' This function can be used generically.	One COM object that implements it
' is CLSID_ACListISF so that the AutoComplete engine can complete relative
' paths.	SetDirectory()will set the "Current Working Directory" and
' AutoComplete with then complete both absolute and relative paths.
' For Example, if ::SetDirectory(L"C:\Program Files")is called, then
' the user can AutoComplete "..\winnt".	order to set the current
' working directory for non-file system paths, "ftp://ftp.microsoft.com/" or
' "Control Panel" for example, use IPersistFolder.
'-------------------------------------------------------------------------

interface ICurrentWorkingDirectory ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'ICurrentWorkingDirectory
	stdmethod GetDirectory(LPWSTR pwzPath, DWORD cchSize),HRESULT
	stdmethod SetDirectory(LPCWSTR pwzPath),HRESULT
endinterface

'-------------------------------------------------------------------------
' IACList interface
' [Member functions]
' IObjMgr::Expand(LPCOLESTR)
'	This function tells an autocomplete list to expand a specific string.
' If the user enters a multi-level path, AutoComplete(CLSID_AutoComplete)
' will use this interface to tell the "AutoComplete Lists" where to expand
' the results.
' For Example, if the user enters "C:\Program Files\Micros", AutoComplete
' first completely enumerate the "AutoComplete Lists" via IEnumString.	Then it
' will call the "AutoComplete Lists" with IACList::Expand(L"C:\Program Files").
' It will then enumerate the IEnumString interface again to get results in
' that directory.
'-------------------------------------------------------------------------

interface IACList ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IACList
	stdmethod Expand(pointer pszExpand),HRESULT
endinterface

'-------------------------------------------------------------------------
' IACList2 interface
' [Description]
'	This interface exists to allow the caller to set filter criteria
' for an AutoComplete List.	AutoComplete Lists generates the list of
' possible AutoComplete completions.	CLSID_ACListISF is one AutoComplete
' List COM object that implements this interface.
'-------------------------------------------------------------------------

typedef AUTOCOMPLETELISTOPTIONS int

$define ACLO_NONE	0	' don't enumerate anything
$define ACLO_CURRENTDIR	1	' enumerate current directory
$define ACLO_MYCOMPUTER	2	' enumerate MyComputer
$define ACLO_DESKTOP	4	' enumerate Desktop Folder
$define ACLO_FAVORITES	8	' enumerate Favorites Folder
$define ACLO_FILESYSONLY	16	' enumerate only the file system
$define ACLO_FILESYSDIRS	32	' enumerate only the file system dirs, UNC shares, and UNC servers.

interface IACList2 ' IACList
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IACList
	stdmethod Expand(pointer pszExpand),HRESULT
'IACList2
	stdmethod SetOptions(DWORD dwFlag),HRESULT
	stdmethod GetOptions(pointer pdwFlag),HRESULT
endinterface

/*-------------------------------------------------------------------------*\
	INTERFACE: IProgressDialog

	DESCRIPTION:
		CLSID_ProgressDialog/IProgressDialog exist to allow a caller to create
	a progress dialog, set it's title, animation, text lines, progress, and
	it will do all the work of updating on a background thread, being modless,
	handling the user cancelling the operation, and estimating the time remaining
	until the operation completes.

	USAGE:
		This is how the dialog is used during operations that require progress
	and the ability to cancel:
		DWORD dwComplete, dwTotal
		pointer	ppd
		CoCreateInstance(CLSID_ProgressDialog, NULL, CLSCTX_INPROC_SERVER, IID_IProgressDialog,(pointer)&ppd)
		ppd->SetTitle(L"My Slow Operation")	' Set the title of the dialog.
		ppd->SetAnimation(hInstApp, IDA_OPERATION_ANIMATION)	' Set the animation to play.
		ppd->StartProgressDialog(hwndParent, punk, PROGDLG_AUTOTIME, NULL)' Display and enable automatic estimated time remaining.
		ppd->SetCancelMsg(L"Please wait while the current operation is cleaned up", NULL);	' Will only be displayed if Cancel button is pressed.

		dwComplete = 0
		dwTotal = CalcTotalUnitsToDo()

		' Reset because CalcTotalUnitsToDo()took a long time and the estimated time
		' is based on the time between ::StartProgressDialog()and the first
		' ::SetProgress()call.
		ppd->Timer(PDTIMER_RESET, NULL)

		for(nIndex = 0; nIndex < nTotal; nIndex++)
		if(TRUE == ppd->HasUserCancelled())
		break

		ppd->SetLine(2, L"I'm processing item n", FALSE, NULL)
		dwComplete += DoSlowOperation()

		ppd->SetProgress(dwCompleted, dwTotal)
		next nIndex

		ppd->StopProgressDialog()
		ppd->Release()
	}
\*-------------------------------------------------------------------------*/

' Flags for IProgressDialog::StartProgressDialog()(dwFlags)
$define PROGDLG_NORMAL	0x00000000	' default normal progress dlg behavior
$define PROGDLG_MODAL	0x00000001	' the dialog is modal to its hwndParent(default is modeless)
$define PROGDLG_AUTOTIME	0x00000002	' automatically updates the "Line3" text with the "time remaining"(you cant call SetLine3 if you passs this!)
$define PROGDLG_NOTIME	0x00000004	' we dont show the "time remaining" if this is set. We need this if dwTotal < dwCompleted for sparse files
$define PROGDLG_NOMINIMIZE	0x00000008	' Do not have a minimize button the caption bar.
$define PROGDLG_NOPROGRESSBAR	0x00000010	' Don't display the progress bar

' Time Actions(dwTimerAction)
$define PDTIMER_RESET	0x00000001	' Reset the timer so the progress will be calculated from now until the first ::SetProgress()is called so
		' those this time will correspond to the values passed to ::SetProgress().	Only do this before ::SetProgress()is called.

'	IProgressDialog

interface IProgressDialog ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IProgressDialog
	stdmethod StartProgressDialog(HWND hwndParent,comref punkEnableModless, DWORD dwFlags, LPCVOID pvResevered),HRESULT
	stdmethod StopProgressDialog(),HRESULT
	stdmethod SetTitle(LPCWSTR pwzTitle),HRESULT
	stdmethod SetAnimation(HINSTANCE hInstAnimation, UINT idAnimation),HRESULT
	stdmethod HasUserCancelled(),BOOL
	stdmethod SetProgress(DWORD dwCompleted, DWORD dwTotal),HRESULT
	stdmethod SetProgress64(ULONGLONG ullCompleted, ULONGLONG ullTotal),HRESULT
	stdmethod SetLine(DWORD dwLineNum,LPCWSTR pwzString, BOOL fCompactPath, LPCVOID pvResevered),HRESULT
	stdmethod SetCancelMsg(LPCWSTR pwzCancelMsg, LPCVOID pvResevered),HRESULT
	stdmethod _Timer(DWORD dwTimerAction, LPCVOID pvResevered),HRESULT
endinterface

'==========================================================================
' IInputObjectSite/IInputObject interfaces
'	These interfaces allow us(or ISVs)to install/update external Internet
' Toolbar for IE and the shell. The frame will simply get the CLSID from
' registry(to be defined)and CoCreateInstance it.
'==========================================================================

'-------------------------------------------------------------------------
' IInputObjectSite interface
'	A site implements this interface so the object can communicate
' focus change to it.
' [Member functions]
' IInputObjectSite::OnFocusChangeIS(punkObj, fSetFocus)
'	Object(punkObj)is getting or losing the focus.
'-------------------------------------------------------------------------

'	IInputObjectSite

interface IInputObjectSite ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IInputObjectSite
	stdmethod OnFocusChangeIS(comref punkObj, BOOL fSetFocus),HRESULT
endinterface

'-------------------------------------------------------------------------
' IInputObject interface
'	An object implements this interface so the site can communicate
' activation and accelerator events to it.
' [Member functions]
' IInputObject::UIActivateIO(fActivate, lpMsg)
'	Activates or deactivates the object.	lpMsg may be NULL.	Returns
'	S_OK if the activation succeeded.
' IInputObject::HasFocusIO()
'	Returns S_OK if the object has the focus, S_FALSE if not.
' IInputObject::TranslateAcceleratorIO(lpMsg)
'	Allow the object to process the message.	Returns S_OK if the
'	message was processed(eaten).
'-------------------------------------------------------------------------

'	IInputObject

interface IInputObject ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IInputObject
	stdmethod UIActivateIO(BOOL fActivate,pointer lpMsg),HRESULT
	stdmethod HasFocusIO(),HRESULT
	stdmethod TranslateAcceleratorIO(pointer lpMsg),HRESULT
endinterface

'==========================================================================
' IDockingWindowSite/IDockingWindow/IDockingWindowFrame interfaces
' IInputObjectSite/IInputObject interfaces
'	These interfaces allow us(or ISVs)to install/update external Internet
' Toolbar for IE and the shell. The frame will simply get the CLSID from
' registry(to be defined)and CoCreateInstance it.
'==========================================================================

'-------------------------------------------------------------------------
' IDockingWindowSite interface
'	A site implements this interface so the object can negotiate for
' and inquire about real estate on the site.
' [Member functions]
' IDockingWindowSite::GetBorderDW(punkObj, prcBorder)
'	Site returns the bounding rectangle of the given source object
'	(punkObj).
' IDockingWindowSite::RequestBorderSpaceDW(punkObj, pbw)
'	Object requests that the site makes room for it, as specified in
'	* pbw.
' IDockingWindowSite::SetBorderSpaceDW(punkObj, pbw)
'	Object requests that the site set the border spacing to the size
'	specified* pbw.
'-------------------------------------------------------------------------

'	IDockingWindowSite

interface IDockingWindowSite ' IOleWindow
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
	stdmethod GetWindow(pointer phwnd),HRESULT
	stdmethod ContextSensitiveHelp(BOOL fEnterMode),HRESULT
'IDockingWindowSite
	stdmethod GetBorderDW(comref punkObj,pointer prcBorder),HRESULT
	stdmethod RequestBorderSpaceDW(comref punkObj,pointer pbw),HRESULT
	stdmethod SetBorderSpaceDW(comref punkObj,pointer pbw),HRESULT
endinterface

'-------------------------------------------------------------------------
' IDockingWindowFrame interface
' [Member functions]
' IDockingWindowFrame::AddToolbar(punkSrc, pwszItem, dwReserved)
' IDockingWindowFrame::RemoveToolbar(punkSrc, dwRemoveFlags)
' IDockingWindowFrame::FindToolbar(pwszItem, riid, ppv)
'-------------------------------------------------------------------------

' flags for RemoveToolbar
$define DWFRF_NORMAL	0x0000
$define DWFRF_DELETECONFIGDATA	0x0001

' flags for AddToolbar
$define DWFAF_HIDDEN	0x0001	' add hidden

'	IDockingWindowFrame

interface IDockingWindowFrame ' IOleWindow
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
	stdmethod GetWindow(pointer phwnd),HRESULT
	stdmethod ContextSensitiveHelp(BOOL fEnterMode),HRESULT
'IDockingWindowFrame
	stdmethod AddToolbar(comref punkSrc,LPCWSTR pwszItem, DWORD dwAddFlags),HRESULT
	stdmethod RemoveToolbar(comref punkSrc, DWORD dwRemoveFlags),HRESULT
	stdmethod FindToolbar(LPCWSTR pwszItem,pointer riid,pointer ppv),HRESULT
endinterface

'-------------------------------------------------------------------------
' IRunnableTask interface
'	This is a free threaded interface used for putting items on a background
' scheduler for execution within the view.	It allows a scheduler to start and
' stop tasks on as many worker threads as it deems necessary.
' Run(), Kill()and Suspend()may be called from different threads.
' [Member functions]
' IRunnableTask::Run()
'	Initiate the task to run.	This should return E_PENDING if the task
'	has been suspended.
' IRunnableTask::Kill()
' IRunnableTask::Suspend()
' IRunnableTask::Resume()
' IRunnableTask::IsRunning()
'-------------------------------------------------------------------------

' Convenient state values
$define IRTIR_TASK_NOT_RUNNING	0
$define IRTIR_TASK_RUNNING	1
$define IRTIR_TASK_SUSPENDED	2
$define IRTIR_TASK_PENDING	3
$define IRTIR_TASK_FINISHED	4

'	IRunnableTask

interface IRunnableTask ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IRunnableTask
	stdmethod Run(),HRESULT
	stdmethod Kill(BOOL fWait),HRESULT
	stdmethod Suspend(),HRESULT
	stdmethod Resume(),HRESULT
	stdmethod IsRunning(),ULONG
endinterface

type TASKOWNERID
	UINT	Data1
	USHORT Data2
	USHORT Data3
	UCHAR Data4[ 8 ]
endtype
type REFTASKOWNERID
	UINT	Data1
	USHORT Data2
	USHORT Data3
	UCHAR Data4[ 8 ]
endtype

' ---IShellTaskScheduler
' An interface for interacting with and controlling a task scheduler. This
' interface does not need to be free-threaded unless the items the queue
' interact with the scheduler as well as the main execution thread on which the
' task scheduler was created.

' IShellTaskScheduler::AddTask()
'	Adds Tasks to the scheduler's background queue. The TASKOWNERID allow particular types
'	of tasks to be grouped so that they can be counted or removed. The lParam allows the task
'	to be associated with a particular item(for example an item a listview).
' IShellTaskScheduler::RemoveTasks()
'	Removes tasks from the scheduler's queue. These can be sepcified terms of their TASKOWNERID
'	or their LPARAM, or both, or neither(TOID_NULL && ITSAT_DEFAULT_LPARAM results all tasks being
'	removed). If a task that matches is currently running and ITaskScheduler::Status()has been
'	passeed ITSSFLAG_KILL_ON_DESTROY then the scheduler will attempt to kill the current task. The
'	fWaitIfRunning parameter is then passed to IRunnableTask::Kill().
' IShellTaskScheduler::CountTasks()
'	Counts the tasks the queue depending upon the TASKOWNERID and the LPARAM passed.(TOID_NULL and
'	ITSAT_DEFAULT_LPARAM will count all tasks the queue)
' IShellTaskScheduler::Status()
'	This sets the ReleaseStatus for the current task and the background thread timeout. When
'	ITaskScheduler::RemoveTasks()is called and there is a task currently running that matches
'	ITSSFLAG_COMPLETE_ON_DESTROY will cause TRUE to be passed to the task's IRunnableTask::Kill().
'	The dwThreadTimeout parameter if not set to the default will cause the background thread to
'	die if no new tasks have been added to the queue the timeout period. The Thread will be
'	recreated when the next new task is added.

'//////////////////////
' Status()flags,
' wait for the current task to complete before deleting the scheduler
$define ITSSFLAG_COMPLETE_ON_DESTROY	0x0000

' kill the current task(if there is one)when the task scheduler is deleted
$define ITSSFLAG_KILL_ON_DESTROY	0x0001

$define ITSSFLAG_SUPPORTS_TERMINATE	0x0002

$define ITSSFLAG_FLAGS_MASK	0x0003

' set the timeout for killing the thread when the object is terminated.
' this timeout can be used to stop the object from blocking the system
' indefinitely.
$define ITSSFLAG_THREAD_TERMINATE_TIMEOUT	0x0010

' set the timeout for threads that are idle the thread pool
$define ITSSFLAG_THREAD_POOL_TIMEOUT	0x0020

' The default timeout passed to release Status to determine how long the thread
' can be asleep before the thread is expired
$define ITSS_THREAD_DESTROY_DEFAULT_TIMEOUT	(60*1000)

' default, we won't kill it...
$define ITSS_THREAD_TERMINATE_TIMEOUT	(INFINITE)

' there is no change to the thread timeout
$define ITSS_THREAD_TIMEOUT_NO_CHANGE	(INFINITE - 1)

' the LPARAM allows task to be associated with items thus all tasks owned by a
' particular item can be accessed by passing a non default value for this parameter
$define ITSAT_DEFAULT_LPARAM	0xffffffff

' Task priorities
' ---------------
' This depends on the cooperation of tasks currently under execution. New tasks will
' be inserted the queue priority order. If a task of a low priority is currently
' under execution when a higher priority task is added, the scheduler will attempt
' to suspend the task currently under execution. It will be resumed when the other tasks
' have been completed.
$define ITSAT_DEFAULT_PRIORITY	0x10000000
$define ITSAT_MAX_PRIORITY	0x7fffffff
$define ITSAT_MIN_PRIORITY	0x00000000

'$define TOID_NULL	CLSID_NULL
extern _TOID_NULL as GUID

interface IShellTaskScheduler ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellTaskScheduler
	stdmethod AddTask(comref pTask,REFTASKOWNERID rtoid byval,DWORD_PTR lParam,DWORD dwPriority),HRESULT
	stdmethod RemoveTasks(REFTASKOWNERID rtoid byval,DWORD_PTR lParam,BOOL fWaitIfRunning),HRESULT
	stdmethod CountTasks(REFTASKOWNERID rtoid byval),UINT
	stdmethod Status(DWORD dwReleaseStatus, DWORD dwThreadTimeout),HRESULT
endinterface

$define ITSSFLAG_TASK_PLACEINFRONT 0x00000001
$define ITSSFLAG_TASK_PLACEINBACK	0x00000002

interface IShellTaskScheduler2 ' IShellTaskScheduler
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellTaskScheduler
	stdmethod AddTask(comref pTask,REFTASKOWNERID rtoid byval,DWORD_PTR lParam,DWORD dwPriority),HRESULT
	stdmethod RemoveTasks(REFTASKOWNERID rtoid byval,DWORD_PTR lParam,BOOL fWaitIfRunning),HRESULT
	stdmethod CountTasks(REFTASKOWNERID rtoid byval),UINT
	stdmethod Status(DWORD dwReleaseStatus, DWORD dwThreadTimeout),HRESULT
'IShellTaskScheduler2
	stdmethod AddTask2(comref pTask,REFTASKOWNERID rtoid byval,DWORD_PTR lParam,DWORD dwPriority,DWORD grfFlags),HRESULT
	stdmethod MoveTask(REFTASKOWNERID rtoid byval,DWORD_PTR lParam,DWORD dwPriority,DWORD grfFlags),HRESULT
endinterface

/** ******** IThumbnailCapture
*	CaptureThumbnail : takes an IHTMLDocument2 and returns a thumbnail of specified
*	size as an hbitmap
 */

'	IThumbnailCapture

interface IThumbnailCapture ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IThumbnailCapture
	stdmethod CaptureThumbnail(pointer pMaxSize,comref pHTMLDoc2,pointer phbmThumbnail),HRESULT
endinterface

'$include <pshpack8.h>

type ENUMSHELLIMAGESTOREDATA
	iwstring	szPath[MAX_PATH]
	FILETIME	ftTimeStamp
endtype

'$include <poppack.h>	/* Return to byte packing */

'	IEnumShellImageStore

interface IEnumShellImageStore ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IEnumShellImageStore
	stdmethod	Reset(),HRESULT
	stdmethod _Next(ULONG celt,pointer prgElt,pointer pceltFetched),HRESULT
	stdmethod	Skip(ULONG celt),HRESULT
	stdmethod	Clone(pointer ppEnum),HRESULT
endinterface

' flags used to determine the capabilities of the storage for the images
$define SHIMSTCAPFLAG_LOCKABLE	0x0001	' does the store require/support locking
$define SHIMSTCAPFLAG_PURGEABLE	0x0002	' does the store require dead items purging externally ?

'	IShellImageStore

' this interface is used to manipulate the Image cache. It can potentially be used
' a free threaded manner conjunction with the Lock parameter to Open and close
interface IShellImageStore ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellImageStore
	' if the lock parameter is used, then all other calls into
	' open and/or create will block until the lock is released.
	stdmethod Open(DWORD dwMode,pointer pdwLock),HRESULT
	stdmethod Create(DWORD dwMode,pointer pdwLock),HRESULT

	' if the lock is passed to either of these two methods, it releases the lock
	' once the operation is complete.
	stdmethod ReleaseLock(pointer pdwLock),HRESULT
	stdmethod Close(pointer pdwLock),HRESULT
	stdmethod Commit(pointer pdwLock),HRESULT
	stdmethod IsLocked(),HRESULT
	stdmethod GetMode(pointer pdwMode),HRESULT
	stdmethod GetCapabilities(pointer pdwCapMask),HRESULT
	stdmethod AddEntry(LPCWSTR pszName,pointer pftTimeStamp, DWORD dwMode, HBITMAP hImage),HRESULT
	stdmethod GetEntry(LPCWSTR pszName, DWORD dwMode,pointer phImage),HRESULT
	stdmethod DeleteEntry(LPCWSTR pszName),HRESULT
	stdmethod IsEntryInStore(LPCWSTR pszName,pointer pftTimeStamp),HRESULT
	stdmethod _Enum(pointer ppEnum),HRESULT
endinterface

'//	IShellFolderBand

' Field mask
$define ISFB_MASK_STATE	0x00000001 ' TRUE if dwStateMask and dwState is valid
$define ISFB_MASK_BKCOLOR	0x00000002 ' TRUE if crBkgnd field is valid
$define ISFB_MASK_VIEWMODE	0x00000004 ' TRUE if wViewMode field is valid
$define ISFB_MASK_SHELLFOLDER	0x00000008
$define ISFB_MASK_IDLIST	0x00000010
$define ISFB_MASK_COLORS	0x00000020 ' TRUE if crXXXX fields are valid(except bkgnd)

$define ISFB_STATE_DEFAULT	0x00000000
$define ISFB_STATE_DEBOSSED	0x00000001
$define ISFB_STATE_ALLOWRENAME	0x00000002
$define ISFB_STATE_NOSHOWTEXT	0x00000004 ' TRUE if _fNoShowText
$define ISFB_STATE_CHANNELBAR	0x00000010 ' TRUE if we want NavigateTarget support
$define ISFB_STATE_QLINKSMODE	0x00000020 ' TRUE if we want to turn off drag & drop onto content items
$define ISFB_STATE_FULLOPEN	0x00000040 ' TRUE if band should maximize when opened
$define ISFB_STATE_NONAMESORT	0x00000080 ' TRUE if band should _not_ sort icons by name
$define ISFB_STATE_BTNMINSIZE	0x00000100 ' TRUE if band should report min thickness of button

$define ISFBVIEWMODE_SMALLICONS	0x0001
$define ISFBVIEWMODE_LARGEICONS	0x0002
$define ISFBVIEWMODE_LOGOS	0x0003

'$include <pshpack8.h>

type BANDINFOSFB
	DWORD	dwMask	' [in] ISFB_MASK mask of valid fields from crBkgnd on
	DWORD	dwStateMask	' [in] ISFB_STATE mask of dwState bits being set/queried
	DWORD	dwState	' [in/out] ISFB_STATE bits
	COLORREF	crBkgnd	' [in/out]
	COLORREF	crBtnLt	' [in/out]
	COLORREF	crBtnDk	' [in/out]
	USHORT wViewMode	' [in/out]
	USHORT wAlign	' not used(yet)
	IShellFolder	psf	' [out]
	pointer pidl	' [out]
endtype

'$include <poppack.h>	/* Return to byte packing */

interface IShellFolderBand ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellFolderBand
	stdmethod InitializeSFB(comref psf,pointer pidl),HRESULT
	stdmethod SetBandInfoSFB(pointer pbi),HRESULT
	stdmethod GetBandInfoSFB(pointer pbi),HRESULT
endinterface

' Command Target IDs
$define SFBID_PIDLCHANGED 0

'//	IDeskBarClient

'	IDeskBarClient

interface IDeskBarClient ' IOleWindow
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
	stdmethod GetWindow(pointer phwnd),HRESULT
	stdmethod ContextSensitiveHelp(BOOL fEnterMode),HRESULT
'IDeskBarClient
	stdmethod SetDeskBarSite(comref punkSite),HRESULT
	stdmethod SetModeDBC(DWORD dwMode),HRESULT
	stdmethod UIActivateDBC(DWORD dwState),HRESULT
	stdmethod _GetSize(DWORD dwWhich,pointer prc),HRESULT
endinterface

$define DBC_GS_IDEAL	0	' get the ideal size
$define DBC_GS_SIZEDOWN	1	' clip the height of a rect to a multiple of the rebar's integral size

$define DBC_HIDE	0 ' Band is hidden(being destroyed)
$define DBC_SHOW	1 ' Band is visible
$define DBC_SHOWOBSCURE 2 ' Band is completely obscured

$define DBCID_EMPTY	0	' bandsite is empty
$define DBCID_ONDRAG	1	'(down)DragMoveEnter/Leave vaIn:I4:eDrag
$define DBCID_CLSIDOFBAR	2	' clsid of bar inside
$define DBCID_RESIZE	3	' resize from keyboard
$define DBCID_GETBAR	4	' returns vaOut:VT_UNKNOWN of hosting dockbar(IDeskBar)

' We need to make sure that WININET.H is included before this interface is
' used because the COMPONENT structure uses INTERNET_MAX_URL_LENGTH
$ifdef _WININET_
'===============================================================\
'	Flags and structures used by IActiveDesktop

type WALLPAPEROPT
	DWORD	dwSize	' size of this Structure.
	DWORD	dwStyle	' WPSTYLE_* mentioned above
endtype

type COMPONENTSOPT
	DWORD	dwSize	//Size of this structure
	BOOL	fEnableComponents 'Enable components?
	BOOL	fActiveDesktop	' Active desktop enabled ?
endtype

type COMPPOS
	DWORD	dwSize	//Size of this structure
	int	iLeft	//Left of top-left corner screen co-ordinates.
	int	iTop	//Top of top-left corner screen co-ordinates.
	DWORD	dwWidth	' Width pixels.
	DWORD	dwHeight	' Height pixels.
	int	izIndex	' Indicates the Z-order of the component.
	BOOL	fCanResize	' Is the component resizeable?
	BOOL	fCanResizeX	' Resizeable X-direction?
	BOOL	fCanResizeY	' Resizeable Y-direction?
	int	iPreferredLeftPercent	//Left of top-left corner as percent of screen width
	int	iPreferredTopPercent	//Top of top-left corner as percent of screen height
endtype

type COMPSTATEINFO
	DWORD	dwSize	' Size of this structure.
	int	iLeft	' Left of the top-left corner screen co-ordinates.
	int	iTop	' Top of top-left corner screen co-ordinates.
	DWORD	dwWidth	' Width pixels.
	DWORD	dwHeight	' Height pixels.
	DWORD	dwItemState	' State of the component(full-screen mode or split-screen or normal state.
endtype

$define COMPONENT_TOP(0x3fffffff)	' izOrder value meaning component is at the top

' iCompType values
$define COMP_TYPE_HTMLDOC	0
$define COMP_TYPE_PICTURE	1
$define COMP_TYPE_WEBSITE	2
$define COMP_TYPE_CONTROL	3
$define COMP_TYPE_CFHTML	4
$define COMP_TYPE_MAX	4

' The following is the COMPONENT structure used IE4.01, IE4.0 and Memphis. It is kept here for compatibility
' reasons.
type IE4COMPONENT
	DWORD	dwSize	//Size of this structure
	DWORD	dwID	//Reserved: Set it always to zero.
	int	iComponentType	//One of COMP_TYPE_*
	BOOL	fChecked	' Is this component enabled?
	BOOL	fDirty	' Had the component been modified and not yet saved to disk?
	BOOL	fNoScroll	' Is the component scrollable?
	COMPPOS cpPos	' Width, height etc.,
	iwstring wszFriendlyName[MAX_PATH]	' Friendly name of component.
	iwstring wszSource[INTERNET_MAX_URL_LENGTH] 'URL of the component.
	iwstring wszSubscribedURL[INTERNET_MAX_URL_LENGTH] 'Subscrined URL
endtype

' The following is the new NT5 component structure. Note that the initial portion of this component exactly
' matches the IE4COMPONENT structure. All new fields are added at the bottom and the dwSize field is used to
' distinguish between IE4COMPONENT and the new COMPONENT structures.
type COMPONENT
	DWORD	dwSize	//Size of this structure
	DWORD	dwID	//Reserved: Set it always to zero.
	int	iComponentType	//One of COMP_TYPE_*
	BOOL	fChecked	' Is this component enabled?
	BOOL	fDirty	' Had the component been modified and not yet saved to disk?
	BOOL	fNoScroll	' Is the component scrollable?
	COMPPOS cpPos	' Width, height etc.,
	iwstring wszFriendlyName[MAX_PATH]	' Friendly name of component.
	iwstring wszSource[INTERNET_MAX_URL_LENGTH] 'URL of the component.
	iwstring wszSubscribedURL[INTERNET_MAX_URL_LENGTH] 'Subscrined URL

	'New fields are added below. Everything above here must exactly match the IE4COMPONENT Structure.
	DWORD	dwCurItemState ' Current state of the Component.
	COMPSTATEINFO	csiOriginal	' Original state of the component when it was first added.
	COMPSTATEINFO	csiRestored	' Restored state of the component.
endtype

' Defines for dwCurItemState
$define IS_NORMAL	0x00000001
$define IS_FULLSCREEN	0x00000002
$define IS_SPLIT	0x00000004
$define IS_VALIDSIZESTATEBITS	(IS_NORMAL | IS_SPLIT | IS_FULLSCREEN)	' The set of IS_* state bits which define the "size" of the component - these bits are mutually exclusive.
$define IS_VALIDSTATEBITS	(IS_NORMAL | IS_SPLIT | IS_FULLSCREEN | 0x80000000 | 0x40000000)	' All of the currently defined IS_* bits.

'//////////////////////////////////////////
' Flags for IActiveDesktop::ApplyChanges()
$define AD_APPLY_SAVE	0x00000001
$define AD_APPLY_HTMLGEN	0x00000002
$define AD_APPLY_REFRESH	0x00000004
$define AD_APPLY_ALL	(AD_APPLY_SAVE | AD_APPLY_HTMLGEN | AD_APPLY_REFRESH)
$define AD_APPLY_FORCE	0x00000008
$define AD_APPLY_BUFFERED_REFRESH 0x00000010
$define AD_APPLY_DYNAMICREFRESH	0x00000020

'//////////////////////////////////////////
' Flags for IActiveDesktop::GetWallpaperOptions()
'	IActiveDesktop::SetWallpaperOptions()
$define WPSTYLE_CENTER	0
$define WPSTYLE_TILE	1
$define WPSTYLE_STRETCH	2
$define WPSTYLE_MAX	3

'//////////////////////////////////////////
' Flags for IActiveDesktop::ModifyComponent()

$define COMP_ELEM_TYPE	0x00000001
$define COMP_ELEM_CHECKED	0x00000002
$define COMP_ELEM_DIRTY	0x00000004
$define COMP_ELEM_NOSCROLL	0x00000008
$define COMP_ELEM_POS_LEFT	0x00000010
$define COMP_ELEM_POS_TOP	0x00000020
$define COMP_ELEM_SIZE_WIDTH	0x00000040
$define COMP_ELEM_SIZE_HEIGHT	0x00000080
$define COMP_ELEM_POS_ZINDEX	0x00000100
$define COMP_ELEM_SOURCE	0x00000200
$define COMP_ELEM_FRIENDLYNAME	0x00000400
$define COMP_ELEM_SUBSCRIBEDURL 0x00000800
$define COMP_ELEM_ORIGINAL_CSI	0x00001000
$define COMP_ELEM_RESTORED_CSI	0x00002000
$define COMP_ELEM_CURITEMSTATE	0x00004000

$define COMP_ELEM_ALL	0x00007FFF

'//////////////////////////////////////////
' Flags for IActiveDesktop::AddDesktopItemWithUI()
typedef DTI_ADTIWUI int
$define DTI_ADDUI_DEFAULT	0x00000000
$define DTI_ADDUI_DISPSUBWIZARD	0x00000001
$define DTI_ADDUI_POSITIONITEM	0x00000002

'//////////////////////////////////////////
' Flags for IActiveDesktop::AddUrl()
$define ADDURL_SILENT	0x0001

'//////////////////////////////////////////
' Default positions for ADI
$define COMPONENT_DEFAULT_LEFT	(0xFFFF)
$define COMPONENT_DEFAULT_TOP	(0xFFFF)

'	Interface for manipulating the Active Desktop.

interface IActiveDesktop ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IActiveDesktop
	stdmethod ApplyChanges(DWORD dwFlags),HRESULT
	stdmethod GetWallpaper(LPWSTR pwszWallpaper, UINT cchWallpaper, DWORD dwReserved),HRESULT
	stdmethod SetWallpaper(LPCWSTR pwszWallpaper, DWORD dwReserved),HRESULT
	stdmethod GetWallpaperOptions(pointer pwpo, DWORD dwReserved),HRESULT
	stdmethod SetWallpaperOptions(pointer pwpo, DWORD dwReserved),HRESULT
	stdmethod GetPattern(LPWSTR pwszPattern, UINT cchPattern, DWORD dwReserved),HRESULT
	stdmethod SetPattern(LPCWSTR pwszPattern, DWORD dwReserved),HRESULT
	stdmethod GetDesktopItemOptions(pointer pco, DWORD dwReserved),HRESULT
	stdmethod SetDesktopItemOptions(pointer pco, DWORD dwReserved),HRESULT
	stdmethod AddDesktopItem(pointer pcomp, DWORD dwReserved),HRESULT
	stdmethod AddDesktopItemWithUI(HWND hwnd,pointer pcomp, DWORD dwReserved),HRESULT
	stdmethod ModifyDesktopItem(pointer pcomp, DWORD dwFlags),HRESULT
	stdmethod RemoveDesktopItem(pointer pcomp, DWORD dwReserved),HRESULT
	stdmethod GetDesktopItemCount(pointer lpiCount, DWORD dwReserved),HRESULT
	stdmethod GetDesktopItem(int nComponent,pointer pcomp, DWORD dwReserved),HRESULT
	stdmethod GetDesktopItemByID(ULONG_PTR dwID,pointer pcomp, DWORD dwReserved),HRESULT
	stdmethod GenerateDesktopItemHtml(LPCWSTR pwszFileName,pointer pcomp, DWORD dwReserved),HRESULT
	stdmethod AddUrl(HWND hwnd,LPCWSTR pszSource,pointer pcomp, DWORD dwFlags),HRESULT
	stdmethod GetDesktopItemBySource(LPCWSTR pwszSource,pointer pcomp, DWORD dwReserved),HRESULT
endinterface

' Flags for SetSafeMode
$define SSM_CLEAR	0x0000
$define SSM_SET	0x0001
$define SSM_REFRESH 0x0002
$define SSM_UPDATE	0x0004

' Flags for Set/GetScheme
$define SCHEME_DISPLAY	0x0001
$define SCHEME_EDIT	0x0002
$define SCHEME_LOCAL	0x0004
$define SCHEME_GLOBAL	0x0008
$define SCHEME_REFRESH	0x0010
$define SCHEME_UPDATE	0x0020
$define SCHEME_DONOTUSE 0x0040 ' used to be SCHEME_ENUMERATE; no longer supported
$define SCHEME_CREATE	0x0080

interface IActiveDesktopP ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IActiveDesktopP
	stdmethod SetSafeMode(DWORD dwFlags),HRESULT
	stdmethod EnsureUpdateHTML(),HRESULT
	stdmethod SetScheme(LPCWSTR pwszSchemeName, DWORD dwFlags),HRESULT
	stdmethod GetScheme(LPWSTR pwszSchemeName,pointer lpdwcchBuffer, DWORD dwFlags),HRESULT
endinterface

'Flags for GetObjectFlags
$define GADOF_DIRTY	0x00000001

interface IADesktopP2 ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IADesktopP2
	stdmethod ReReadWallpaper(),HRESULT
	stdmethod GetADObjectFlags(pointer lpdwFlags, DWORD dwMask),HRESULT
	stdmethod UpdateAllDesktopSubscriptions(),HRESULT
	stdmethod MakeDynamicChanges(comref pOleObj),HRESULT
endinterface

$endif
' _WININET_
'===============================================================/

$define MAX_COLUMN_NAME_LEN 80
$define MAX_COLUMN_DESC_LEN 128

'$include <pshpack1.h>

type SHCOLUMNINFO,1
	SHCOLUMNID	scid	' the unique identifier of this column
	VARTYPE	vt	' the native type of the data returned
	DWORD	fmt	' this listview format(LVCFMT_LEFT, usually)
	UINT	cChars	' the default width of the column, characters
	DWORD	csFlags	' SHCOLSTATE flags
	iwstring wszTitle[MAX_COLUMN_NAME_LEN]	' the title of the column
	iwstring wszDescription[MAX_COLUMN_DESC_LEN]	' full description of this column
endtype

'$include <poppack.h>	/* Return to default */

'$include <pshpack8.h>

type SHCOLUMNINIT
	ULONG	dwFlags	' initialization flags
	ULONG	dwReserved	' reserved for future use.
	iwstring wszFolder[MAX_PATH]	' fully qualified folder path(or empty if multiple folders)
endtype

$define SHCDF_UPDATEITEM	0x00000001	' this flag is a hint that the file has changed since the last call to GetItemData

type SHCOLUMNDATA
	ULONG	dwFlags	' combination of SHCDF_ flags.
	DWORD	dwFileAttributes	' file attributes.
	ULONG	dwReserved	' reserved for future use.
	pointer	pwszExt	' address of file name extension
	iwstring wszFile[MAX_PATH]	' Absolute path of file.
endtype

'$include <poppack.h>	/* Return to byte packing */

' Note: these objects must be threadsafe!	GetItemData _will_ be called
' simultaneously from multiple threads.
interface IColumnProvider ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IColumnProvider
	stdmethod Initialize(pointer psci),HRESULT
	stdmethod GetColumnInfo(DWORD dwIndex,pointer psci),HRESULT
	stdmethod GetItemData(pointer pscid,pointer pscd,pointer pvarData),HRESULT
endinterface

'/////////////////////////////////////////////////////
' Drag and Drop helper
' Purpose: To expose the Shell drag images
' This interface is implemented the shell by CLSID_DragDropHelper.
' To use:
'	If you are the source of a drag(i.e. response to LV_DRAGBEGIN or
'	equivelent begin drag message)call
'	IDragSourceHelper::InitializeFromWindow
'	(<hwnd of window supporting DI_GETDRAGIMAGE>,
'	<pointer to POINT indicating offset to the mouse from
'	the upper left corner of the image>,
'	<pointer to data object>)
'	NOTE: The Data object must support IDataObject::SetData with multiple
'	data types and _GetData must implement data type cloning
'	(Including HGLOBAL), not just aliasing.
'	If you wish to have an image while over your application add the
'	IDragImages::Dr* calls to your IDropTarget implementation. For Example:
'	STDMETHODIMP CUserDropTarget::DragEnter(pointer pDataObject,
'	DWORD grfKeyState,
'	POINTL pt, pointer pdwEffect)
'	Process your DragEnter
'	Call IDragImages::DragEnter last.
'	_pDropTargetHelper->DragEnter(_hwndDragOver, pDataObject,
'	(pointer)&pt,pointer pdwEffect)
'	return hres
'	}
'	If you wish to be able to source a drag image from a custom control,
'	implement a handler for the RegisterWindowMessage(DI_GETDRAGIMAGE).
'	The LPARAM is a pointer to an SHDRAGIMAGE structure.
'	sizeDragImage	-	Calculate the length and width required to render
'	the images.
'	ptOffset	-	Calculate the offset from the upper left corner to
'	the mouse cursor within the image
'	hbmpDragImage	-	CreateBitmap(sizeDragImage.cx, sizeDragImage.cy,
'	GetDeviceCaps(hdcScreen, PLANES),
'	GetDeviceCaps(hdcScreen, BITSPIXEL),
'	NULL)
'	Drag Images will only be displayed on Windows NT 5.0 or later.
'	Note about IDropTargetHelper::Show - This method is provided for
'	showing/hiding the Drag image low color depth video modes. When
'	painting to a window that is currently being dragged over(i.e. For
'	indicating a selection)you need to hide the drag image by calling this
'	method passing FALSE. After the window is done painting, Show the image
'	again by passing TRUE.

'$include <pshpack8.h>

type SHDRAGIMAGE
	SIZE	sizeDragImage	' - The length and Width of the rendered image
	POINT	ptOffset	' - The Offset from the mouse cursor to the upper left corner of the image
	HBITMAP	hbmpDragImage	' - The Bitmap containing the rendered drag images
	COLORREF	crColorKey	' - The COLORREF that has been blitted to the background of the images
endtype

'$include <poppack.h>	/* Return to byte packing */

' This is sent to a window to get the rendered images to a bitmap
' Call RegisterWindowMessage to get the ID
$define DI_GETDRAGIMAGE "ShellGetDragImage"

interface IDropTargetHelper ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IDropTargetHelper
	stdmethod DragEnter(HWND hwndTarget, IDataObject pDataObject, pointer ppt, DWORD dwEffect),HRESULT
	stdmethod DragLeave(),HRESULT
	stdmethod DragOver(pointer ppt, DWORD dwEffect),HRESULT
	stdmethod Drop(IDataObject pDataObject, pointer ppt, DWORD dwEffect),HRESULT
	stdmethod Show(BOOL fShow),HRESULT
endinterface

interface IDragSourceHelper ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IDragSourceHelper
	stdmethod InitializeFromBitmap(pointer pshdi, IDataObject pDataObject),HRESULT
	stdmethod InitializeFromWindow(HWND hwnd, pointer ppt, IDataObject pDataObject),HRESULT
endinterface

'==========================================================================
' Clipboard format which may be supported by IDataObject from system
' defined shell folders(such as directories, network, ...).
'==========================================================================

$define CFSTR_SHELLIDLIST	"Shell IDList Array"	' CF_IDLIST
$define CFSTR_SHELLIDLISTOFFSET	"Shell Object Offsets"	' CF_OBJECTPOSITIONS
$define CFSTR_NETRESOURCES	"Net Resource"	' CF_NETRESOURCE
$define CFSTR_FILEDESCRIPTORA	"FileGroupDescriptor"	' CF_FILEGROUPDESCRIPTORA
$define CFSTR_FILEDESCRIPTORW	"FileGroupDescriptorW"	' CF_FILEGROUPDESCRIPTORW
$define CFSTR_FILECONTENTS	"FileContents"	' CF_FILECONTENTS
$define CFSTR_FILENAMEA	"FileName"	' CF_FILENAMEA
$define CFSTR_FILENAMEW	"FileNameW"	' CF_FILENAMEW
$define CFSTR_PRINTERGROUP	"PrinterFriendlyName"	' CF_PRINTERS
$define CFSTR_FILENAMEMAPA	"FileNameMap"	' CF_FILENAMEMAPA
$define CFSTR_FILENAMEMAPW	"FileNameMapW"	' CF_FILENAMEMAPW
$define CFSTR_SHELLURL	"UniformResourceLocator"
$define CFSTR_INETURLA	"UniformResourceLocator"
$define CFSTR_INETURLW	"UniformResourceLocatorW"
$define CFSTR_PREFERREDDROPEFFECT	"Preferred DropEffect"
$define CFSTR_PERFORMEDDROPEFFECT	"Performed DropEffect"
$define CFSTR_PASTESUCCEEDED	"Paste Succeeded"
$define CFSTR_INDRAGLOOP	"InShellDragLoop"
$define CFSTR_DRAGCONTEXT	"DragContext"
$define CFSTR_MOUNTEDVOLUME	"MountedVolume"
$define CFSTR_PERSISTEDDATAOBJECT	"PersistedDataObject"
$define CFSTR_TARGETCLSID	"TargetCLSID"	' HGLOBAL with a CLSID of the drop target
$define CFSTR_LOGICALPERFORMEDDROPEFFECT "Logical Performed DropEffect"
$define CFSTR_AUTOPLAY_SHELLIDLISTS	"Autoplay Enumerated IDList Array"	'	(HGLOBAL with LPIDA)

$ifdef UNICODE
$define CFSTR_FILEDESCRIPTOR	"FileGroupDescriptorW"
$define CFSTR_FILENAME	"FileNameW"
$define CFSTR_FILENAMEMAP	"FileNameMapW"
$define CFSTR_INETURL	"UniformResourceLocatorW"
$else
$define CFSTR_FILEDESCRIPTOR	"FileGroupDescriptor"
$define CFSTR_FILENAME	"FileName"
$define CFSTR_FILENAMEMAP	"FileNameMap"
$define CFSTR_INETURL	"UniformResourceLocator"
$endif

$define DVASPECT_SHORTNAME	2 ' use for CF_HDROP to get short name version of file paths
$define DVASPECT_COPY	3 ' use to indicate format is a "Copy" of the data(FILECONTENTS, FILEDESCRIPTOR, etc)
$define DVASPECT_LINK	4 ' use to indicate format is a "Shortcut" to the data(FILECONTENTS, FILEDESCRIPTOR, etc)

'$include <pshpack8.h>
' format of CF_NETRESOURCE
$ifndef __winnetwk_inc__
type NETRESOURCE
	DWORD	dwScope
	DWORD	dwType
	DWORD	dwDisplayType
	DWORD	dwUsage
	LPSTR	lpLocalName
	LPSTR	lpRemoteName
	LPSTR	lpComment
	LPSTR	lpProvider
endtype
$endif
type NRESARRAY
	UINT cItems
	NETRESOURCE nr[1]
endtype
'$include <poppack.h>	/* Return to byte packing */

' format of CF_IDLIST
type CIDA
	UINT cidl	' number of relative IDList
	UINT aoffset[1]	' [0]: folder IDList, [1]-[cidl]: item IDList
endtype

' FILEDESCRIPTOR.dwFlags field indicate which fields are to be used
typedef FD_FLAGS int
$define FD_CLSID	0x0001
$define FD_SIZEPOINT	0x0002
$define FD_ATTRIBUTES	0x0004
$define FD_CREATETIME	0x0008
$define FD_ACCESSTIME	0x0010
$define FD_WRITESTIME	0x0020
$define FD_FILESIZE	0x0040
$define FD_PROGRESSUI	0x4000	' Show Progress UI w/Drag and Drop
$define FD_LINKUI	0x8000	' 'link' UI is prefered

type FILEDESCRIPTORA
	DWORD dwFlags

	CLSID clsid
	SIZEL sizel
	POINTL pointl

	DWORD dwFileAttributes
	FILETIME ftCreationTime
	FILETIME ftLastAccessTime
	FILETIME ftLastWriteTime
	DWORD nFileSizeHigh
	DWORD nFileSizeLow
	istring	cFileName[ MAX_PATH ]
endtype

type FILEDESCRIPTORW
	DWORD dwFlags

	CLSID clsid
	SIZEL sizel
	POINTL pointl

	DWORD dwFileAttributes
	FILETIME ftCreationTime
	FILETIME ftLastAccessTime
	FILETIME ftLastWriteTime
	DWORD nFileSizeHigh
	DWORD nFileSizeLow
	iwstring cFileName[ MAX_PATH ]
endtype
$ifdef UNICODE
type FILEDESCRIPTOR
	DWORD dwFlags

	CLSID clsid
	SIZEL sizel
	POINTL pointl

	DWORD dwFileAttributes
	FILETIME ftCreationTime
	FILETIME ftLastAccessTime
	FILETIME ftLastWriteTime
	DWORD nFileSizeHigh
	DWORD nFileSizeLow
	iwstring cFileName[ MAX_PATH ]
endtype
$else
type FILEDESCRIPTOR
	DWORD dwFlags

	CLSID clsid
	SIZEL sizel
	POINTL pointl

	DWORD dwFileAttributes
	FILETIME ftCreationTime
	FILETIME ftLastAccessTime
	FILETIME ftLastWriteTime
	DWORD nFileSizeHigh
	DWORD nFileSizeLow
	istring	cFileName[ MAX_PATH ]
endtype
$endif

' format of CF_FILEGROUPDESCRIPTOR
type FILEGROUPDESCRIPTORA
	UINT cItems
	FILEDESCRIPTOR fgd[1]
endtype

type FILEGROUPDESCRIPTORW
	UINT cItems
	FILEDESCRIPTORW fgd[1]
endtype
$ifdef UNICODE
type FILEGROUPDESCRIPTOR
	UINT cItems
	FILEDESCRIPTORW fgd[1]
endtype
$else
type FILEGROUPDESCRIPTOR
	UINT cItems
	FILEDESCRIPTOR fgd[1]
endtype
$endif

' format of CF_HDROP and CF_PRINTERS, the HDROP case the data that follows
' is a double null terinated list of file names, for printers they are printer
' friendly names
type DROPFILES
	DWORD pFiles	' offset of file list
	POINT pt	' drop point(client coords)
	BOOL fNC	' is it on NonClient area
		' and pt is screen coords
	BOOL fWide	' WIDE character switch
endtype

'====== File System Notification APIs ===============================
type SHChangeNotifyEntry
	pointer pidl
	BOOL	fRecursive
endtype

'	File System Notification flags

$define SHCNE_RENAMEITEM	0x00000001
$define SHCNE_CREATE	0x00000002
$define SHCNE_DELETE	0x00000004
$define SHCNE_MKDIR	0x00000008
$define SHCNE_RMDIR	0x00000010
$define SHCNE_MEDIAINSERTED	0x00000020
$define SHCNE_MEDIAREMOVED	0x00000040
$define SHCNE_DRIVEREMOVED	0x00000080
$define SHCNE_DRIVEADD	0x00000100
$define SHCNE_NETSHARE	0x00000200
$define SHCNE_NETUNSHARE	0x00000400
$define SHCNE_ATTRIBUTES	0x00000800
$define SHCNE_UPDATEDIR	0x00001000
$define SHCNE_UPDATEITEM	0x00002000
$define SHCNE_SERVERDISCONNECT	0x00004000
$define SHCNE_UPDATEIMAGE	0x00008000
$define SHCNE_DRIVEADDGUI	0x00010000
$define SHCNE_RENAMEFOLDER	0x00020000
$define SHCNE_FREESPACE	0x00040000

' SHCNE_EXTENDED_EVENT: the extended event is identified dwItem1,
' packed LPITEMIDLIST format(same as SHCNF_DWORD packing).
' Additional information can be passed the dwItem2 parameter
' of SHChangeNotify(called "pidl2" below), which if present, must also
' be LPITEMIDLIST format.
' Unlike the standard events, the extended events are ORDINALs, so we
' don't run of bits.	Extended events follow the SHCNEE_* naming
' convention.
' The dwItem2 parameter varies according to the extended event.

$define SHCNE_EXTENDED_EVENT	0x04000000

$define SHCNE_ASSOCCHANGED	0x08000000

$define SHCNE_DISKEVENTS	0x0002381F
$define SHCNE_GLOBALEVENTS	0x0C0581E0 ' Events that dont match pidls first
$define SHCNE_ALLEVENTS	0x7FFFFFFF
$define SHCNE_INTERRUPT	0x80000000 ' The presence of this flag indicates
		' that the event was generated by an
		' interrupt.	It is stripped before
		' the clients of SHCNNotify_ see it.

' SHCNE_EXTENDED_EVENT extended events.	These events are ordinals.
' This is not a bitfield.

$define SHCNEE_ORDERCHANGED	2	' pidl2 is the changed folder
$define SHCNEE_MSI_CHANGE	4	' pidl2 is a SHChangeProductKeyAsIDList
$define SHCNEE_MSI_UNINSTALL	5	' pidl2 is a SHChangeProductKeyAsIDList

' Flags
' uFlags & SHCNF_TYPE is an ID which indicates what dwItem1 and dwItem2 mean
$define SHCNF_IDLIST	0x0000	' LPITEMIDLIST
$define SHCNF_PATHA	0x0001	' path name
$define SHCNF_PRINTER	0x0002	' printer friendly name
$define SHCNF_DWORD	0x0003	' DWORD
$define SHCNF_PATHW	0x0005	' path name
$define SHCNF_PRINTERW	0x0006	' printer friendly name
$define SHCNF_TYPE	0x00FF
$define SHCNF_FLUSH	0x1000
$define SHCNF_FLUSHNOWAIT 0x2000

'	APIs
declare import, SHChangeNotify(LONG wEventId, UINT uFlags,pointer dwItem1,pointer dwItem2)

' IShellChangeNotify

' IShellChangeNotify

interface IShellChangeNotify ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellChangeNotify
	stdmethod OnChange(LONG lEvent,pointer pidl1,pointer pidl2),HRESULT
endinterface

' IQueryInfo
'-------------------------------------------------------------------------
' IQueryInfo interface
' [Methods]
'	::GetInfoTip()
'-------------------------------------------------------------------------

' IQueryInfo

interface IQueryInfo ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IQueryInfo
	stdmethod GetInfoTip(DWORD dwFlags,pointer ppwszTip),HRESULT
	stdmethod GetInfoFlags(pointer pdwFlags),HRESULT
endinterface

$define QITIPF_DEFAULT	0x00000000
$define QITIPF_USENAME	0x00000001
$define QITIPF_LINKNOTARGET	0x00000002
$define QITIPF_LINKUSETARGET	0x00000004
$define QITIPF_USESLOWTIP	0x00000008	' Flag says it's OK to take a long time generating tip

$define QIF_CACHED	0x00000001
$define QIF_DONTEXPANDFOLDER 0x00000002

' SHAddToRecentDocs
$define SHARD_PIDL	0x00000001
$define SHARD_PATHA	0x00000002
$define SHARD_PATHW	0x00000003

$ifdef UNICODE
$define SHARD_PATH SHARD_PATHW
$else
$define SHARD_PATH SHARD_PATHA
$endif

declare import, SHAddToRecentDocs(UINT uFlags,pointer pv)

type SHChangeDWORDAsIDList
	USHORT	cb
	DWORD	dwItem1
	DWORD	dwItem2
	USHORT	cbZero
endtype

type SHChangeUpdateImageIDList
	USHORT cb
	int iIconIndex
	int iCurIndex
	UINT uFlags
	DWORD dwProcessID
	iwstring szName[MAX_PATH]
	USHORT cbZero
endtype

declare import, SHHandleUpdateImage(pointer pidlExtra),int

type SHChangeProductKeyAsIDList
	USHORT cb
	iwstring wszProductKey[39]
	USHORT cbZero
endtype

declare import, SHUpdateImageA(LPCSTR pszHashItem, int iIndex, UINT uFlags, int iImageIndex)
declare import, SHUpdateImageW(LPCWSTR pszHashItem, int iIndex, UINT uFlags, int iImageIndex)
$ifdef UNICODE
declare import, SHUpdateImage alias SHUpdateImageW(LPCWSTR pszHashItem, int iIndex, UINT uFlags, int iImageIndex)
$else
declare import, SHUpdateImage alias SHUpdateImageA(LPCSTR pszHashItem, int iIndex, UINT uFlags, int iImageIndex)
$endif

declare import, SHChangeNotifyRegister(HWND hwnd, int fSources, LONG fEvents, UINT wMsg, int cEntries,pointer pshcne),ULONG
declare import, SHChangeNotifyDeregister(UINT ulID),BOOL
declare import, SHChangeNotification_Lock(HANDLE hChangeNotification, DWORD dwProcessId,pointer pppidl,pointer plEvent),HANDLE
declare import, SHChangeNotification_Unlock(HANDLE hLock),BOOL
' The pidls that are given to the view via the ChangeNotifyEvents are simple Pidls,
' SHGetRealIDL()will convert them to true PIDLs.
declare import, SHGetRealIDL(IShellFolder psf,pointer pidlSimple,pointer ppidlReal),HRESULT

declare import, SHGetInstanceExplorer(pointer ppunk),HRESULT

' SHGetDataFromIDListA/W
' SHGetDataFromIDList nFormat values TCHAR
$define SHGDFIL_FINDDATA	1
$define SHGDFIL_NETRESOURCE	2
$define SHGDFIL_DESCRIPTIONID	3

$define SHDID_ROOT_REGITEM	1
$define SHDID_FS_FILE	2
$define SHDID_FS_DIRECTORY	3
$define SHDID_FS_OTHER	4
$define SHDID_COMPUTER_DRIVE35	5
$define SHDID_COMPUTER_DRIVE525	6
$define SHDID_COMPUTER_REMOVABLE	7
$define SHDID_COMPUTER_FIXED	8
$define SHDID_COMPUTER_NETDRIVE	9
$define SHDID_COMPUTER_CDROM	10
$define SHDID_COMPUTER_RAMDISK	11
$define SHDID_COMPUTER_OTHER	12
$define SHDID_NET_DOMAIN	13
$define SHDID_NET_SERVER	14
$define SHDID_NET_SHARE	15
$define SHDID_NET_RESTOFNET	16
$define SHDID_NET_OTHER	17
$define SHDID_COMPUTER_IMAGING	18
$define SHDID_COMPUTER_AUDIO	19
$define SHDID_COMPUTER_SHAREDDOCS	20

'$include <pshpack8.h>

type SHDESCRIPTIONID
	DWORD	dwDescriptionId
	CLSID	clsid
endtype

'$include <poppack.h>	/* Return to byte packing */

' these delegate to IShellFolder2::GetItemData()

declare import, SHGetDataFromIDListA(IShellFolder psf,pointer pidl, int nFormat,pointer pv, int cb),HRESULT
declare import, SHGetDataFromIDListW(IShellFolder psf,pointer pidl, int nFormat,pointer pv, int cb),HRESULT
$ifdef UNICODE
declare import, SHGetDataFromIDList alias SHGetDataFromIDListW(IShellFolder psf,pointer pidl, int nFormat,pointer pv, int cb),HRESULT
$else
declare import, SHGetDataFromIDList alias SHGetDataFromIDListA(IShellFolder psf,pointer pidl, int nFormat,pointer pv, int cb),HRESULT
$endif

'===========================================================================

' PathResolve flags
$define PRF_VERIFYEXISTS	0x0001
$define PRF_TRYPROGRAMEXTENSIONS	(0x0002 | PRF_VERIFYEXISTS)
$define PRF_FIRSTDIRDEF	0x0004
$define PRF_DONTFINDLNK	0x0008	' if PRF_TRYPROGRAMEXTENSIONS is specified
declare import, RestartDialog(HWND hwnd,LPCWSTR lpPrompt, DWORD dwReturn),int
declare import, RestartDialogEx(HWND hwnd,LPCWSTR lpPrompt, DWORD dwReturn, DWORD dwReasonCode),int

declare import, SHCoCreateInstance(LPCWSTR pszCLSID,pointer pclsid,comref pUnkOuter,pointer riid,pointer ppv)
' For CallCPLEntry16
typedef FARPROC16 UINT
declare import, CallCPLEntry16(HINSTANCE hinst, FARPROC16 lpfnEntry, HWND hwndCPL, UINT msg, LPARAM lParam1, LPARAM lParam2),LRESULT
declare import, SHCreateStdEnumFmtEtc(UINT cfmt,pointer afmt, pointer ppenumFormatEtc),HRESULT
declare import, SHDoDragDrop(HWND hwnd,comref pdata,comref pdsrc, DWORD dwEffect,pointer pdwEffect),HRESULT
' stuff for doing auto scrolling
$define NUM_POINTS	3
type AUTO_SCROLL_DATA	' asd
	int iNextSample
	DWORD dwLastScroll
	BOOL bFull
	POINT pts[NUM_POINTS]
	DWORD dwTimes[NUM_POINTS]
endtype

declare import, DAD_SetDragImage(HIMAGELIST him,pointer pptOffset),BOOL
declare import, DAD_DragEnterEx(HWND hwndTarget,pointer ptStart),BOOL
declare import, DAD_DragEnterEx2(HWND hwndTarget,pointer ptStart,comref pdtObject),BOOL
declare import, DAD_ShowDragImage(BOOL fShow),BOOL
declare import, DAD_DragMove(pointer pt),BOOL
declare import, DAD_DragLeave(),BOOL
declare import, DAD_AutoScroll(HWND hwnd, pointer pad,pointer pptNow),BOOL

type CABINETSTATE
	USHORT cLength
	USHORT nVersion

/*	BOOL fFullPathTitle	: 1
	BOOL fSaveLocalView	: 1
	BOOL fNotShell	: 1
	BOOL fSimpleDefault	: 1
	BOOL fDontShowDescBar	: 1
	BOOL fNewWindowMode	: 1
	BOOL fShowCompColor	: 1;	' NT: Show compressed volumes a different colour
	BOOL fDontPrettyNames	: 1;	' NT: Do 8.3 name conversion, or not!
	BOOL fAdminsCreateCommonGroups : 1;	' NT: Administrators create comon groups*/
	BOOL fFullPathTitle
/*	UINT fUnusedFlags : 7;*/
	UINT fUnusedFlags
	UINT fMenuEnumFilter

endtype

$define CABINETSTATE_VERSION 2

' APIs for reading and writing the cabinet state.
declare import, ReadCabinetState(pointer lpState, int iSize),BOOL
declare import, WriteCabinetState(pointer lpState),BOOL
declare import, PathMakeUniqueName(LPWSTR pszUniqueName, UINT cchMax,LPCWSTR pszTemplate,LPCWSTR pszLongPlate,pointer pszDir),BOOL
declare import, PathQualify(LPWSTR psz)
declare import, PathIsExe(LPCWSTR pszPath),BOOL
declare import, PathIsSlowA(pointer pszFile, DWORD dwAttr),BOOL
declare import, PathIsSlowW(pointer pszFile, DWORD dwAttr),BOOL

$ifdef UNICODE
declare import, PathIsSlow alias PathIsSlowW(LPCWSTR pszFile, DWORD dwAttr),BOOL
$else
declare import, PathIsSlow alias PathIsSlowA(LPCSTR pszFile, DWORD dwAttr),BOOL
$endif
'	Return codes from PathCleanupSpec.	Negative return values are
'	unrecoverable errors
$define PCS_FATAL	0x80000000
$define PCS_REPLACEDCHAR	0x00000001
$define PCS_REMOVEDCHAR	0x00000002
$define PCS_TRUNCATED	0x00000004
$define PCS_PATHTOOLONG	0x00000008	' Always combined with FATAL

declare import, PathCleanupSpec(LPCWSTR pszDir,LPWSTR pszSpec),int
declare import, PathResolve(LPWSTR pszPath,LPCWSTR dirs, UINT fFlags),int
declare import, GetFileNameFromBrowse(HWND hwnd,LPWSTR pszFilePath, UINT cbFilePath,LPCWSTR pszWorkingDir,LPCWSTR pszDefExt,LPCWSTR pszFilters,LPCWSTR pszTitle),BOOL
declare import, DriveType(int iDrive),int

declare import, RealDriveType(int iDrive, BOOL fOKToHitNet),int
declare import, IsNetDrive(int iDrive),int
' Flags for Shell_MergeMenus
$define MM_ADDSEPARATOR	0x00000001
$define MM_SUBMENUSHAVEIDS	0x00000002
$define MM_DONTREMOVESEPS	0x00000004

declare import, Shell_MergeMenus(HMENU hmDst, HMENU hmSrc, UINT uInsert, UINT uIDAdjust, UINT uIDAdjustMax, ULONG uFlags),UINT

/*
*	The SHObjectProperties API provides an easy way to invoke
*	the Properties context menu command on shell objects.
*
*	PARAMETERS
*
*	hwnd	The window handle of the window which will own the dialog
*	dwType	A SHOP_ value as defined below
*	lpObject	Name of the object, see SHOP_ values below
*	lpPage	The name of the property sheet page to open to or NULL.
*
*	RETURN
*
*	TRUE if the Properties command was invoked
 */
declare import, SHObjectProperties(HWND hwnd, DWORD dwType,LPCWSTR lpObject,LPCWSTR lpPage),BOOL

$define SHOP_PRINTERNAME 0x00000001	' lpObject points to a printer friendly name
$define SHOP_FILEPATH	0x00000002	' lpObject points to a fully qualified path+file name
$define SHOP_VOLUMEGUID	0x00000004	' lpObject points to a Volume GUID

/*
*	The SHFormatDrive API provides access to the Shell
*	format dialog. This allows apps which want to format disks
*	to bring up the same dialog that the Shell does to do it.
*
*	This dialog is not sub-classable. You cannot put custom
*	controls it. If you want this ability, you will have
*	to write your own front end for the DMaint_FormatDrive
*	engine.
*
*	NOTE that the user can format as many diskettes the specified
*	drive, or as many times, as he/she wishes to. There is no way to
*	force any specififc number of disks to format. If you want this
*	ability, you will have to write your own front end for the
*	DMaint_FormatDrive engine.
*
*	NOTE also that the format will not start till the user pushes the
*	start button the dialog. There is no way to do auto start. If
*	you want this ability, you will have to write your own front end
*	for the DMaint_FormatDrive engine.
*
*	PARAMETERS
*
*	hwnd	= The window handle of the window which will own the dialog
*	NOTE that unlike SHCheckDrive, hwnd == NULL does not cause
*	this dialog to come up as a "top level application" window.
*	This parameter should always be non-null, this dialog is
*	only designed to be the child of another window, not a
*	stand-alone application.
*	drive	= The 0 based(A: == 0)drive number of the drive to format
*	fmtID	= The ID of the physical format to format the disk with
*	NOTE: The special value SHFMT_ID_DEFAULT means "use the
*	default format specified by the DMaint_FormatDrive
*	engine". If you want to FORCE a particular format
*	ID "up front" you will have to call
*	DMaint_GetFormatOptions yourself before calling
*	this to obtain the valid list of phys format IDs
*	(contents of the PhysFmtIDList array the
*	FMTINFOSTRUCT).
*	options = There is currently only two option bits defined
*
*	SHFMT_OPT_FULL
*	SHFMT_OPT_SYSONLY
*
*	The normal defualt the Shell format dialog is
*	"Quick Format", setting this option bit indicates that
*	the caller wants to start with FULL format selected
*	(this is useful for folks detecting "unformatted" disks
*	and wanting to bring up the format dialog).
*
*	The SHFMT_OPT_SYSONLY initializes the dialog to
*	default to just sys the disk.
*
*	All other bits are reserved for future expansion and
*	must be 0.
*
*	Please note that this is a bit field and not a value
*	and treat it accordingly.
*
*	RETURN
*	The return is either one of the SHFMT_* values, or if the
*	returned DWORD value is not == to one of these values, then
*	the return is the physical format ID of the last succesful
*	format. The LOWORD of this value can be passed on subsequent
*	calls as the fmtID parameter to "format the same type you did
*	last time".
*
 */
declare import, SHFormatDrive(HWND hwnd, UINT drive, UINT fmtID, UINT options),DWORD
' Special value of fmtID which means "use the default format"
$define SHFMT_ID_DEFAULT	0xFFFF

' Option bits for options parameter
$define SHFMT_OPT_FULL	0x0001
$define SHFMT_OPT_SYSONLY	0x0002

' Special return values. PLEASE NOTE that these are DWORD values.
$define SHFMT_ERROR	0xFFFFFFFF	' Error on last format, drive may be formatable
$define SHFMT_CANCEL	0xFFFFFFFE	' Last format was canceled
$define SHFMT_NOFORMAT	0xFFFFFFFD	' Drive is not formatable

$ifndef HPSXA_DEFINED
$define HPSXA_DEFINED
typedef HPSXA UINT

declare import, SHCreatePropSheetExtArray(HKEY hKey,LPCWSTR pszSubKey, UINT max_iface),HPSXA
declare import, SHDestroyPropSheetExtArray(HPSXA hpsxa)
declare import, SHAddFromPropSheetExtArray(HPSXA hpsxa,pointer lpfnAddPage, LPARAM lParam),UINT
declare import, SHReplaceFromPropSheetExtArray(HPSXA hpsxa, UINT uPageID,pointer lpfnReplaceWith, LPARAM lParam),UINT
declare import, ILClone(pointer pidl),pointer
declare import, ILGetNext(pointer pidl),pointer
declare import, ILGetSize(pointer pidl),UINT
declare import, ILFindLastID(pointer pidl),pointer
declare import, ILRemoveLastID(pointer pidl),BOOL
declare import, ILAppendID(pointer pidl,pointer pmkid, BOOL fAppend),pointer
declare import, ILFree(pointer pidl),pointer
declare import, ILCloneFirst(pointer pidl),pointer
declare import, ILIsEqual(pointer pidl1,pointer pidl2),BOOL
declare import, ILIsParent(pointer pidl1,pointer pidl2, BOOL fImmediate),BOOL
declare import, ILFindChild(pointer pidlParent,pointer pidlChild),pointer
declare import, ILCombine(pointer pidl1,pointer pidl2),pointer
declare import, ILLoadFromStream(comref pstm,pointer pidl),HRESULT
declare import, ILSaveToStream(comref pstm,pointer pidl),HRESULT

declare import, ILCreateFromPathA(LPCSTR pszPath),pointer
declare import, ILCreateFromPathW(LPCWSTR pszPath),pointer
$ifdef UNICODE
declare import, ILCreateFromPath alias ILCreateFromPathW(LPCWSTR pszPath),pointer
$else
declare import, ILCreateFromPath alias ILCreateFromPathA(LPCSTR pszPath),pointer
$endif

declare import, SHILCreateFromPath(LPCWSTR szPath,pointer ppidl,pointer rgfInOut),HRESULT

'	IDefViewFrame
interface IDefViewFrame ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IDefViewFrame
	stdmethod GetWindowLV(pointer phwnd),HRESULT
	stdmethod ReleaseWindowLV(),HRESULT
	stdmethod GetShellFolder(pointer ppsf),HRESULT
endinterface

'===========================================================================
' Shell restrictions.(Parameter for SHRestricted)
typedef RESTRICTIONS int

$define REST_NONE	0x00000000
$define REST_NORUN	0x00000001
$define REST_NOCLOSE	0x00000002
$define REST_NOSAVESET	0x00000004
$define REST_NOFILEMENU	0x00000008
$define REST_NOSETFOLDERS	0x00000010
$define REST_NOSETTASKBAR	0x00000020
$define REST_NODESKTOP	0x00000040
$define REST_NOFIND	0x00000080
$define REST_NODRIVES	0x00000100
$define REST_NODRIVEAUTORUN	0x00000200
$define REST_NODRIVETYPEAUTORUN	0x00000400
$define REST_NONETHOOD	0x00000800
$define REST_STARTBANNER	0x00001000
$define REST_RESTRICTRUN	0x00002000
$define REST_NOPRINTERTABS	0x00004000
$define REST_NOPRINTERDELETE	0x00008000
$define REST_NOPRINTERADD	0x00010000
$define REST_NOSTARTMENUSUBFOLDERS	0x00020000
$define REST_MYDOCSONNET	0x00040000
$define REST_NOEXITTODOS	0x00080000
$define REST_ENFORCESHELLEXTSECURITY	0x00100000
$define REST_LINKRESOLVEIGNORELINKINFO	0x00200000
$define REST_NOCOMMONGROUPS	0x00400000
$define REST_SEPARATEDESKTOPPROCESS	0x00800000
$define REST_NOWEB	0x01000000
$define REST_NOTRAYCONTEXTMENU	0x02000000
$define REST_NOVIEWCONTEXTMENU	0x04000000
$define REST_NONETCONNECTDISCONNECT	0x08000000
$define REST_STARTMENULOGOFF	0x10000000
$define REST_NOSETTINGSASSIST	0x20000000
$define REST_NOINTERNETICON	0x40000001
$define REST_NORECENTDOCSHISTORY	0x40000002
$define REST_NORECENTDOCSMENU	0x40000003
$define REST_NOACTIVEDESKTOP	0x40000004
$define REST_NOACTIVEDESKTOPCHANGES	0x40000005
$define REST_NOFAVORITESMENU	0x40000006
$define REST_CLEARRECENTDOCSONEXIT	0x40000007
$define REST_CLASSICSHELL	0x40000008
$define REST_NOCUSTOMIZEWEBVIEW	0x40000009
$define REST_NOHTMLWALLPAPER	0x40000010
$define REST_NOCHANGINGWALLPAPER	0x40000011
$define REST_NODESKCOMP	0x40000012
$define REST_NOADDDESKCOMP	0x40000013
$define REST_NODELDESKCOMP	0x40000014
$define REST_NOCLOSEDESKCOMP	0x40000015
$define REST_NOCLOSE_DRAGDROPBAND	0x40000016	' Disable Close and Drag & Drop on ALL Bands
$define REST_NOMOVINGBAND	0x40000017	' Disable Moving ALL Bands
$define REST_NOEDITDESKCOMP	0x40000018
$define REST_NORESOLVESEARCH	0x40000019
$define REST_NORESOLVETRACK	0x4000001A
$define REST_FORCECOPYACLWITHFILE	0x4000001B
$define REST_NOLOGO3CHANNELNOTIFY	0x4000001C
$define REST_NOFORGETSOFTWAREUPDATE	0x4000001D
$define REST_NOSETACTIVEDESKTOP	0x4000001E	' No Active desktop on Settings Menu
$define REST_NOUPDATEWINDOWS	0x4000001F	' No Windows Update on Settings Menu
$define REST_NOCHANGESTARMENU	0x40000020	' No Context menu or Drag and Drop on Start menu
$define REST_NOFOLDEROPTIONS	0x40000021	' No Folder Options on Settings Menu
$define REST_HASFINDCOMPUTERS	0x40000022	' Show Start/Search/Computers
$define REST_INTELLIMENUS	0x40000023
$define REST_RUNDLGMEMCHECKBOX	0x40000024
$define REST_ARP_ShowPostSetup	0x40000025	' ARP: Show Post-Setup page
$define REST_NOCSC	0x40000026	' Disable the ClientSide caching on SM
$define REST_NOCONTROLPANEL	0x40000027	' Remove the Control Panel only from SM|Settings
$define REST_ENUMWORKGROUP	0x40000028	' Enumerate workgroup root of nethood
$define REST_ARP_NOARP	0x40000029	' ARP: Don't Allow ARP to come up at all
$define REST_ARP_NOREMOVEPAGE	0x4000002A	' ARP: Don't allow Remove page
$define REST_ARP_NOADDPAGE	0x4000002B	' ARP: Don't allow Add page
$define REST_ARP_NOWINSETUPPAGE	0x4000002C	' ARP: Don't allow opt components page
$define REST_GREYMSIADS	0x4000002D	' SM: Allow the greying of Darwin Ads SM
$define REST_NOCHANGEMAPPEDDRIVELABEL	0x4000002E	' Don't enable the UI which allows users to rename mapped drive labels
$define REST_NOCHANGEMAPPEDDRIVECOMMENT	0x4000002F	' Don't enable the UI which allows users to change mapped drive comments
$define REST_MaxRecentDocs	0x40000030
$define REST_NONETWORKCONNECTIONS	0x40000031	' No Start Menu | Settings |Network Connections
$define REST_FORCESTARTMENULOGOFF	0x40000032	' Force logoff on the Start Menu
$define REST_NOWEBVIEW	0x40000033	' Disable Web View
$define REST_NOCUSTOMIZETHISFOLDER	0x40000034	' Disable Customize This Folder
$define REST_NOENCRYPTION	0x40000035	' Don't allow file encryption
'	Do NOT use me	0x40000036
$define REST_DONTSHOWSUPERHIDDEN	0x40000037	' don't show super hidden files
$define REST_NOSHELLSEARCHBUTTON	0x40000038
$define REST_NOHARDWARETAB	0x40000039	' No Hardware tab on Drives or control panel
$define REST_NORUNASINSTALLPROMPT	0x4000003A	' Don't bring up "Run As" prompt for install programs
$define REST_PROMPTRUNASINSTALLNETPATH	0x4000003B	' Force the	"Run As" prompt for install programs on unc/network shares
$define REST_NOMANAGEMYCOMPUTERVERB	0x4000003C	' No Manage verb on My Computer
$define REST_NORECENTDOCSNETHOOD	0x4000003D	' dont add the recent docs shares to nethood
$define REST_DISALLOWRUN	0x4000003E	' don't allow certain apps to be run
$define REST_NOWELCOMESCREEN	0x4000003F	' don't allow the welcome screen to be displayed.
$define REST_RESTRICTCPL	0x40000040	' only allow certain cpls to be run
$define REST_DISALLOWCPL	0x40000041	' don't allow certain cpls to be run
$define REST_NOSMBALLOONTIP	0x40000042	' No Start Menu Balloon Tip
$define REST_NOSMHELP	0x40000043	' No Help on the Start Menu
$define REST_NOWINKEYS	0x40000044	' No Windows-X Hot keys
$define REST_NOENCRYPTONMOVE	0x40000045	' Don't automatically try to encrypt files that are moved to encryped directories
$define REST_NOLOCALMACHINERUN	0x40000046	' ignore HKLM\sw\ms\win\cv\Run and all of it's sub keys
$define REST_NOCURRENTUSERRUN	0x40000047	' ignore HKCU\sw\ms\win\cv\Run and all of it's sub keys
$define REST_NOLOCALMACHINERUNONCE	0x40000048	' ignore HKLM\sw\ms\win\cv\RunOnce and all of it's sub keys
$define REST_NOCURRENTUSERRUNONCE	0x40000049	' ignore HKCU\sw\ms\win\cv\RunOnce and all of it's sub keys
$define REST_FORCEACTIVEDESKTOPON	0x4000004A	' Force ActiveDesktop to be turned ON all the time.
$define REST_NOCOMPUTERSNEARME	0x4000004B	' removes the "Computers near me" link
$define REST_NOVIEWONDRIVE	0x4000004C	' disallows CreateViewObject()on specified drives(CFSFolder only)
$define REST_NONETCRAWL	0x4000004D	' disables the crawling of the WNet namespace.
$define REST_NOSHAREDDOCUMENTS	0x4000004E	' don't auto share the Shared Documents/create link
$define REST_NOSMMYDOCS	0x4000004F	' Don't show the My Documents item on the Start Menu.
$define REST_NOSMMYPICS	0x40000050	' Don't show the My Pictures item on the Start Menu
$define REST_ALLOWBITBUCKDRIVES	0x40000051	' Bit mask indicating which which drives have bit bucket support
$define REST_NONLEGACYSHELLMODE	0x40000052	' new consumer shell modes
$define REST_NOCONTROLPANELBARRICADE	0x40000053	' The webview barricade Control Panel
$define REST_NOSTARTPAGE	0x40000054	' Whistler Start Page on desktop.
$define REST_NOAUTOTRAYNOTIFY	0x40000055	' Whistler auto-tray notify feature
$define REST_NOTASKGROUPING	0x40000056	' Whistler taskbar button grouping feature
$define REST_NOCDBURNING	0x40000057	' whistler cd burning feature
$define REST_MYCOMPNOPROP	0x40000058	' disables Properties on My Computer's context menu
$define REST_MYDOCSNOPROP	0x40000059	' disables Properties on My Documents' context menu
$define REST_NOSTARTPANEL	0x4000005A	' Windows start panel(New start menu)for Whistler.
$define REST_NODISPLAYAPPEARANCEPAGE	0x4000005B	' disable Themes and Appearance tabs the Display Control Panel.
$define REST_NOTHEMESTAB	0x4000005C	' disable the Themes tab the Display Control Panel.
$define REST_NOVISUALSTYLECHOICE	0x4000005D	' disable the visual style drop down the Appearance tab of the Display Control Panel.
$define REST_NOSIZECHOICE	0x4000005E	' disable the size drop down the Appearance tab of the Display Control Panel.
$define REST_NOCOLORCHOICE	0x4000005F	' disable the color drop down the Appearance tab of the Display Control Panel.
$define REST_SETVISUALSTYLE	0x40000060	' Load the specified file as the visual style.
$define REST_STARTRUNNOHOMEPATH	0x40000061	' dont use the %HOMEPATH% env var for the Start-Run dialog
$define REST_NOUSERNAMEINSTARTPANEL	0x40000062	' don't show the username is the startpanel.
$define REST_NOMYCOMPUTERICON	0x40000063	' don't show my computer anywhere hide its contents
$define REST_NOSMNETWORKPLACES	0x40000064	' don't show network places startpanel.
$define REST_NOSMPINNEDLIST	0x40000065	' don't show the pinned list startpanel.
$define REST_NOSMMYMUSIC	0x40000066	' don't show MyMusic folder startpanel
$define REST_NOSMEJECTPC	0x40000067	' don't show "Undoc PC" command startmenu
$define REST_NOSMMOREPROGRAMS	0x40000068	' don't show "More Programs" button StartPanel.
$define REST_NOSMMFUPROGRAMS	0x40000069	' don't show the MFU programs list StartPanel.
$define REST_NOTRAYITEMSDISPLAY	0x4000006A	' disables the display of the system tray
$define REST_NOTOOLBARSONTASKBAR	0x4000006B	' disables toolbar display on the taskbar
$define REST_NOSMCONFIGUREPROGRAMS	0x4000006F	' No Configure Programs on Settings Menu
$define REST_HIDECLOCK	0x40000070	' don't show the clock
$define REST_NOLOWDISKSPACECHECKS	0x40000071	' disable the low disk space checking
$define REST_NOENTIRENETWORK	0x40000072	' removes the "Entire Network" link(i.e. from "My Network Places")
$define REST_NODESKTOPCLEANUP	0x40000073	' disable the desktop cleanup wizard
$define REST_BITBUCKNUKEONDELETE	0x40000074	' disables recycling of files
$define REST_BITBUCKCONFIRMDELETE	0x40000075	' always show the delete confirmation dialog when deleting files
$define REST_BITBUCKNOPROP	0x40000076	' disables Properties on Recycle Bin's context menu
$define REST_NODISPBACKGROUND	0x40000077	' disables the Desktop tab the Display CPL
$define REST_NODISPSCREENSAVEPG	0x40000078	' disables the Screen Saver tab the Display CPL
$define REST_NODISPSETTINGSPG	0x40000079	' disables the Settings tab the Display CPL
$define REST_NODISPSCREENSAVEPREVIEW	0x4000007A	' disables the screen saver on the Screen Saver tab the Display CPL
$define REST_NODISPLAYCPL	0x4000007B	' disables the Display CPL
$define REST_HIDERUNASVERB	0x4000007C	' hides the "Run As..." context menu item
$define REST_NOTHUMBNAILCACHE	0x4000007D	' disables use of the thumbnail cache
$define REST_NOSTRCMPLOGICAL	0x4000007E	' dont use StrCmpLogical()instead use default CompareString()
$define REST_NOPUBLISHWIZARD	0x4000007F	' disables publishing wizard(WPW)
$define REST_NOONLINEPRINTSWIZARD	0x40000080	' disables online prints wizard(OPW)
$define REST_NOWEBSERVICES	0x40000081	' disables the web specified services for both OPW and WPW
$define REST_ALLOWUNHASHEDWEBVIEW	0x40000082	' allow the user to be promted to accept web view templates that don't already have an md5 hash the registry
$define REST_ALLOWLEGACYWEBVIEW	0x40000083	' allow legacy webview template to be shown.
$define REST_REVERTWEBVIEWSECURITY	0x40000084	' disable added webview security measures(revert to w2k functionality).
$define REST_INHERITCONSOLEHANDLES	0x40000086	' ShellExec()will check for the current process and target process being console processes to inherit handles
$define REST_SORTMAXITEMCOUNT	0x40000087	' Do not sort views with more items than this key. Useful for viewing big amount of files one folder.
$define REST_NOREMOTERECURSIVEEVENTS	0x40000089	' Dont register network change events recursively to avoid network traffic
$define REST_NOREMOTECHANGENOTIFY	0x40000091	' Do not register for remote change notifies
$define REST_NOSIMPLENETIDLIST	0x40000092	' No simple network IDLists
$define REST_NOENUMENTIRENETWORK	0x40000093	' Don't enumerate entire network if we happen to get to it(in conjunction with REST_NOENTIRENETWORK)
$define REST_NODETAILSTHUMBNAILONNETWORK	0x40000094	' Disable Thumbnail for Network files DUI Details pane
$define REST_NOINTERNETOPENWITH	0x40000095	' dont allow looking on the internet for file associations
$define REST_ALLOWLEGACYLMZBEHAVIOR	0x4000009A	' allowable LMZ behavior for ActiveX objects changed XPSP2 this policy gets the pre-XPSP2 behavior
$define REST_DONTRETRYBADNETNAME	0x4000009B	' Network Places: if provider returns ERROR_BAD_NET_NAME give up
$define REST_ALLOWFILECLSIDJUNCTIONS	0x4000009C	' re-enable legacy support for file.{guid} junctions FileSystem Folder
$define REST_NOUPNPINSTALL	0x4000009D	' disable "install UPnP" task My Net Places

$define REST_NODISCONNECT	0x41000001	' No Disconnect option Start menu
$define REST_NOSECURITY	0x41000002	' No Security option start menu
$define REST_NOFILEASSOCIATE	0x41000003	' Do not allow user to change file association
$define REST_ALLOWCOMMENTTOGGLE	0x41000004	' Allow the user to toggle the positions of the Comment and the Computer Name
$define REST_USEDESKTOPINICACHE	0x41000005	' Cache desktop.ini entries from network folders

declare import, OpenRegStream(HKEY hkey,LPCWSTR pszSubkey,LPCWSTR pszValue, DWORD grfMode),IStream
declare import, SHFindFiles(pointer pidlFolder,pointer pidlSaveFile),BOOL
declare import, PathGetShortPath(LPWSTR pszLongPath)
declare import, PathYetAnotherMakeUniqueName(LPWSTR pszUniqueName, LPCWSTR pszPath, LPCWSTR pszShort, LPCWSTR pszFileSpec),BOOL
declare import, Win32DeleteFile(LPCWSTR pszPath),BOOL
' Path processing function
$define PPCF_ADDQUOTES	0x00000001	' return a quoted name if required
$define PPCF_ADDARGUMENTS	0x00000003	' appends arguments(and wraps quotes if required)
$define PPCF_NODIRECTORIES	0x00000010	' don't match to directories
$define PPCF_FORCEQUALIFY	0x00000040	' qualify even non-relative names
$define PPCF_LONGESTPOSSIBLE	0x00000080	' always find the longest possible name

declare import, PathProcessCommand(LPCWSTR lpSrc, LPWSTR lpDest, int iMax, DWORD dwFlags),LONG
declare import, SHRestricted(RESTRICTIONS rest),DWORD
declare import, SignalFileOpen(pointer pidl),BOOL
declare import, SHSimpleIDListFromPath(LPCWSTR pszPath),pointer
declare import, SHLoadOLE(LPARAM lParam)
declare import, SHStartNetConnectionDialogA(HWND hwnd,LPCSTR pszRemoteName, DWORD dwType),HRESULT
declare import, SHStartNetConnectionDialogW(HWND hwnd,LPCWSTR pszRemoteName, DWORD dwType),HRESULT
declare import, SHDefExtractIconA(LPCSTR pszIconFile, int iIndex, UINT uFlags,_pointer phiconLarge,pointer phiconSmall, UINT nIconSize),HRESULT
declare import, SHDefExtractIconW(LPCWSTR pszIconFile, int iIndex, UINT uFlags,_pointer phiconLarge,pointer phiconSmall, UINT nIconSize),HRESULT
$ifdef UNICODE
declare import, SHStartNetConnectionDialog alias SHStartNetConnectionDialogW(HWND hwnd,LPCWSTR pszRemoteName, DWORD dwType),HRESULT
declare import, SHDefExtractIcon alias SHDefExtractIconW(LPCWSTR pszIconFile, int iIndex, UINT uFlags,_pointer phiconLarge,pointer phiconSmall, UINT nIconSize),HRESULT
$else
declare import, SHStartNetConnectionDialog alias SHStartNetConnectionDialogA(HWND hwnd,LPCSTR pszRemoteName, DWORD dwType),HRESULT
declare import, SHDefExtractIcon alias SHDefExtractIconA(LPCSTR pszIconFile, int iIndex, UINT uFlags,_pointer phiconLarge,pointer phiconSmall, UINT nIconSize),HRESULT
$endif

declare import, Shell_GetImageLists(pointer phiml,pointer phimlSmall),BOOL
declare import, Shell_GetCachedImageIndex(LPCWSTR pszIconPath, int iIconIndex, UINT uIconFlags),int

' IDocViewSite
' IDocViewSite
interface IDocViewSite ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IDocViewSite
	stdmethod OnSetTitle(pointer pvTitle),HRESULT
endinterface
$define VALIDATEUNC_NOUI	0x0002	' don't bring up UI
$define VALIDATEUNC_CONNECT	0x0001	' connect a drive letter
$define VALIDATEUNC_PRINT	0x0004	' validate as print share instead of disk share
$define VALIDATEUNC_VALID	0x0007	' valid flags

declare import, SHValidateUNC(HWND hwndOwner,LPWSTR pszFile, UINT fConnect),BOOL
$define OPENPROPS_NONE	0x0000
$define OPENPROPS_INHIBITPIF	0x8000
$define GETPROPS_NONE	0x0000
$define SETPROPS_NONE	0x0000
$define CLOSEPROPS_NONE	0x0000
$define CLOSEPROPS_DISCARD	0x0001

$define PIFNAMESIZE	30
$define PIFSTARTLOCSIZE 63
$define PIFDEFPATHSIZE	64
$define PIFPARAMSSIZE	64
$define PIFSHPROGSIZE	64
$define PIFSHDATASIZE	64
$define PIFDEFFILESIZE	80
$define PIFMAXFILEPATH	260

type PROPPRG
	USHORT flPrg	' see PRG_ flags
	USHORT flPrgInit	' see PRGINIT_ flags
	istring	achTitle[PIFNAMESIZE]	' name[30]
	istring	achCmdLine[PIFSTARTLOCSIZE+PIFPARAMSSIZE+1]' startfile[63] + params[64]
	istring	achWorkDir[PIFDEFPATHSIZE]	' defpath[64]
	USHORT wHotKey	' PfHotKeyScan thru PfHotKeyVal
	istring	achIconFile[PIFDEFFILESIZE]	' name of file containing icon
	USHORT wIconIndex	' index of icon within file
	DWORD	dwEnhModeFlags	' reserved enh-mode flags
	DWORD	dwRealModeFlags	' real-mode flags(see RMOPT_*)
	istring	achOtherFile[PIFDEFFILESIZE]	' name of "other" file directory
	istring	achPIFFile[PIFMAXFILEPATH]	' name of PIF file
endtype

declare import, PifMgr_OpenProperties(LPCWSTR pszApp,LPCWSTR pszPIF, UINT hInf, UINT flOpt),HANDLE
declare import, PifMgr_GetProperties(HANDLE hProps,LPCSTR pszGroup,pointer lpProps, int cbProps, UINT flOpt),int
declare import, PifMgr_SetProperties(HANDLE hProps,LPCSTR pszGroup,pointer lpProps, int cbProps, UINT flOpt),int
declare import, PifMgr_CloseProperties(HANDLE hProps, UINT flOpt),HANDLE

declare import, SHSetInstanceExplorer(comref punk)
declare import, IsUserAnAdmin(),BOOL

'	IInitializeObject

interface IInitializeObject ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IInitializeObject
	stdmethod Initialize(),HRESULT
endinterface

$define BMICON_LARGE 0
$define BMICON_SMALL 1

'	IBanneredBar

interface IBanneredBar ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IBanneredBar
	stdmethod SetIconSize(DWORD iIcon),HRESULT
	stdmethod GetIconSize(pointer piIcon),HRESULT
	stdmethod SetBitmap(HBITMAP hBitmap),HRESULT
	stdmethod GetBitmap(pointer phBitmap),HRESULT
endinterface

declare import, SHShellFolderView_Message(HWND hwndMain, UINT uMsg, LPARAM lParam),LRESULT

' Callback interface for the IShellFolderView

'	IShellFolderViewCB

interface IShellFolderViewCB ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'IShellFolderViewCB
	stdmethod MessageSFVCB(UINT uMsg, WPARAM wParam, LPARAM lParam),HRESULT
endinterface

'$include <pshpack8.h>

'$define UNSIZED_ARRAY2
'$define EMPTY_SIZE2 UNSIZED_ARRAY2

$define QCMINFO_PLACE_BEFORE	0
$define QCMINFO_PLACE_AFTER	1
type QCMINFO_IDMAP_PLACEMENT
	UINT id
	UINT fFlags
endtype

type QCMINFO_IDMAP
	UINT	nMaxIds
	QCMINFO_IDMAP_PLACEMENT pIdList[1]
endtype

type QCMINFO
	HMENU	hmenu	' in
	UINT	indexMenu	' in
	UINT	idCmdFirst	' in/out
	UINT	idCmdLast	' in
	pointer pIdMap ' / unused
endtype

' TBINFO flags
$define TBIF_APPEND	0
$define TBIF_PREPEND	1
$define TBIF_REPLACE	2
$define TBIF_DEFAULT	0x00000000
$define TBIF_INTERNETBAR	0x00010000
$define TBIF_STANDARDTOOLBAR	0x00020000
$define TBIF_NOTOOLBAR	0x00030000

type TBINFO
	UINT	cbuttons	' out
	UINT	uFlags	'(one of TBIF_ flags)
endtype

type DETAILSINFO
	pointer pidl
	int fmt
	int cxChar
	STRRET str
	int iImage
endtype

type SFVM_PROPPAGE_DATA
	DWORD	dwReserved
	LPFNADDPROPSHEETPAGE pfn
	LPARAM	lParam
endtype

type SFVM_HELPTOPIC_DATA
	iwstring wszHelpFile[MAX_PATH]
	iwstring wszHelpTopic[MAX_PATH]
endtype

'	uMsg	wParam	lParam
$define SFVM_MERGEMENU	1	' -	LPQCMINFO
$define SFVM_INVOKECOMMAND	2	' idCmd	-
$define SFVM_GETHELPTEXT	3	' idCmd,cchMax	pszText
$define SFVM_GETTOOLTIPTEXT	4	' idCmd,cchMax	pszText
$define SFVM_GETBUTTONINFO	5	' -	LPTBINFO
$define SFVM_GETBUTTONS	6	' idCmdFirst,cbtnMax LPTBBUTTON
$define SFVM_INITMENUPOPUP	7	' idCmdFirst,nIndex	hmenu
$define SFVM_FSNOTIFY	14	' LPCITEMIDLIST*	lEvent
$define SFVM_WINDOWCREATED	15	' hwnd	-
$define SFVM_GETDETAILSOF	23	' iColumn	DETAILSINFO*
$define SFVM_COLUMNCLICK	24	' iColumn	-
$define SFVM_QUERYFSNOTIFY	25	' -	SHChangeNotifyEntry*
$define SFVM_DEFITEMCOUNT	26	' -	pointer
$define SFVM_DEFVIEWMODE	27	' -	FOLDERVIEWMODE*
$define SFVM_UNMERGEMENU	28	' -	hmenu
$define SFVM_UPDATESTATUSBAR	31	' fInitialize	-
$define SFVM_BACKGROUNDENUM	32	' -	-
$define SFVM_DIDDRAGDROP	36	' dwEffect	pointer
$define SFVM_SETISFV	39	' -	IShellFolderView*
$define SFVM_THISIDLIST	41	' -	LPITMIDLIST*
$define SFVM_ADDPROPERTYPAGES	47	' -	SFVM_PROPPAGE_DATA*
$define SFVM_BACKGROUNDENUMDONE	48	' -	-
$define SFVM_GETNOTIFY	49	' pointer	pointer
$define SFVM_GETSORTDEFAULTS	53	' iDirection	iParamSort
$define SFVM_SIZE	57	' -	-
$define SFVM_GETZONE	58	' -	pointer
$define SFVM_GETPANE	59	' Pane ID	pointer
$define SFVM_GETHELPTOPIC	63	' -	SFVM_HELPTOPIC_DATA*
$define SFVM_GETANIMATION	68	' HINSTANCE*	pointer
' SHCreateShellFolderView struct
type SFV_CREATE
	UINT	cbSize
	IShellFolder	pshf
	IShellView	psvOuter
	IShellFolderViewCB psfvcb ' No callback if NULL
endtype

declare import, SHCreateShellFolderView(pointer pcsfv, pointer ppsv),HRESULT

typedef LPFNDFMCALLBACK pointer
'typedef HRESULT(pointer LPFNDFMCALLBACK)(pointer psf, HWND hwnd,pointer pdtobj, UINT uMsg, WPARAM wParam, LPARAM lParam)

declare import, CDefFolderMenu_Create2(pointer pidlFolder, HWND hwnd,UINT cidl,pointer apidl,comref psf,pointer lpfn,UINT nKeys,pointer ahkeyClsKeys,pointer ppcm),int
declare import, SHOpenPropSheetA(LPCSTR pszCaption,pointer ahkeys, UINT cikeys,pointer	pclsidDefault,comref pdtobj,comref psb,LPCSTR pStartPage),BOOL
declare import, SHOpenPropSheetW(LPCWSTR pszCaption,pointer ahkeys, UINT cikeys,pointer	pclsidDefault,comref pdtobj,comref psb,LPCWSTR pStartPage),BOOL
$ifdef UNICODE
declare import, SHOpenPropSheet alias SHOpenPropSheetW(LPCWSTR pszCaption,pointer ahkeys, UINT cikeys,pointer	pclsidDefault,comref pdtobj,comref psb,LPCWSTR pStartPage),BOOL

$else
declare import, SHOpenPropSheet alias SHOpenPropSheetA(LPCSTR pszCaption,pointer ahkeys, UINT cikeys,pointer	pclsidDefault,comref pdtobj,comref psb,LPCSTR pStartPage),BOOL

$endif
'	uMsg	wParam	lParam
$define DFM_MERGECONTEXTMENU	1	' uFlags	LPQCMINFO
$define DFM_INVOKECOMMAND	2	' idCmd	pszArgs
$define DFM_GETDEFSTATICID	14	' idCmd*	0
' Commands from DFM_INVOKECOMMAND when strings are passed in
$define DFM_CMD_PROPERTIES	-5

'typedef pointer	LPFNVIEWCALLBACK)(pointer psvOuter,pointer psf, HWND hwndMain, UINT uMsg, WPARAM wParam, LPARAM lParam)
typedef LPFNVIEWCALLBACK int
' SHCreateShellFolderViewEx struct
type SFV
	UINT	cbSize
	IShellFolder	pshf
	IShellView	psvOuter
	pointer pidl
	LONG	lEvents
	LPFNVIEWCALLBACK pfnCallback	' No callback if NULL
	FOLDERVIEWMODE	fvm
endtype

' Tell the FolderView to rearrange.	The lParam will be passed to
' IShellFolder::CompareIDs
$define SFVM_REARRANGE	0x00000001
'$define ShellFolderView_ReArrange(_hwnd, _lparam)\
'	(BOOL)SHShellFolderView_Message(_hwnd, SFVM_REARRANGE, _lparam)
' Add an OBJECT into the view
$define SFVM_ADDOBJECT	0x00000003
'$define ShellFolderView_AddObject(_hwnd, _pidl)\
'	(LPARAM)SHShellFolderView_Message(_hwnd, SFVM_ADDOBJECT,(LPARAM)_pidl)
' Remove an OBJECT into the view
$define SFVM_REMOVEOBJECT	0x00000006
'$define ShellFolderView_RemoveObject(_hwnd, _pidl)\
'	(LPARAM)SHShellFolderView_Message(_hwnd, SFVM_REMOVEOBJECT,(LPARAM)_pidl)

' updates an object by passing pointer to two PIDLS, the first
' is the old pidl, the second one is the one with update information.
' _ppidl[1] must be a* copy* of a pidl, as control over the lifetime
' of the pidl belongs to the view after successful completion of
' this call.	(Unsuccessful completion(a -1 return)implies failure
' and the caller must free the memory.)	Win95 waits a while before
' freeing the pidl, IE4 frees the pidl immediately.
' IShellFolderView::UpdateObject does not suffer from this problem.
$define SFVM_UPDATEOBJECT	0x00000007
'$define ShellFolderView_UpdateObject(_hwnd, _ppidl)\
'	(LPARAM)SHShellFolderView_Message(_hwnd, SFVM_UPDATEOBJECT,(LPARAM)_ppidl)

' Returns an array of the selected IDS to the caller.
'	lparam is a pointer to receive the idlists into
'	return value is the count of items the array.
$define SFVM_GETSELECTEDOBJECTS 0x00000009
'$define ShellFolderView_GetSelectedObjects(_hwnd, ppidl)\
'	(LPARAM)SHShellFolderView_Message(_hwnd, SFVM_GETSELECTEDOBJECTS,(LPARAM)ppidl)
type SFV_SETITEMPOS
		pointer pidl
		POINT pt
endtype

' Sets the position of an item the viewer
'	lparam is a pointer to a SVF_SETITEMPOS
'	return value is unused
$define SFVM_SETITEMPOS	0x0000000e
'$define ShellFolderView_SetItemPos(_hwnd, _pidl, _x, _y)\
'{	SFV_SETITEMPOS _sip = {_pidl, {_x, _y}} \
'	SHShellFolderView_Message(_hwnd, SFVM_SETITEMPOS,(LPARAM)(LPSFV_SETITEMPOS)&_sip);}
'	Notifies a ShellView when one of its objects get put on the clipboard
'	as a result of a menu command.
'	lparam is the dwEffect(DROPEFFECT_MOVE, DROPEFFECT_COPY)
'	return value is void.
$define SFVM_SETCLIPBOARD	0x00000010
'$define ShellFolderView_SetClipboard(_hwnd, _dwEffect)\
'	()SHShellFolderView_Message(_hwnd, SFVM_SETCLIPBOARD,(LPARAM)(DWORD)(_dwEffect))
$define SFVM_SETPOINTS	0x00000017
'$define ShellFolderView_SetPoints(_hwnd, _pdtobj)\
'	()SHShellFolderView_Message(_hwnd, SFVM_SETPOINTS,(LPARAM)_pdtobj)
'$include <poppack.h>	/* Return to byte packing */
declare import, SHFind_InitMenuPopup(HMENU hmenu, HWND hwndOwner, UINT idCmdFirst, UINT idCmdLast),IContextMenu
declare import, SHCreateShellFolderViewEx(pointer pcsfv, pointer	ppsv)

' PROPIDs for Internet Shortcuts(FMTID_Intshcut)to be used with
' IPropertySetStorage/IPropertyStorage
' The known property ids and their variant types are:
'	PID_IS_URL	[VT_LPWSTR]	URL
'	PID_IS_NAME	[VT_LPWSTR]	Name of the internet shortcut
'	PID_IS_WORKINGDIR	[VT_LPWSTR]	Working directory for the shortcut
'	PID_IS_HOTKEY	[VT_UI2]	Hotkey for the shortcut
'	PID_IS_SHOWCMD	[VT_I4]	Show command for shortcut
'	PID_IS_ICONINDEX	[VT_I4]	Index into file that has icon
'	PID_IS_ICONFILE	[VT_LPWSTR]	File that has the icon
'	PID_IS_WHATSNEW	[VT_LPWSTR]	What's New text
'	PID_IS_AUTHOR	[VT_LPWSTR]	Author
'	PID_IS_DESCRIPTION	[VT_LPWSTR]	Description text of site
'	PID_IS_COMMENT	[VT_LPWSTR]	User annotated comment

$define PID_IS_URL	2
$define PID_IS_NAME	4
$define PID_IS_WORKINGDIR	5
$define PID_IS_HOTKEY	6
$define PID_IS_SHOWCMD	7
$define PID_IS_ICONINDEX	8
$define PID_IS_ICONFILE	9
$define PID_IS_WHATSNEW	10
$define PID_IS_AUTHOR	11
$define PID_IS_DESCRIPTION	12
$define PID_IS_COMMENT	13

' PROPIDs for Internet Sites(FMTID_InternetSite)to be used with
' IPropertySetStorage/IPropertyStorage
' The known property ids and their variant types are:
'	PID_INTSITE_WHATSNEW	[VT_LPWSTR]	What's New text
'	PID_INTSITE_AUTHOR	[VT_LPWSTR]	Author
'	PID_INTSITE_LASTVISIT	[VT_FILETIME] Time site was last visited
'	PID_INTSITE_LASTMOD	[VT_FILETIME] Time site was last modified
'	PID_INTSITE_VISITCOUNT	[VT_UI4]	Number of times user has visited
'	PID_INTSITE_DESCRIPTION	[VT_LPWSTR]	Description text of site
'	PID_INTSITE_COMMENT	[VT_LPWSTR]	User annotated comment
'	PID_INTSITE_RECURSE	[VT_UI4]	Levels to recurse(0-3)
'	PID_INTSITE_WATCH	[VT_UI4]	PIDISM_ flags
'	PID_INTSITE_SUBSCRIPTION [VT_UI8]	Subscription cookie
'	PID_INTSITE_URL	[VT_LPWSTR]	URL
'	PID_INTSITE_TITLE	[VT_LPWSTR]	Title
'	PID_INTSITE_CODEPAGE	[VT_UI4]	Codepage of the document
'	PID_INTSITE_TRACKING	[VT_UI4]	Tracking
'	PID_INTSITE_ICONINDEX	[VT_I4]	Retrieve the index to the icon
'	PID_INTSITE_ICONFILE	[VT_LPWSTR]	Retrieve the file containing the icon index.

$define PID_INTSITE_WHATSNEW	2
$define PID_INTSITE_AUTHOR	3
$define PID_INTSITE_LASTVISIT	4
$define PID_INTSITE_LASTMOD	5
$define PID_INTSITE_VISITCOUNT	6
$define PID_INTSITE_DESCRIPTION	7
$define PID_INTSITE_COMMENT	8
$define PID_INTSITE_FLAGS	9
$define PID_INTSITE_CONTENTLEN	10
$define PID_INTSITE_CONTENTCODE	11
$define PID_INTSITE_RECURSE	12
$define PID_INTSITE_WATCH	13
$define PID_INTSITE_SUBSCRIPTION	14
$define PID_INTSITE_URL	15
$define PID_INTSITE_TITLE	16
$define PID_INTSITE_CODEPAGE	18
$define PID_INTSITE_TRACKING	19
$define PID_INTSITE_ICONINDEX	20
$define PID_INTSITE_ICONFILE	21

' Flags for PID_IS_FLAGS
$define PIDISF_RECENTLYCHANGED	0x00000001
$define PIDISF_CACHEDSTICKY	0x00000002
$define PIDISF_CACHEIMAGES	0x00000010
$define PIDISF_FOLLOWALLLINKS	0x00000020

' Values for PID_INTSITE_WATCH
$define PIDISM_GLOBAL	0	' Monitor based on global setting
$define PIDISM_WATCH	1	' User says watch
$define PIDISM_DONTWATCH	2	' User says don't watch

'//////////////////////////////////////////////////////////////////
' The shell keeps track of some per-user state to handle display
' options that is of major interest to ISVs.
' The key one requested right now is "DoubleClickInWebView".

type SHELLSTATEA
/*	BOOL fShowAllObjects : 1
	BOOL fShowExtensions : 1
	BOOL fNoConfirmRecycle : 1

	BOOL fShowSysFiles : 1
	BOOL fShowCompColor : 1
	BOOL fDoubleClickInWebView : 1
	BOOL fDesktopHTML : 1
	BOOL fWin95Classic : 1
	BOOL fDontPrettyPath : 1
	BOOL fShowAttribCol : 1 ' No longer used, dead bit
	BOOL fMapNetDrvBtn : 1
	BOOL fShowInfoTip : 1
	BOOL fHideIcons : 1
	BOOL fWebView : 1
	BOOL fFilter : 1
	BOOL fShowSuperHidden : 1
	BOOL fNoNetCrawling : 1;*/
	BOOL fShowAllObjects

	DWORD dwWin95Unused ' Win95 only - no longer supported pszHiddenFileExts
	UINT	uWin95Unused ' Win95 only - no longer supported cbHiddenFileExts

	' Note: Not a typo!	This is a persisted structure so we cannot use LPARAM
	LONG	lParamSort
	int	iSortDirection

	UINT	version

	' new for win2k. need notUsed var to calc the right size of ie4 struct
	' FIELD_OFFSET does not work on bit fields
	UINT uNotUsed ' feel free to rename and use
/*	BOOL fSepProcess: 1
	' new for Whistler.
	BOOL fStartPanelOn: 1	//Indicates if the Whistler StartPanel mode is ON or OFF.
	BOOL fShowStartPage: 1	//Indicates if the Whistler StartPage on desktop is ON or OFF.*/
	BOOL fSepProcess
	/*UINT fSpareFlags : 13;*/
	UINT fSpareFlags
endtype

type SHELLSTATEW
/*	BOOL fShowAllObjects : 1
	BOOL fShowExtensions : 1
	BOOL fNoConfirmRecycle : 1
	BOOL fShowSysFiles : 1
	BOOL fShowCompColor : 1
	BOOL fDoubleClickInWebView : 1
	BOOL fDesktopHTML : 1
	BOOL fWin95Classic : 1
	BOOL fDontPrettyPath : 1
	BOOL fShowAttribCol : 1
	BOOL fMapNetDrvBtn : 1
	BOOL fShowInfoTip : 1
	BOOL fHideIcons : 1
	BOOL fWebView : 1
	BOOL fFilter : 1
	BOOL fShowSuperHidden : 1
	BOOL fNoNetCrawling : 1;*/
	BOOL fShowAllObjects
	DWORD dwWin95Unused ' Win95 only - no longer supported pszHiddenFileExts
	UINT	uWin95Unused ' Win95 only - no longer supported cbHiddenFileExts

	' Note: Not a typo!	This is a persisted structure so we cannot use LPARAM
	LONG	lParamSort
	int	iSortDirection
	UINT	version

	' new for win2k. need notUsed var to calc the right size of ie4 struct
	' FIELD_OFFSET does not work on bit fields
	UINT uNotUsed ' feel free to rename and use
/*	BOOL fSepProcess: 1
	' new for Whistler.
	BOOL fStartPanelOn: 1	//Indicates if the Whistler StartPage mode is ON or OFF.
	BOOL fShowStartPage: 1	//Indicates if the Whistler StartPage on desktop is ON or OFF.*/
	BOOL fSepProcess
	' If you need a new flag, steal a bit from from fSpareFlags.
	/*UINT fSpareFlags : 13;*/
	UINT fSpareFlags
endtype
$ifdef UNICODE
type SHELLSTATE
/*	BOOL fShowAllObjects : 1
	BOOL fShowExtensions : 1
	BOOL fNoConfirmRecycle : 1
	BOOL fShowSysFiles : 1
	BOOL fShowCompColor : 1
	BOOL fDoubleClickInWebView : 1
	BOOL fDesktopHTML : 1
	BOOL fWin95Classic : 1
	BOOL fDontPrettyPath : 1
	BOOL fShowAttribCol : 1
	BOOL fMapNetDrvBtn : 1
	BOOL fShowInfoTip : 1
	BOOL fHideIcons : 1
	BOOL fWebView : 1
	BOOL fFilter : 1
	BOOL fShowSuperHidden : 1
	BOOL fNoNetCrawling : 1;*/
	BOOL fShowAllObjects
	DWORD dwWin95Unused ' Win95 only - no longer supported pszHiddenFileExts
	UINT	uWin95Unused ' Win95 only - no longer supported cbHiddenFileExts

	' Note: Not a typo!	This is a persisted structure so we cannot use LPARAM
	LONG	lParamSort
	int	iSortDirection
	UINT	version

	' new for win2k. need notUsed var to calc the right size of ie4 struct
	' FIELD_OFFSET does not work on bit fields
	UINT uNotUsed ' feel free to rename and use
/*	BOOL fSepProcess: 1
	' new for Whistler.
	BOOL fStartPanelOn: 1	//Indicates if the Whistler StartPage mode is ON or OFF.
	BOOL fShowStartPage: 1	//Indicates if the Whistler StartPage on desktop is ON or OFF.*/
	BOOL fSepProcess
	' If you need a new flag, steal a bit from from fSpareFlags.
	/*UINT fSpareFlags : 13;*/
	UINT fSpareFlags
endtype
$else
type SHELLSTATE
/*	BOOL fShowAllObjects : 1
	BOOL fShowExtensions : 1
	BOOL fNoConfirmRecycle : 1

	BOOL fShowSysFiles : 1
	BOOL fShowCompColor : 1
	BOOL fDoubleClickInWebView : 1
	BOOL fDesktopHTML : 1
	BOOL fWin95Classic : 1
	BOOL fDontPrettyPath : 1
	BOOL fShowAttribCol : 1 ' No longer used, dead bit
	BOOL fMapNetDrvBtn : 1
	BOOL fShowInfoTip : 1
	BOOL fHideIcons : 1
	BOOL fWebView : 1
	BOOL fFilter : 1
	BOOL fShowSuperHidden : 1
	BOOL fNoNetCrawling : 1;*/
	BOOL fShowAllObjects

	DWORD dwWin95Unused ' Win95 only - no longer supported pszHiddenFileExts
	UINT	uWin95Unused ' Win95 only - no longer supported cbHiddenFileExts

	' Note: Not a typo!	This is a persisted structure so we cannot use LPARAM
	LONG	lParamSort
	int	iSortDirection

	UINT	version

	' new for win2k. need notUsed var to calc the right size of ie4 struct
	' FIELD_OFFSET does not work on bit fields
	UINT uNotUsed ' feel free to rename and use
/*	BOOL fSepProcess: 1
	' new for Whistler.
	BOOL fStartPanelOn: 1	//Indicates if the Whistler StartPanel mode is ON or OFF.
	BOOL fShowStartPage: 1	//Indicates if the Whistler StartPage on desktop is ON or OFF.*/
	BOOL fSepProcess
	/*UINT fSpareFlags : 13;*/
	UINT fSpareFlags
endtype
$endif

$define SHELLSTATEVERSION_IE4	9
$define SHELLSTATEVERSION_WIN2K 10

'$define SHELLSTATE_SIZE_WIN95 FIELD_OFFSET(SHELLSTATE,lParamSort)
'$define SHELLSTATE_SIZE_NT4	FIELD_OFFSET(SHELLSTATE,version)
'$define SHELLSTATE_SIZE_IE4	FIELD_OFFSET(SHELLSTATE,uNotUsed)
'$define SHELLSTATE_SIZE_WIN2K sizeof(SHELLSTATE)

declare import, SHGetSetSettings(pointer lpss, DWORD dwMask, BOOL bSet)

'	SysFiles are these windows special files:
'	"dll sys vxd 386 drv"
'	hidden files are files with the FILE_ATTRIBUTE_HIDDEN attribute
'	system files are files with the FILE_ATTRIBUTE_SYSTEM attribute
'	fShowAllObjects fShowSysFiles	Result
'	--------------- -------------	------
'	0	0	hide hidden + SysFiles + system files
'	0	1	hide hidden files.
'	1	0	show all files.
'	1	1	show all files.
type SHELLFLAGSTATE ' bits in aurora yet not supported
/*	BOOL fShowAllObjects : 1
	BOOL fShowExtensions : 1
	BOOL fNoConfirmRecycle : 1
	BOOL fShowSysFiles : 1
	BOOL fShowCompColor : 1
	BOOL fDoubleClickInWebView : 1
	BOOL fDesktopHTML : 1
	BOOL fWin95Classic : 1
	BOOL fDontPrettyPath : 1
	BOOL fShowAttribCol : 1
	BOOL fMapNetDrvBtn : 1
	BOOL fShowInfoTip : 1
	BOOL fHideIcons : 1
	UINT fRestFlags : 3;*/
	BOOL f
	UINT fRestFlags
endtype

$define SSF_SHOWALLOBJECTS	0x00000001
$define SSF_SHOWEXTENSIONS	0x00000002
$define SSF_HIDDENFILEEXTS	0x00000004
$define SSF_SERVERADMINUI	0x00000004
$define SSF_SHOWCOMPCOLOR	0x00000008
$define SSF_SORTCOLUMNS	0x00000010
$define SSF_SHOWSYSFILES	0x00000020
$define SSF_DOUBLECLICKINWEBVIEW	0x00000080
$define SSF_SHOWATTRIBCOL	0x00000100
$define SSF_DESKTOPHTML	0x00000200
$define SSF_WIN95CLASSIC	0x00000400
$define SSF_DONTPRETTYPATH	0x00000800
$define SSF_SHOWINFOTIP	0x00002000
$define SSF_MAPNETDRVBUTTON	0x00001000
$define SSF_NOCONFIRMRECYCLE	0x00008000
$define SSF_HIDEICONS	0x00004000
$define SSF_FILTER	0x00010000
$define SSF_WEBVIEW	0x00020000
$define SSF_SHOWSUPERHIDDEN	0x00040000
$define SSF_SEPPROCESS	0x00080000
$define SSF_NONETCRAWLING	0x00100000
$define SSF_STARTPANELON	0x00200000
$define SSF_SHOWSTARTPAGE	0x00400000

' SHGetSettings(LPSHELLFLAGSTATE lpss, DWORD dwMask)
' Specify the bits you are interested in dwMask and they will be
' filled the lpss structure.
' When these settings change, a WM_SETTINGCHANGE message is sent
' with the string lParam value of "ShellState".
declare import, SHGetSettings(pointer lpsfs, DWORD dwMask)

' SHBindToParent(pointer pidl, REFIID riid,pointer ppv,pointer ppidlLast)
' Given a pidl, you can get an interface pointer(as specified by riid)of the pidl's parent folder(in ppv)
' If ppidlLast is non-NULL, you can also get the pidl of the last item.
declare import, SHBindToParent(pointer pidl,pointer riid,pointer ppv,pointer ppidlLast),HRESULT

' declare import, SHParseDisplayName(PCWSTR pszName,pointer pbc,pointer ppidl, SFGAOF sfgaoIn,pointer psfgaoOut)
'	given a string it will call psfDesktop->ParseDisplayName()to try and create a pidl
'	if no pbc specified, it uses the preferred options for parsing.
'	this includes mapping file system paths to their appropriate aliased location(RegisterObjectParam(STR_PARSE_TRANSLATE_ALIASES))
'	psfgaoOut is optional for SFGAO attributes
declare import, SHParseDisplayName(pointer pszName,pointer pbc,pointer ppidl, SFGAOF sfgaoIn,pointer psfgaoOut),HRESULT

' SHPathPrepareForWrite(HWND hwnd,pointer punkEnableModless, LPCTSTR pszPath, DWORD dwFlags)
' DESCRIPTION:
'	This API will prepare the path for the caller.	This includes:
' 1. Prompting for the ejectable media to be re-inserted.(Floppy, CD-ROM, ZIP drive, etc.)
' 2. Prompting for the media to be formatted.(Floppy, hard drive, etc.)
' 3. Remount mapped drives if the connection was lost.(\\unc\share mapped to N: becomes disconnected)
' 4. If the path doesn't exist, create it.	(SHPPFW_DIRCREATE and SHPPFW_ASKDIRCREATE)
' 5. Display an error if the media is read only.(SHPPFW_NOWRITECHECK not set)
' PARAMETERS:
'	hwnd: Parernt window for UI.	NULL means don't display UI. OPTIONAL
'	punkEnableModless: Parent that will be set to modal during UI using IOleInPlaceActiveObject::EnableModeless(). OPTIONAL
'	pszPath: Path to verify is valid for writting.	This can be a UNC or file drive path.	The path
'	should only contain directories.	Pass SHPPFW_IGNOREFILENAME if the last path segment
'	is always filename to ignore.
'	dwFlags: SHPPFW_* Flags to modify behavior
'-------------------------------------------------------------------------
$define SHPPFW_NONE	0x00000000
$define SHPPFW_DEFAULT	0x00000001	' May change
$define SHPPFW_DIRCREATE	0x00000001	' Create the directory if it doesn't exist without asking the user.
$define SHPPFW_ASKDIRCREATE	0x00000002	' Create the directory if it doesn't exist after asking the user.
$define SHPPFW_IGNOREFILENAME	0x00000004	' Ignore the last item pszPath because it's a file.	Example: pszPath="C:\DirA\DirB", only use "C:\DirA".
$define SHPPFW_NOWRITECHECK	0x00000008	' Caller only needs to read from the drive, so don't check if it's READ ONLY.
$define SHPPFW_MEDIACHECKONLY	0x00000010	' do the retrys on the media(or net path), return errors if the file can't be found

declare import, SHPathPrepareForWriteA(HWND hwnd,comref punkEnableModless,LPCSTR pszPath, DWORD dwFlags),HRESULT
declare import, SHPathPrepareForWriteW(HWND hwnd,comref punkEnableModless,LPCWSTR pszPath, DWORD dwFlags),HRESULT
$ifdef UNICODE
declare import, SHPathPrepareForWrite alias SHPathPrepareForWriteW(HWND hwnd,comref punkEnableModless,LPCWSTR pszPath, DWORD dwFlags),HRESULT
$else
declare import, SHPathPrepareForWrite alias SHPathPrepareForWriteA(HWND hwnd,comref punkEnableModless,LPCSTR pszPath, DWORD dwFlags),HRESULT
$endif

'--------------------------------------------------------------------------
' Interface used for exposing the INI file methods on a shortcut file
'--------------------------------------------------------------------------

' INamedPropertyBag
interface INamedPropertyBag ' IUnknown
	stdmethod QueryInterface(pointer riid,pointer ppvObject),HRESULT
	stdmethod AddRef(),ULONG
	stdmethod Release(),ULONG
'INamedPropertyBag
	stdmethod ReadPropertyNPB(LPCOLESTR pszBagname,LPCOLESTR pszPropName,/*out in*/pointer pVar),HRESULT
	stdmethod WritePropertyNPB(LPCOLESTR pszBagname,LPCOLESTR pszPropName,pointer pVar),HRESULT
	stdmethod RemovePropertyNPB(LPCOLESTR pszBagname,LPCOLESTR pszPropName),HRESULT
endinterface

$ifdef __urlmon_inc__
'	NOTE: urlmon.h must be included before shlobj.h to access this function.
'	SoftwareUpdateMessageBox
'	Provides a standard message box for the alerting the user that a software
'	update is available or installed. No UI will be displayed if there is no
'	update available or if the available update version is less than or equal
'	to the Advertised update version.
'	hWnd	- [in] Handle of owner window
'	szDistUnit	- [in] Unique identifier string for a code distribution unit. For
'	ActiveX controls and Active Setup installed components, this
'	is typically a GUID string.
'	dwFlags	- [in] Must be 0.
'	psdi	- [in,out] Pointer to SOFTDISTINFO(see URLMon.h). May be NULL.
'	cbSize should be initialized
'	by the caller to sizeof(SOFTDISTINFO), dwReserved should be set to 0.
'	RETURNS:
'	IDNO	- The user chose cancel. If* pbRemind is FALSE, the caller should save the
'	update version from the SOFTDISTINFO and pass it as the Advertised
'	version future calls.
'	IDYES	- The user has selected Update Now/About Update. The caller should navigate to
'	the SOFTDISTINFO's pszHREF to initiate the install or learn about it.
'	The caller should save the update version from the SOFTDISTINFO and pass
'	it as the Advertised version future calls.
'	IDIGNORE - There is no pending software update. Note: There is
'	no Ignore button the standard UI. This occurs if the available
'	version is less than the installed version or is not present or if the
'	Advertised version is greater than or equal to the update version.
'	IDABORT	- An error occured. Call GetSoftwareUpdateInfo()for a more specific HRESULT.
'	Note: There is no Abort button the standard UI.

declare import, SoftwareUpdateMessageBox(HWND hWnd,LPCWSTR szDistUnit, DWORD dwFlags,pointer psdi),DWORD
$endif
' if __urlmon_h__

'	SHPropStgCreate()
'	Wrap of IPropertySetStorage::Open/Create
'	This function ensures proper handling of code page retrieval/assignment
'	for the requested property set operation.
'	psstg,	'	Address of IPropertySetStorage vtable
'	fmtid,	'	property set ID
'	pclsid,	'	class ID associated with the set. This can be NULL
'	grfFlags,	'	PROPSETFLAG_xxx.	All sets containing ansi bytes should be created with
		'	PROPSETFLAG_ANSI, otherwise PROPSETFLAG_DEFAULT.
'	grfMode,	'	STGM_ flags.	Must contain STGM_DIRECT|STGM_EXCLUSIVE.
'	dwDisposition,	'	OPEN_EXISTING. OPEN_ALWAYS, CREATE_NEW, or CREATE_ALWAYS
'	pointer ppstg,	' Address to receive requested vtable
'	puCodePage	'	Optional address to receive the code page ID for the set.
declare import, SHPropStgCreate(comref psstg, REFFMTID fmtid, pointer pclsid, DWORD grfFlags, DWORD grfMode, DWORD dwDisposition, pointer ppstg, pointer puCodePage),HRESULT

'	SHPropStgReadMultiple()
'	IPropertyStorage::ReadMultiple wrap
'	The wrap ensures ANSI/UNICODE translations are handled properly for
'	legacy property sets.
'	pps,	' address of IPropertyStorage vtable.
'	uCodePage, //Code page value retrieved from SHCreatePropertySet
'	cpspec,	//Count of properties being read
'	rgpspec,	//Array of the properties to be read
'	rgvar	//Array of PROPVARIANTs containing the property values on return
declare import, SHPropStgReadMultiple(comref pps, UINT uCodePage, ULONG cpspec,pointer rgpspec,pointer rgvar),HRESULT

'	SHPropStgWriteMultiple()
'	IPropertyStorage::WriteMultiple wrap
'	The wrap ensures ANSI/UNICODE translations are handled properly for
'	legacy property sets.
'	pps,	' address of IPropertyStorage vtable.
'	uCodePage, // code page retrieved from SHCreatePropertySet.
'	cpspec,	' The number of properties being set
'	rgpspec,	' Property specifiers
'	rgvar,	' Array of PROPVARIANT values
'	propidNameFirst // Minimum value for property identifiers. This value should be >= PID_FIRST_USABLE
declare import, SHPropStgWriteMultiple(comref pps, pointer puCodePage, ULONG cpspec,pointer rgpspec,pointer rgvar, PROPID propidNameFirst),HRESULT

declare import, SHCreateFileExtractIconA(LPCSTR pszFile, DWORD dwFileAttributes,pointer riid,pointer ppv),HRESULT
declare import, SHCreateFileExtractIconW(LPCWSTR pszFile, DWORD dwFileAttributes,pointer riid,pointer ppv),HRESULT
$ifdef UNICODE
declare import, SHCreateFileExtractIcon alias SHCreateFileExtractIconW(LPCWSTR pszFile, DWORD dwFileAttributes,pointer riid,pointer ppv),HRESULT
$else
declare import, SHCreateFileExtractIcon alias SHCreateFileExtractIconA(LPCSTR pszFile, DWORD dwFileAttributes,pointer riid,pointer ppv),HRESULT
$endif
'$include <pshpack8.h>
declare import, SHLimitInputEdit(HWND hwndEdit,comref psf),HRESULT

' The SHMultiFileProperties API displays a property sheet for a
' set of files specified an IDList Array.
' Parameters:
'	pdtobj	- Data object containing list of files.	The data
'	object must provide the "Shell IDList Array"
'	clipboard format.	The parent folder's implementation of
'	IShellFolder::GetDisplayNameOf must return a fully-qualified
'	filesystem path for each item response to the
'	SHGDN_FORPARSING flag.
'	dwFlags - Reserved for future use.	Should be set to 0.
' Returns:
'	S_OK
'
declare import, SHMultiFileProperties(comref pdtobj, DWORD dwFlags),HRESULT

$endif

'typedef void(pointer PFNASYNCICONTASKBALLBACK)(pointer pidl, LPVOID pvData, LPVOID pvHint, INT iIconIndex, INT iOpenIconIndex)
typedef PFNASYNCICONTASKBALLBACK int
' HRESULT SHMapIDListToImageListIndexAsync(pointer pts,pointer psf,pointer pidl, UINT flags,
'	PFNASYNCICONTASKBALLBACK pfn, LPVOID pvData, LPVOID pvHint,pointer piIndex,pointer piIndexSel)
' A usefull function for asynchronously mapping idlist into index into system
' image list.	Optionally it can also look up the index of the selected icon.
' pts	Task scheduler interface to use to create the background task
' psf	Shell folder relating to the pidl
' pidl	Item whose icon is requested
' flags	GIL_ flags
' pfn	Function called back when the background task is done
' pvData	User data passed back the(*pfn)callback
' pvHint	User data passed back the(*pfn)callback
' piIndex	Icon index returned. This is the temporary index if the function returns E_PENDING. The final index will be provided thru the callback
' piIndexSel	Optional icon index for the open icon case(GIL_OPENICON).
' Returns S_OK if all the requested info was available. E_PENDING means that you get temporary icons, and will be called back
'	asynchronously with the final icons. Other failure code means the function failed.
declare import, SHMapIDListToImageListIndexAsync(comref pts,comref psf,pointer pidl, UINT flags,PFNASYNCICONTASKBALLBACK pfn,pointer pvData,pointer pvHint,pointer piIndex,pointer piIndexSel)

' A usefull function Defview for mapping idlist into index into system
' image list.	Optionally it can also look up the index of the selected
' icon.
declare import, SHMapPIDLToSystemImageListIndex(comref pshf,pointer pidl,pointer piIndexSel),int

declare import, SHCLSIDFromString(PCWSTR lpsz,pointer pclsid),HRESULT
declare import, SHFlushClipboard(),HRESULT
declare import, SHCreateQueryCancelAutoPlayMoniker(pointer ppmoniker),HRESULT
declare import, SHGetShellStyleHInstance(),HINSTANCE
declare import, PerUserInit()
declare import, SHRunControlPanel(LPCWSTR lpcszCmdLine, HWND hwndMsgParent),BOOL

$ifndef _SHLOBJ_NO_PICKICONDLG
declare import, PickIconDlg(HWND hwnd,LPWSTR pszIconPath, UINT cbIconPath,pointer piIconIndex),int
$endif

type AASHELLMENUFILENAME
	SHORT	cbTotal
	BYTE	rgbReserved[12]
	iwstring szFileName[1] 	' variable length string
endtype

type AASHELLMENUITEM
	pointer	lpReserved1
	int	iReserved
	UINT	uiReserved
	pointer lpName ' name of file
	LPWSTR psz 			' text to use if no file
endtype

declare import, SHGetAttributesFromDataObject(comref pdo, DWORD dwAttributeMask,pointer pdwAttributes,pointer pcItems),HRESULT

'$include <poppack.h>	/* Return to byte packing */

'$include <poppack.h>

declare import, ImportPrivacySettings(LPCWSTR szFilename, pointer pfParsePrivacyPreferences, pointer pfParsePerSiteRules),BOOL
declare import, DoPrivacyDlg(HWND hwndParent,LPOLESTR pszUrl,comref pPrivacyEnum, BOOL fReportAllSites),HRESULT

$endif
$endif

